@@ -36,5 +36,5 @@ let start() =
     cmdline_cfg
 ;;
 
-Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+Netsys_signal.init();
 start();;
