@@ -2,7 +2,7 @@
 
 open Printf
 
-let generate (cgi : Netcgi_types.cgi_activation) =
+let generate (cgi : Netcgi.cgi_activation) =
   (* A Netcgi-based content provider *)
   cgi # set_header
     ~cache:`No_cache
@@ -41,9 +41,12 @@ let on_request notification =
   ( try
       let env = notification # environment in
       let cgi =
-	new Netcgi.std_activation
-	  ~env:(env :> Netcgi_env.cgi_environment)
-	  ~operating_type:Netcgi.buffered_transactional_optype () in
+	Netcgi_common.cgi_with_args 
+	  (new Netcgi_common.cgi)
+	  (env :> Netcgi.cgi_environment)
+	  Netcgi.buffered_transactional_outtype
+	  env#input_channel
+	  (fun _ _ _ -> `Automatic) in
       generate cgi;
     with
 	e ->
@@ -73,24 +76,7 @@ let serve_connection ues fd =
    *)
   printf "Connected\n";
   flush stdout;
-  let config : Nethttpd_engine.http_engine_config =
-    object
-      method config_timeout_next_request = 15.0
-      method config_timeout = 300.0
-      method config_cgi = Netcgi_env.default_config
-      method config_error_response n = "<html>Error " ^ string_of_int n ^ "</html>"
-      method config_log_error _ _ _ _ msg =
-        (printf "Error log: %s\n" msg; flush stdout)
-      method config_max_reqline_length = 256
-      method config_max_header_length = 32768
-      method config_max_trailer_length = 32768
-      method config_limit_pipeline_length = 5
-      method config_limit_pipeline_size = 250000  (* no effect here! *)
-      method config_input_flow_control = false
-      method config_output_flow_control = true
-      method config_announce_server = `Ocamlnet
-    end
-  in
+  let config = Nethttpd_engine.default_http_engine_config in
   Unix.set_nonblock fd;
   let http_engine = 
     new Nethttpd_engine.http_engine ~on_request_header () config fd ues in
@@ -133,6 +119,30 @@ let start() =
   (* Start the main event loop. *)
   Unixqueue.run ues
 ;;
+let conf_debug() =
+  (* Set the environment variable DEBUG to either:
+       - a list of Netlog module names
+       - the keyword "ALL" to output all messages
+       - the keyword "LIST" to output a list of modules
+     By setting DEBUG_WIN32 additional debugging for Win32 is enabled.
+   *)
+  let debug = try Sys.getenv "DEBUG" with Not_found -> "" in
+  if debug = "ALL" then
+    Netlog.Debug.enable_all()
+  else if debug = "LIST" then (
+    List.iter print_endline (Netlog.Debug.names());
+    exit 0
+  )
+  else (
+    let l = Netstring_str.split (Netstring_str.regexp "[ \t\r\n]+") debug in
+    List.iter
+      (fun m -> Netlog.Debug.enable_module m)
+      l
+  );
+  if (try ignore(Sys.getenv "DEBUG_WIN32"); true with Not_found -> false) then
+    Netsys_win32.Debug.debug_c_wrapper true
+;;
 
-Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+Netsys_signal.init();
+conf_debug();
 start();;
