@@ -7,7 +7,7 @@ open Printf
 
 let counter = ref 0
 
-let hit_counter env (cgi : Netcgi_types.cgi_activation) =
+let hit_counter env (cgi : Netcgi.cgi_activation) =
   let cur_counter = !counter in
   incr counter;
 
@@ -21,9 +21,9 @@ let hit_counter env (cgi : Netcgi_types.cgi_activation) =
   cgi # output # output_string "<html><body>\n";
   cgi # output # output_string (sprintf "Counter = %d<br>\n" cur_counter);
   List.iter
-    (fun (name, arg) ->
+    (fun arg ->
        cgi # output # output_string (sprintf "Arg %s = %s<br>\n"
-				   (Netencoding.Html.encode_from_latin1 name)
+				   (Netencoding.Html.encode_from_latin1 arg#name)
 				   (Netencoding.Html.encode_from_latin1 arg#value))
     )
     cgi # arguments;
@@ -76,26 +76,13 @@ let srv =
 
 
 let serve_connection ues fd =
-  let config : http_engine_config =
-    object
-      method config_timeout_next_request = 15.0
-      method config_timeout = 300.0
-      method config_cgi = Netcgi_env.default_config
-      method config_error_response n = "<html>Error " ^ string_of_int n ^ "</html>"
-      method config_log_error _ _ _ _ msg =
-        printf "Error log: %s\n" msg; flush stdout
-      method config_max_reqline_length = 256
-      method config_max_header_length = 32768
-      method config_max_trailer_length = 32768
-      method config_limit_pipeline_length = 5
-      method config_limit_pipeline_size = 250000
-
-      method config_input_flow_control = true
-      method config_output_flow_control = true
-      method config_announce_server = `Ocamlnet
-    end in
-  
-  let pconfig = new Nethttpd_engine.buffering_engine_processing_config in
+  let config =
+    new Nethttpd_engine.modify_http_engine_config
+      ~config_input_flow_control:true
+      ~config_output_flow_control:true
+      Nethttpd_engine.default_http_engine_config in
+  let pconfig = 
+    new Nethttpd_engine.buffering_engine_processing_config in
 
   Unix.set_nonblock fd;
 
@@ -137,6 +124,30 @@ let start () =
   (* Unixqueue.set_debug_mode true; *)
   Unixqueue.run ues
 ;;
+let conf_debug() =
+  (* Set the environment variable DEBUG to either:
+       - a list of Netlog module names
+       - the keyword "ALL" to output all messages
+       - the keyword "LIST" to output a list of modules
+     By setting DEBUG_WIN32 additional debugging for Win32 is enabled.
+   *)
+  let debug = try Sys.getenv "DEBUG" with Not_found -> "" in
+  if debug = "ALL" then
+    Netlog.Debug.enable_all()
+  else if debug = "LIST" then (
+    List.iter print_endline (Netlog.Debug.names());
+    exit 0
+  )
+  else (
+    let l = Netstring_str.split (Netstring_str.regexp "[ \t\r\n]+") debug in
+    List.iter
+      (fun m -> Netlog.Debug.enable_module m)
+      l
+  );
+  if (try ignore(Sys.getenv "DEBUG_WIN32"); true with Not_found -> false) then
+    Netsys_win32.Debug.debug_c_wrapper true
+;;
 
-Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+Netsys_signal.init();
+conf_debug();
 start();;
