@@ -7,7 +7,6 @@
 (* Dynamic page: The "adder", known from cgi                          *)
 (**********************************************************************)
 
-open Netcgi1_compat.Netcgi_types;;
 open Printf;;
 
 let text = Netencoding.Html.encode_from_latin1;;
@@ -100,7 +99,7 @@ let generate_page (cgi : Netcgi.cgi_activation) =
 ;;
 
 
-let process2 (cgi : Netcgi.cgi_activation) =
+let process (cgi : Netcgi.cgi_activation) =
   (* The [try] block catches errors during the page generation. *)
   try
     (* Set the header. The header specifies that the page must not be
@@ -149,11 +148,6 @@ let process2 (cgi : Netcgi.cgi_activation) =
 ;;
 
 
-let process1 (cgi : Netcgi1_compat.Netcgi_types.cgi_activation) =
-  let cgi' = Netcgi1_compat.Netcgi_types.of_compat_activation cgi in
-  process2 cgi'
-
-
 (**********************************************************************)
 (* Create the webserver                                               *)
 (**********************************************************************)
@@ -166,7 +160,23 @@ let start() =
 
   let opt_list' =
     [ "-mt", Arg.Set use_mt,
-      "  Use multi-threading instead of multi-processing"
+      "  Use multi-threading instead of multi-processing";
+
+      "-debug", Arg.String (fun s -> Netlog.Debug.enable_module s),
+      "<module>  Enable debug messages for <module>";
+
+      "-debug-all", Arg.Unit (fun () -> Netlog.Debug.enable_all()),
+      "  Enable all debug messages";
+
+      "-debug-list", Arg.Unit (fun () -> 
+                                 List.iter print_endline (Netlog.Debug.names());
+                                 exit 0),
+      "  Show possible modules for -debug, then exit";
+
+      "-debug-win32", Arg.Unit (fun () -> 
+                                  Netsys_win32.Debug.debug_c_wrapper true),
+      "  Special debug log of Win32 wrapper"
+
     ] @ opt_list in
 
   Arg.parse 
@@ -179,7 +189,7 @@ let start() =
     else
       Netplex_mp.mp() in  (* multi-processing *)
   let adder =
-    { Nethttpd_services.dyn_handler = (fun _ -> process1);
+    { Nethttpd_services.dyn_handler = (fun _ -> process);
       dyn_activation = Nethttpd_services.std_activation `Std_activation_buffered;
       dyn_uri = None;                 (* not needed *)
       dyn_translator = (fun _ -> ""); (* not needed *)
@@ -197,5 +207,5 @@ let start() =
     cmdline_cfg
 ;;
 
-Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+Netsys_signal.init();
 start();;
