@@ -13,7 +13,7 @@
 
 open Printf
 
-let error_service_connection (cgi : Netcgi_types.cgi_activation) host port finish err =
+let error_service_connection (cgi : Netcgi.cgi_activation) host port finish err =
   (* This is called when the connection with the proxied service could not be
    * established.
    *)
@@ -29,7 +29,7 @@ let error_service_connection (cgi : Netcgi_types.cgi_activation) host port finis
 ;;
 
 
-let on_service_connection env (cgi : Netcgi_types.cgi_activation) host port finish ues conn =
+let on_service_connection env (cgi : Netcgi.cgi_activation) host port finish ues conn =
   (* The connection with the proxied service is established. Now just copy all data
    * from the descriptor to the asynchronous output channel in [env].
    *)
@@ -60,7 +60,7 @@ let on_service_connection env (cgi : Netcgi_types.cgi_activation) host port fini
   )
 ;;
 
-let error_file_service (cgi : Netcgi_types.cgi_activation) filename finish err =
+let error_file_service (cgi : Netcgi.cgi_activation) filename finish err =
   printf "Error opening file\n";
   flush stdout;
   cgi # set_header
@@ -72,7 +72,7 @@ let error_file_service (cgi : Netcgi_types.cgi_activation) filename finish err =
   finish()
 ;;
 
-let setup_file_service env (cgi : Netcgi_types.cgi_activation) filename finish ues =
+let setup_file_service env (cgi : Netcgi.cgi_activation) filename finish ues =
   try
     let fd = Unix.openfile filename [Unix.O_RDONLY] 0 in
     Unix.set_nonblock fd;
@@ -115,9 +115,12 @@ let on_request ues notification =
        * are going to use the latter for our gateway.
        *)
       let cgi =
-	new Netcgi.std_activation
-	  ~env:(env :> Netcgi_env.cgi_environment)
-	  ~operating_type:(`Direct "") () in
+	Netcgi_common.cgi_with_args 
+	  (new Netcgi_common.cgi)
+	  (env :> Netcgi.cgi_environment)
+	  (`Direct "")
+	  env#input_channel
+	  (fun _ _ _ -> `Automatic) in
       let file = try Some( (cgi # argument "file")#value ) with Not_found -> None in
       let host = cgi # argument_value ~default:"127.0.0.1" "host" in
       let port = int_of_string (cgi # argument_value ~default:"37" "port") in
@@ -168,24 +171,7 @@ let serve_connection ues fd =
    *)
   printf "Connected\n";
   flush stdout;
-  let config : Nethttpd_engine.http_engine_config =
-    object
-      method config_timeout_next_request = 15.0
-      method config_timeout = 300.0
-      method config_cgi = Netcgi_env.default_config
-      method config_error_response n = "<html>Error " ^ string_of_int n ^ "</html>"
-      method config_log_error _ _ _ _ msg =
-        (printf "Error log: %s\n" msg; flush stdout)
-      method config_max_reqline_length = 256
-      method config_max_header_length = 32768
-      method config_max_trailer_length = 32768
-      method config_limit_pipeline_length = 5
-      method config_limit_pipeline_size = 250000  (* no effect here! *)
-      method config_input_flow_control = false
-      method config_output_flow_control = true
-      method config_announce_server = `Ocamlnet
-    end
-  in
+  let config = Nethttpd_engine.default_http_engine_config in
   Unix.set_nonblock fd;
   let http_engine = 
     new Nethttpd_engine.http_engine ~on_request_header:(on_request_header ues) () 
@@ -230,5 +216,31 @@ let start() =
   Unixqueue.run ues
 ;;
 
-Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+let conf_debug() =
+  (* Set the environment variable DEBUG to either:
+       - a list of Netlog module names
+       - the keyword "ALL" to output all messages
+       - the keyword "LIST" to output a list of modules
+     By setting DEBUG_WIN32 additional debugging for Win32 is enabled.
+   *)
+  let debug = try Sys.getenv "DEBUG" with Not_found -> "" in
+  if debug = "ALL" then
+    Netlog.Debug.enable_all()
+  else if debug = "LIST" then (
+    List.iter print_endline (Netlog.Debug.names());
+    exit 0
+  )
+  else (
+    let l = Netstring_str.split (Netstring_str.regexp "[ \t\r\n]+") debug in
+    List.iter
+      (fun m -> Netlog.Debug.enable_module m)
+      l
+  );
+  if (try ignore(Sys.getenv "DEBUG_WIN32"); true with Not_found -> false) then
+    Netsys_win32.Debug.debug_c_wrapper true
+;;
+
+
+Netsys_signal.init();
+conf_debug();
 start();;
