@@ -1,4 +1,4 @@
-(* $Id: ssl_exts.mli 1168 2008-03-04 20:20:20Z gerd $ *)
+(* $Id: ssl_exts.mli 1745 2012-03-01 17:31:29Z gerd $ *)
 
 (** A few extensions to the ocaml-ssl library [Ssl] *)
 
@@ -43,4 +43,6 @@ val get_mode : Ssl.socket -> ssl_mode
 val set_mode : Ssl.socket -> ssl_mode -> unit
   (** Sets additional mode bits. It is not possible to clear bits. *)
 
+val certificate_fingerprint : Ssl.certificate -> string
+  (** Returns the fingerprint in the cert in ':'-separated hex notation *)
 
