@@ -1,5 +1,5 @@
 (* 
- * $Id: unixqueue.mli 1616 2011-06-10 15:08:57Z gerd $
+ * $Id: unixqueue.mli 1696 2012-02-08 19:27:53Z gerd $
  *)
 
 open Unix;;
@@ -247,6 +247,13 @@ val create_unix_event_system : unit -> event_system
       backward compatibility.
    *)
 
+class performance_event_system : unit -> event_system
+  (** The implementation using {!Netsys_pollset_generic.performance_pollset}.
+   *)
+
+val performance_event_system : unit -> event_system
+  (** Same as function *)
+
 
 (** {1 Using event systems} *)
 
