@@ -1,5 +1,5 @@
 (* 
- * $Id: uq_socks5.ml 459 2006-04-30 19:49:19Z gerd $
+ * $Id: uq_socks5.ml 1662 2011-08-29 23:05:06Z gerd $
  *)
 
 open Uq_engines;;
@@ -10,11 +10,8 @@ let socks_error s =
   raise (Socks_error s);;
 
 let addr_of_name name =
-  try Unix.inet_addr_of_string name
-  with
-      Failure _ ->
-	let entry = Unix.gethostbyname name in (* may fail *)
-	entry.Unix.h_addr_list.(0)
+  let entry = Uq_resolver.get_host_by_name name in
+  entry.Unix.h_addr_list.(0)
 ;;
 
 class [ 't ] engine_mixin (init_state : 't engine_state) =
@@ -176,9 +173,13 @@ end
 let create_socks_request rq_type sockspec =
   let dst_addr_len = 
     match sockspec with
-	`Sock_inet(_, _, port) ->  (* IP version 4 *)
+	`Sock_inet(_, ip, port) ->
 	  if port < 0 || port > 65535 then socks_error "Illegal port";
-	  4
+	  ( match Netsys.domain_of_inet_addr ip with
+	      | Unix.PF_INET -> 4
+	      | Unix.PF_INET6 -> 16
+	      | _ -> assert false
+	  )
       | `Sock_inet_byname(_, name, port) ->
 	  if port < 0 || port > 65535 then socks_error "Illegal port";
 	  let l = String.length name in
@@ -199,15 +200,21 @@ let create_socks_request rq_type sockspec =
   rq.[2] <- '\000';
   ( match sockspec with
 	`Sock_inet(_, addr, port) ->
-	  rq.[3] <- '\001';
-	  let s = Unix.string_of_inet_addr addr in
-	  Scanf.sscanf s "%d.%d.%d.%d" (fun a3 a2 a1 a0 -> 
-					  rq.[4] <- Char.chr a3;
-					  rq.[5] <- Char.chr a2;
-					  rq.[6] <- Char.chr a1;
-					  rq.[7] <- Char.chr a0);
-	  rq.[8] <- Char.chr (port lsr 8);
-	  rq.[9] <- Char.chr (port land 0xff);
+	  ( match Netsys.domain_of_inet_addr addr with
+	      | Unix.PF_INET ->
+		  rq.[3] <- '\001';
+		  let p = Netsys.protostring_of_inet_addr addr in
+		  String.blit p 0 rq 4 4;
+		  rq.[8] <- Char.chr (port lsr 8);
+		  rq.[9] <- Char.chr (port land 0xff);
+	      | Unix.PF_INET6 ->
+		  rq.[3] <- '\004';
+		  let p = Netsys.protostring_of_inet_addr addr in
+		  String.blit p 0 rq 4 16;
+		  rq.[20] <- Char.chr (port lsr 8);
+		  rq.[21] <- Char.chr (port land 0xff);
+	      | _ -> assert false
+	  )
        | `Sock_inet_byname(_, name, port) ->
 	  rq.[3] <- '\003';
 	  let l = String.length name in
@@ -225,9 +232,13 @@ let create_socks_request rq_type sockspec =
 let fill_udp_header buf sockspec =
   let dst_addr_len = 
     match sockspec with
-	`Sock_inet(_, _, port) ->  (* IP version 4 *)
+	`Sock_inet(_, ip, port) ->
 	  if port < 0 || port > 65535 then socks_error "Illegal port";
-	  4
+	  ( match Netsys.domain_of_inet_addr ip with
+	      | Unix.PF_INET -> 4
+	      | Unix.PF_INET6 -> 16
+	      | _ -> assert false
+	  )
       | `Sock_inet_byname(_, name, port) ->
 	  if port < 0 || port > 65535 then socks_error "Illegal port";
 	  let l = String.length name in
@@ -243,15 +254,21 @@ let fill_udp_header buf sockspec =
   buf.[2] <- '\000';   (* no fragmentation *)
   ( match sockspec with
 	`Sock_inet(_, addr, port) ->
-	  buf.[3] <- '\001';
-	  let s = Unix.string_of_inet_addr addr in
-	  Scanf.sscanf s "%d.%d.%d.%d" (fun a3 a2 a1 a0 -> 
-					  buf.[4] <- Char.chr a3;
-					  buf.[5] <- Char.chr a2;
-					  buf.[6] <- Char.chr a1;
-					  buf.[7] <- Char.chr a0);
-	  buf.[8] <- Char.chr (port lsr 8);
-	  buf.[9] <- Char.chr (port land 0xff);
+	  ( match Netsys.domain_of_inet_addr addr with
+	      | Unix.PF_INET ->
+		  buf.[3] <- '\001';
+		  let p = Netsys.protostring_of_inet_addr addr in
+		  String.blit p 0 buf 4 4;
+		  buf.[8] <- Char.chr (port lsr 8);
+		  buf.[9] <- Char.chr (port land 0xff);
+	      | Unix.PF_INET6 ->
+		  buf.[3] <- '\004';
+		  let p = Netsys.protostring_of_inet_addr addr in
+		  String.blit p 0 buf 4 16;
+		  buf.[20] <- Char.chr (port lsr 8);
+		  buf.[21] <- Char.chr (port land 0xff);
+	      | _ -> assert false
+	  )
        | `Sock_inet_byname(_, name, port) ->
 	  buf.[3] <- '\003';
 	  let l = String.length name in
@@ -266,15 +283,6 @@ let fill_udp_header buf sockspec =
 ;;
 
 
-let inet_addr_of_ipv4 msg =
-  let n3 = Char.code msg.[0] in
-  let n2 = Char.code msg.[1] in
-  let n1 = Char.code msg.[2] in
-  let n0 = Char.code msg.[3] in
-  Unix.inet_addr_of_string(Printf.sprintf "%d.%d.%d.%d" n3 n2 n1 n0)
-;;
-
-
 exception Fragmented
 
 let read_udp_header buf len =
@@ -287,11 +295,19 @@ let read_udp_header buf len =
       '\001' ->
 	(* IP version 4 address *)
 	if len < 10 then socks_error "Protocol error";
-	let addr = inet_addr_of_ipv4 (String.sub buf 4 4) in
+	let addr = Netsys.inet_addr_of_protostring (String.sub buf 4 4) in
 	let p1 = Char.code buf.[8] in
 	let p0 = Char.code buf.[9] in
 	let port = (p1 lsl 8) lor p0 in
 	(10, `Sock_inet(Unix.SOCK_DGRAM, addr, port))
+    | '\004' ->
+	(* IP version 6 address *)
+	if len < 22 then socks_error "Protocol error";
+	let addr = Netsys.inet_addr_of_protostring (String.sub buf 4 16) in
+	let p1 = Char.code buf.[20] in
+	let p0 = Char.code buf.[21] in
+	let port = (p1 lsl 8) lor p0 in
+	(10, `Sock_inet(Unix.SOCK_DGRAM, addr, port))
     | '\003' ->
 	(* Domainname *)
 	let namelen = Char.code buf.[4] in
@@ -301,9 +317,6 @@ let read_udp_header buf len =
 	let p0 = Char.code buf.[6+namelen] in
 	let port = (p1 lsl 8) lor p0 in
 	(7+namelen, `Sock_inet_byname(Unix.SOCK_DGRAM, name, port))
-    | '\004' ->
-	(* IP version 6 address *)
-	socks_error "IP version 6 not supported"
     | _ ->
 	socks_error "Protocol error";
 ;;
@@ -539,15 +552,25 @@ let receive_socks_reply socks_in stype followup =
 	  socks_in # expect 1;  (* length of domainname *)
 	  Trans got_reply_domainname_state
       | '\003' ->  (* IPV6 address *)
-	  socks_error "IP version 6 not supported";
+	  socks_in # expect 16;
+	  Trans got_reply_ipv6_state
       | _ ->
 	  socks_error "Protocol error (2)";
 
   and got_reply_ipv4_state() =
-    (* Interpret incoming message: Four bytes forming an IP address *)
+    (* Interpret incoming message: Four bytes forming an IPv4 address *)
     let msg = socks_in # message in
     assert(String.length msg = 4);
-    let addr = inet_addr_of_ipv4 msg in
+    let addr = Netsys.inet_addr_of_protostring msg in
+    let bnd_spec = `Sock_inet(stype, addr, 0) in
+    socks_in # expect 2;
+    Trans (got_reply_port_state bnd_spec)
+
+  and got_reply_ipv6_state() =
+    (* Interpret incoming message: 16 bytes forming an IPv6 address *)
+    let msg = socks_in # message in
+    assert(String.length msg = 16);
+    let addr = Netsys.inet_addr_of_protostring msg in
     let bnd_spec = `Sock_inet(stype, addr, 0) in
     socks_in # expect 2;
     Trans (got_reply_port_state bnd_spec)
@@ -665,10 +688,10 @@ object(self)
     new seq_engine
       proxy_conn_eng
       (fun conn_stat ->
-	 let socks_socket = client_socket conn_stat in
+	 let socks_socket = client_endpoint conn_stat in
 	 let udp_socket = Unix.socket Unix.PF_INET Unix.SOCK_DGRAM 0 in
 	 Unix.set_nonblock udp_socket;
-	 Unix.set_close_on_exec udp_socket;
+	 Netsys.set_close_on_exec udp_socket;
 	 Unix.bind udp_socket (Unix.ADDR_INET(Unix.inet_addr_any,0));
 	 let eng =
 	   new socks5_datagram_automaton socks_socket udp_socket ues in
@@ -737,7 +760,7 @@ object(self)
 	  new seq_engine
 	    proxy_conn_eng
 	    (fun conn_stat ->
-	       let socks_socket = client_socket conn_stat in
+	       let socks_socket = client_endpoint conn_stat in
 	       let eng =
 		 new socks5_connect_automaton socks_socket sockspec ues in
 	       when_state
@@ -756,7 +779,7 @@ end
 
 class socks5_accept_automaton notify socks_socket ues =
 object(self)
-  inherit [Unix.file_descr * sockspec] automaton socks_socket ues
+  inherit [Unix.file_descr * inetspec option] automaton socks_socket ues
 
   initializer
     let socks_in = Lazy.force socks_in in
@@ -768,7 +791,7 @@ object(self)
 
     and got_accept_reply client_spec =
       notify socks_socket;
-      Finish(socks_socket, client_spec)
+      Finish(socks_socket, Some client_spec)
 
     in
     socks_state <- continue();
@@ -790,7 +813,8 @@ object(self)
 
   val mutable responsible_for_socks_socket = true
 
-  method server_address = actual_bind_spec
+  method server_address = `Socket(actual_bind_spec,
+				  Uq_engines.default_connect_options)
 
   method multiple_connections = false
 
@@ -872,7 +896,7 @@ object(self)
 	  new seq_engine
 	    proxy_conn_eng
 	    (fun conn_stat ->
-	       let socks_socket = client_socket conn_stat in
+	       let socks_socket = client_endpoint conn_stat in
 	       let eng = new socks5_listen_automaton socks_socket sockspec ues
 	       in
 	       when_state
