@@ -245,9 +245,16 @@ object(self)
     with _ -> raise(Sys_error "Netcgi_apache#out_channel#flush: EOF")
 
   method close_out () = (* Apache closes the channel itself *)
-    if closed then raise Netchannels.Closed_channel;
-    self#flush();
-    closed <- true
+    if not closed then (
+      closed <- true;
+      try
+	self#flush();
+      with
+	| error ->
+	    Netlog.logf `Err
+	      "Netcgi_apache: Suppressed error in close_out: %s"
+	      (Netexn.to_string error);
+    )
 
   (* raw_out_channel *)
   method pos_out = sent
@@ -543,7 +550,7 @@ let load_file filename =
 		   filename ^ ": Dynlink.Error: " ^ Dynlink.error_message e,
 		   "Couldn't load the script."))
    | e -> raise(Error(`Internal_server_error,
-		      filename ^ ": " ^ Printexc.to_string e,
+		      filename ^ ": " ^ Netexn.to_string e,
 		      "Couldn't load the script.")));
   (* If the file was loaded and thus the toplevel code executed,
      [current_script] should now be [Some handler]. *)
