@@ -1,4 +1,4 @@
-(* $Id: telnet_client.ml 1121 2007-05-06 18:20:37Z gerd $
+(* $Id: telnet_client.ml 1614 2011-06-09 15:08:56Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -17,8 +17,31 @@
  *   more complicated.
  *)
 
+module Debug = struct
+  let enable = ref false
+end
+
+let dlog = Netlog.Debug.mk_dlog "Ftp_client" Debug.enable
+let dlogr = Netlog.Debug.mk_dlogr "Ftp_client" Debug.enable
+
+let () =
+  Netlog.Debug.register_module "Ftp_client" Debug.enable
+
+
+
 exception Telnet_protocol of exn;;
 
+let () =
+  Netexn.register_printer
+    (Telnet_protocol Not_found)
+    (fun e ->
+       match e with
+	 | Telnet_protocol e' ->
+	     "Telnet_client.Telnet_protocol(" ^ Netexn.to_string e' ^ ")"
+	 | _ ->
+	     assert false
+    )
+
 
 type telnet_command =
     Telnet_data of string
@@ -45,31 +68,31 @@ type telnet_command =
 
 let prerr_command cmd =
   match cmd with
-      Telnet_data s    -> prerr_endline ("Telnet: Data " ^ s)
-    | Telnet_nop       -> prerr_endline "Telnet: NOP";
-    | Telnet_dm        -> prerr_endline "Telnet: DM";
-    | Telnet_brk       -> prerr_endline "Telnet: BRK";
-    | Telnet_ip        -> prerr_endline "Telnet: IP";
-    | Telnet_ao        -> prerr_endline "Telnet: AO";
-    | Telnet_ayt       -> prerr_endline "Telnet: AYT";
-    | Telnet_ec        -> prerr_endline "Telnet: EC";
-    | Telnet_el        -> prerr_endline "Telnet: EL";
-    | Telnet_ga        -> prerr_endline "Telnet: GA";
-    | Telnet_sb c      -> prerr_endline ("Telnet: DB " ^ 
-					 string_of_int(Char.code c));
-    | Telnet_se        -> prerr_endline "Telnet: SE";
-    | Telnet_will c    -> prerr_endline ("Telnet: WILL " ^
-					 string_of_int(Char.code c));
-    | Telnet_wont c    -> prerr_endline ("Telnet: WONT " ^
-					 string_of_int(Char.code c));
-    | Telnet_do c      -> prerr_endline ("Telnet: DO " ^
-					 string_of_int(Char.code c));
-    | Telnet_dont c    -> prerr_endline ("Telnet: DONT " ^
-					 string_of_int(Char.code c));
-    | Telnet_unknown c -> prerr_endline ("Telnet: unknown command " ^
-					 string_of_int(Char.code c));
-    | Telnet_eof       -> prerr_endline "Telnet: <eof>";
-    | Telnet_timeout   -> prerr_endline "Telnet: <timeout>";
+      Telnet_data s    -> dlog ("command: Data " ^ s)
+    | Telnet_nop       -> dlog "command: NOP";
+    | Telnet_dm        -> dlog "command: DM";
+    | Telnet_brk       -> dlog "command: BRK";
+    | Telnet_ip        -> dlog "command: IP";
+    | Telnet_ao        -> dlog "command: AO";
+    | Telnet_ayt       -> dlog "command: AYT";
+    | Telnet_ec        -> dlog "command: EC";
+    | Telnet_el        -> dlog "command: EL";
+    | Telnet_ga        -> dlog "command: GA";
+    | Telnet_sb c      -> dlog ("command: DB " ^ 
+				  string_of_int(Char.code c));
+    | Telnet_se        -> dlog "command: SE";
+    | Telnet_will c    -> dlog ("command: WILL " ^
+				  string_of_int(Char.code c));
+    | Telnet_wont c    -> dlog ("command: WONT " ^
+				  string_of_int(Char.code c));
+    | Telnet_do c      -> dlog ("command: DO " ^
+				  string_of_int(Char.code c));
+    | Telnet_dont c    -> dlog ("command: DONT " ^
+				  string_of_int(Char.code c));
+    | Telnet_unknown c -> dlog ("command: unknown command " ^
+				  string_of_int(Char.code c));
+    | Telnet_eof       -> dlog "command: <eof>";
+    | Telnet_timeout   -> dlog "command: <timeout>";
 ;;
 
 
@@ -82,7 +105,6 @@ type sockstate =
 
 type telnet_options =
     { connection_timeout : float;
-      verbose_connection : bool;
       verbose_input : bool;
       verbose_output : bool;
     }
@@ -203,14 +225,13 @@ class telnet_session =
     val mutable group = None
     val mutable socket = Unix.stdin
     val mutable socket_state = Down
-    val mutable connecting = false
+    val mutable connecting = None
     val mutable polling_wr = false
     val mutable input_timed_out = false
     val mutable output_timed_out = false
 
     val mutable options = 
 	    { connection_timeout = 300.0;
-	      verbose_connection = false;
 	      verbose_input = false;
 	      verbose_output = false;
 	    }
@@ -470,23 +491,24 @@ class telnet_session =
 	g1
 	0.0
 	(fun () -> 
-	   try
-	     self # connect_server;
-	     (* 'group' must not be set earlier, because it is used as
-	      * indicator whether a connection is established or not.
-	      *)
-	     group <- Some g;
-	     let timeout_value = options.connection_timeout in
-	     Unixqueue.add_resource esys g (Unixqueue.Wait_in socket, 
-					    timeout_value);
-	     Unixqueue.add_close_action esys g (socket, 
-						(fun _ -> self # shutdown));
-	     Unixqueue.add_handler esys g (self # handler);
-	     polling_wr <- false;
-	     self # maintain_polling;
-	   with
-	       Unix.Unix_error(_,_,_) as x -> exn_handler x
-	     | Sys_error _ as x -> exn_handler x
+	   self # connect_server
+	     (fun () ->
+		dlog "Telnet connection: Connected!";
+		(* 'group' must not be set earlier, because it is used as
+	         * indicator whether a connection is established or not.
+		 *)
+		group <- Some g;
+		let timeout_value = options.connection_timeout in
+		Unixqueue.add_resource esys g (Unixqueue.Wait_in socket, 
+					       timeout_value);
+		Unixqueue.add_handler esys g (self # handler);
+		polling_wr <- false;
+		self # maintain_polling
+	     )
+	     (fun err ->
+		dlog "Telnet connection: Connection error!";
+		exn_handler err
+	     )
 	)
 
 
@@ -508,76 +530,84 @@ class telnet_session =
       self # update()
 
 
-    method private inet_addr hostname =
-      try
-	(* TODO: 'inet_addr_of_string' may block *)
-	syscall (fun () -> Unix.inet_addr_of_string hostname)
-      with
-	  Failure _ ->
-	    try
-	      let h = syscall (fun () -> Unix.gethostbyname hostname) in
-	      h.Unix.h_addr_list.(0)
-	    with Not_found ->
-	      raise 
-		(Sys_error 
-		   ("Telnet_client: host name lookup failed for " ^ hostname));
+    method expect_input flag =
+      match group with
+	| None ->
+	    failwith "Telnet_client: not attached"
+	| Some g ->
+	    Unixqueue.remove_resource esys g (Unixqueue.Wait_in socket);
+	    let timeout_value =
+	      if flag then 
+		options.connection_timeout
+	      else
+		(-1.0) in
+	    Unixqueue.add_resource esys g (Unixqueue.Wait_in socket, 
+					   timeout_value)
+	    
 
-
-    method private connect_server =
+    method private connect_server f_ok f_err =
 
       begin match connector with
-	  Telnet_connect(hostname, port) ->
-	    if options.verbose_connection then
-	      prerr_endline ("Telnet connection: Connecting to server " ^ 
-			     hostname);
+	| Telnet_connect(hostname, port) ->
+	    dlog ("Telnet connection: Connecting to server " ^ 
+		    hostname);
 
-	    let addr = self # inet_addr hostname in
+	    let g1 = Unixqueue.new_group esys in
 
-	    let dom = Netsys.domain_of_inet_addr addr in
-	    let s = syscall 
-		      (fun () -> Unix.socket dom Unix.SOCK_STREAM 0) in
-	    (* Connect in non-blocking mode: *)
-	    syscall (fun () -> Unix.set_nonblock s);
-	    (* Urgent data is received inline: *)
-	    syscall (fun () -> Unix.setsockopt s Unix.SO_OOBINLINE true);
-	    begin try
-	      syscall (fun () -> Unix.connect s (Unix.ADDR_INET (addr, port)));
-	      connecting <- false;
-	      if options.verbose_connection then
-		prerr_endline "Telnet connection: Connected!";
-	    with
-		Unix.Unix_error(Unix.EINPROGRESS,_,_) ->
-		  (* The 'connect' has not yet been finished. *)
-		  connecting <- true;
-		  (* The 'connect' operation continues in the background.
-		   * It is guaranteed that the socket becomes writeable if
-		   * the connection is established.
-		   * (Of course, it becomes readable if there is already data
-		   * to read, but if the other side does not send us anything
-		   * only writeability is indicated.)
-		   * If the connection fails: This situation is not very well
-		   * described in the manual pages. The "Single Unix Spec"
-		   * says nothing about this case. In the Linux manpages I 
-		   * found that it is possible to read the O_ERROR socket option
-		   * (see connect(2)). By experience I found out that the socket
-		   * indicates readability, and that the following "read" 
-		   * syscall then reports the error correctly.
-		   * The O_ERROR socket option is not supported by O'Caml, so
-		   * the latter is assumed.
-		   *)
-	      | any ->
-		  syscall (fun () -> Unix.close s);
-		  raise any;
-	    end;
+	    let eng =
+	      Uq_engines.connector 
+		(`Socket(`Sock_inet_byname(Unix.SOCK_STREAM,
+					   hostname,
+					   port),
+			 Uq_engines.default_connect_options))
+		esys in
+	    
+	    connecting <- Some eng;
 
-	    socket <- s;
+	    Uq_engines.when_state
+	      ~is_done:(function
+			  | `Socket(s,_) ->
+			      Unixqueue.clear esys g1;
+			      socket <- s;
+			      connecting <- None;
+			      syscall
+				(fun () -> 
+				   Unix.setsockopt s Unix.SO_OOBINLINE true);
+			      Netlog.Debug.track_fd
+				~owner:"Telnet_client"
+				~descr:("connection to " ^ 
+					  hostname ^  ":" ^ string_of_int port)
+				s;
+			      f_ok()
+			  | _ -> assert false
+		       )
+	      ~is_error:(function
+			   | err ->
+			       Unixqueue.clear esys g1;
+			       connecting <- None;
+			       f_err err
+			)
+	      ~is_aborted:(fun () -> 
+			     Unixqueue.clear esys g1;
+			     connecting <- None
+			  )
+	      eng;
+	    let timeout_value = options.connection_timeout in
+	    Unixqueue.once esys g1 timeout_value eng#abort
 
 	| Telnet_socket s ->
-	    connecting <- false;
+	    connecting <- None;
 	    syscall(fun () -> Unix.setsockopt s Unix.SO_OOBINLINE true);
 	    socket <- s;
-	    if options.verbose_connection then
-	      prerr_endline "Telnet connection: Got connected socket";
+	    Netlog.Debug.track_fd
+	      ~owner:"Telnet_client"
+	      ~descr:("connection to " ^ 
+			try Netsys.string_of_sockaddr(Netsys.getpeername s)
+			with _ -> "(noaddr)")
+	      s;
+	    dlog "Telnet connection: Got connected socket";
+	    let g1 = Unixqueue.new_group esys in
+	    Unixqueue.once esys g1 0.0 f_ok
       end;
       
       socket_state <- Up_rw;
@@ -591,13 +621,12 @@ class telnet_session =
 
 
     method private shutdown =
-      if options.verbose_connection then 
-	prerr_endline "Telnet connection: Shutdown!";
+      dlog "Telnet connection: Shutdown!";
       begin match socket_state with
 	  Down -> ()
 	| (Up_rw | Up_r) -> 
-	    if options.verbose_connection then 
-	      prerr_endline "Telnet connection: Closing socket!";
+	    dlog "Telnet connection: Closing socket!";
+	    Netlog.Debug.release_fd socket;
 	    try
 	      syscall (fun () -> Unix.close socket)
 	    with
@@ -612,12 +641,10 @@ class telnet_session =
 
 
     method private abort_connection =
-
-      (* By removing the input and output resources, the event queue is told
-       * that nothing more will be done with the group g, and because of this
-       * the queue invokes the 'close action' (here self # shutdown) and
-       * cleans up the queue.
-       *)
+      ( match connecting with
+	  | None -> ()
+	  | Some eng -> eng#abort()
+      );
       match group with
 	  Some g -> 
 	    Unixqueue.remove_resource esys g (Unixqueue.Wait_in socket);
@@ -625,6 +652,7 @@ class telnet_session =
 	      Unixqueue.remove_resource esys g (Unixqueue.Wait_out socket);
 	      polling_wr <- false;
 	    end;
+	    self # shutdown;
 	    assert (group = None);
         | None -> 
 	    ()
@@ -673,21 +701,6 @@ class telnet_session =
       end;
 
 
-    method private check_connection =
-      (* You need to call this method only if 'connecting' is true, and of
-       * course if the socket is either readable or writeable.
-       * The socket is set to blocking mode, again. The connect time
-       * is measured and recorded.
-       * TODO: find out if a socket error happened in the meantime.
-       *)
-      if connecting then begin
-	syscall(fun () -> Unix.clear_nonblock socket);
-	connecting <- false;
-	if options.verbose_connection then
-	  prerr_endline "Telnet connection: Got connection status";
-      end
-
-
     method private handler  _ _ ev =
       let g = match group with
 	  Some x -> x
@@ -696,7 +709,7 @@ class telnet_session =
 	    raise Equeue.Reject
       in
       match ev with
-	  Unixqueue.Input_arrived (g0,fd0) ->
+	| Unixqueue.Input_arrived (g0,fd0) ->
 	    if g0 = g then 
 	      try self # handle_input with 
 		  Unix.Unix_error(_,_,_) as x -> exn_handler x
@@ -734,8 +747,7 @@ class telnet_session =
 	
 	Queue.add Telnet_timeout input_queue;
 	
-	if options.verbose_connection then 
-	  prerr_endline "Telnet connection: Timeout event!";
+	dlog "Telnet connection: Timeout event!";
 	
 	self # abort_connection;
 	
@@ -763,24 +775,27 @@ class telnet_session =
 
       input_timed_out <- false;
 
-      if options.verbose_connection then 
-	prerr_endline "Telnet connection: Input event!";
+      dlog "Telnet connection: Input event!";
 
       let _g = match group with
 	  Some x -> x
 	| None -> assert false
       in
       
-      if connecting then
-	self # check_connection;
-
       (* Read data into the primary_buffer *)
 
-      let n =
+      let n, eof =
 	syscall
 	  (fun () ->
-	     Unix.read socket primary_buffer 0 (String.length primary_buffer)) in
-      let eof = (n=0) in
+	     try
+	       let n =
+		 Unix.read 
+		   socket primary_buffer 0 (String.length primary_buffer) in
+	       (n, n=0)
+	     with
+	       | Unix.Unix_error(Unix.EAGAIN,_,_) -> 
+		   (0, false)
+	  ) in
 
       Netbuffer.add_sub_string input_buffer primary_buffer 0 n;
 
@@ -902,18 +917,17 @@ class telnet_session =
       in
 
       if eof then begin
-	if options.verbose_connection then
-	  prerr_endline "Telnet connection: Got EOF";
+	dlog "got EOF";
 	Queue.add Telnet_eof input_queue;
 	self # abort_connection;
       end
       else
 	interpret 0;
 
-      if options.verbose_input then begin
-	prerr_endline "Telnet input queue:";
+      if !Debug.enable && options.verbose_input then begin
+	dlog "Telnet input queue:";
 	Queue.iter prerr_command input_queue;
-	prerr_endline "Telnet: <end of queue>";
+	dlog "<end of queue>";
       end;
 
 
@@ -945,17 +959,13 @@ class telnet_session =
 
       output_timed_out <- false;
 
-      if options.verbose_connection then 
-	prerr_endline "Telnet connection: Output event!";
+      dlog "Telnet connection: Output event!";
 
       let _g = match group with
 	  Some x -> x
 	| None -> assert false
       in
 
-      if connecting then
-	self # check_connection;
-
       (* If the write buffer is empty, copy new commands from the write
        * queue to the write buffer.
        *)
@@ -1052,16 +1062,15 @@ class telnet_session =
 
 	if q == synch_queue then begin
 	  sending_urgent_data <- true;
-	  if options.verbose_connection then
-	    prerr_endline "Sending urgent data";
+	  dlog "Sending urgent data";
 	end
 	else
 	  sending_urgent_data <- false;
 
-	if options.verbose_output then begin
-	  prerr_endline "Telnet output queue:";
+	if !Debug.enable && options.verbose_output then begin
+	  dlog "Telnet output queue:";
 	  Queue.iter prerr_command output_queue;
-	  prerr_endline "Telnet: <end of queue>";
+	  dlog "<end of queue>";
 	end;
 
 	try copy() with Queue.Empty -> ()
@@ -1074,13 +1083,17 @@ class telnet_session =
 	let k = 
 	  syscall
 	    (fun () ->
-	       Unix.send socket (Netbuffer.unsafe_buffer output_buffer) 0 l flags) in
+	       try
+		 Unix.send
+		   socket (Netbuffer.unsafe_buffer output_buffer) 0 l flags
+	       with
+		 | Unix.Unix_error(Unix.EAGAIN,_,_) -> 0
+	    ) in
 	Netbuffer.delete output_buffer 0 k;
       end;
 
-      if Netbuffer.length output_buffer = 0 & send_eof then begin
-	if options.verbose_connection then
-	  prerr_endline "Telnet connection: Sending EOF";
+      if Netbuffer.length output_buffer = 0 && send_eof then begin
+	dlog "Telnet connection: Sending EOF";
 	syscall(fun () -> Unix.shutdown socket Unix.SHUTDOWN_SEND);
 	socket_state <- Up_r;
       end;
@@ -1089,4 +1102,5 @@ class telnet_session =
   end
 ;;
 
-
+let () =
+  Netsys_signal.init()
