@@ -1,4 +1,4 @@
-(* $Id: nethttpd_kernel.ml,v 1.2 2005/07/25 22:36:41 stolpmann Exp $
+(* $Id: nethttpd_kernel.ml 1641 2011-07-20 19:42:58Z gerd $
  *
  *)
 
@@ -18,16 +18,29 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WDialog; if not, write to the Free Software
+ * along with Nethttpd; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 
+module Debug = struct
+  let enable = ref false
+end
+
+let dlog = Netlog.Debug.mk_dlog "Nethttpd_kernel" Debug.enable
+let dlogr = Netlog.Debug.mk_dlogr "Nethttpd_kernel" Debug.enable
+
+let () =
+  Netlog.Debug.register_module "Nethttpd_kernel" Debug.enable
+
+
 open Nethttpd_types
 open Nethttp
 open Nethttp.Header
+open Printf
 
 type fatal_error =
     [ `Broken_pipe
+    | `Broken_pipe_ignore
     | `Message_too_long
     | `Timeout
     | `Unix_error of Unix.error
@@ -37,6 +50,7 @@ type fatal_error =
 let string_of_fatal_error =
   function
     | `Broken_pipe -> "Nethttpd: Broken pipe"
+    | `Broken_pipe_ignore -> "Nethttpd: Ignorable broken pipe"
     | `Message_too_long -> "Nethttpd: Message too long, dropping it"
     | `Timeout -> "Nethttpd: Connection timed out"
     | `Unix_error ue -> ("Nethttpd: System error: " ^ Unix.error_message ue)
@@ -107,6 +121,25 @@ type front_token_opt = [ `None | front_token_x ]
 type resp_state =
     [ `Inhibited | `Queued | `Active | `Processed | `Error | `Dropped ]
 
+type announcement =
+    [`Ignore | `Ocamlnet | `Ocamlnet_and of string | `As of string ]
+
+
+let string_of_front_token_x =
+  function
+    | `Resp_wire_data(s,pos,len) ->
+	let n = min len 20 in
+	sprintf 
+	  "Resp_wire_data(%s%s)"
+	  (String.escaped (String.sub s pos n))
+	  (if len > n then "..." else "")
+    | `Resp_end ->
+	"Resp_end"
+    | `Resp_wire_action _ ->
+	"Resp_wire_action"
+
+
+
 class type http_response  =
 object
   method state : resp_state
@@ -120,6 +153,7 @@ object
   method transfer_encoding : transfer_coding
   method front_token : front_token
   method advance : int -> unit
+  method body_size : int64
 end
 
 
@@ -137,8 +171,9 @@ let string_of_state =
  * - make http_repsonse_impl thread-safe
  * - implement trailers
  *)
-class http_response_impl ?(close=false) ?(suppress_body=false) 
-                         (req_version : protocol) : http_response =
+class http_response_impl ?(close=false) ?(suppress_body=false) fdi
+                         (req_version : protocol) 
+                         (ann_server : announcement) : http_response =
   (* - [close]: If true, the connection will be closed after this response
    * - [suppress_body]: If true, the body will not be transmitted (e.g. in response
    *   to a HEAD request)
@@ -166,6 +201,9 @@ object(self)
   method set_state s = 
     let old_state = state in
     state <- s;
+    dlogr (fun () ->
+	     sprintf "FD %Ld resp-%d: response state: %s"
+	       fdi (Oo.id self) (string_of_state s));
     if s <> old_state && (s = `Processed || s = `Error || s = `Dropped) then (
       (* do all actions on the queue *)
       try
@@ -178,7 +216,6 @@ object(self)
 	  Queue.Empty -> ()
     );
     if s <> old_state then callback();
-    (* if s <> old_state then prerr_endline ("CHANGING STATE TO " ^ string_of_state s); *)
 
   method set_callback f = callback <- f
 
@@ -228,7 +265,10 @@ object(self)
 	  (* Set [announced_content_length]: *)
 	  ( try
 	      let len = get_content_length resp_header in  (* or Not_found *)
-	      announced_content_length <- Some len
+	      announced_content_length <- Some len;
+	      dlogr (fun () ->
+		       sprintf "FD %Ld resp-%d: response anncounced_content_length=%Ld"
+			 fdi (Oo.id self) len)
 	    with
 		Not_found -> 
 		  announced_content_length <- None
@@ -239,15 +279,29 @@ object(self)
 		  transfer_encoding <- ( match announced_content_length with
 					   | Some _ -> `Identity
 					   | None -> `Chunked );
+		  dlogr (fun () ->
+			   sprintf "FD %Ld resp-%d: response transfer_encoding=%s"
+			     fdi (Oo.id self)
+			     (match transfer_encoding with
+				| `Identity -> "identity"
+				| `Chunked -> "chunked"
+			     ))
 	      | _ ->
 		  (* Other protocol version: fall back to conservative defaults *)
 		  close_connection <- true;
 		  transfer_encoding <- `Identity;
+		  dlogr (fun () ->
+			   sprintf "FD %Ld resp-%d: response \
+                             transfer_encoding=identity; close_connection=true"
+			     fdi (Oo.id self))
 	  );
 	  (* Update the header: *)
-	  ( match transfer_encoding with
-	      | `Identity -> resp_header # delete_field "Transfer-Encoding"
-	      | `Chunked  -> set_transfer_encoding resp_header ["chunked", []]
+	  ( match transfer_encoding, suppress_body with
+	      | (`Identity, false)
+	      | (_, true) ->
+		  resp_header # delete_field "Transfer-Encoding"
+	      | (`Chunked, false) -> 
+		  set_transfer_encoding resp_header ["chunked", []]
 	  );
 	  resp_header # delete_field "Trailer";
 	  set_date resp_header (Unix.time());
@@ -256,9 +310,17 @@ object(self)
 	      | true  -> set_connection resp_header ["close"]
 	  );
 	  resp_header # delete_field "Upgrade";
-	  let server =
-	    try get_server resp_header with Not_found -> "" in
-	  set_server resp_header (server ^ " Ocamlnet_Nethttpd");
+	  ( match ann_server with
+	      | `Ignore -> ()
+	      | `Ocamlnet ->
+		  let sh = "Ocamlnet/" ^ Netconst.ocamlnet_version in
+		  set_server resp_header sh
+	      | `Ocamlnet_and s ->
+		  let sh = s ^ " Ocamlnet/" ^ Netconst.ocamlnet_version in
+		  set_server resp_header sh
+	      | `As sh ->
+		  set_server resp_header sh
+	  );
 	  (* Convert the header to a data chunk: *)
 	  let b = Netbuffer.create 4096 in
 	  let ch = new Netchannels.output_netbuffer b in
@@ -285,10 +347,12 @@ object(self)
 		  in
 		  if len' > 0 then
 		    Queue.push (`Resp_wire_data (s,pos,len')) queue;
-		  (*
 		  if len > 0 && len' = 0 then
-		    failwith "Nethttpd_kernel.http_response: response larger than announced in header";
-		   *)
+		    dlogr (fun () ->
+			     sprintf "FD %Ld resp-%d: response warning: \
+                                  response is longer than announced" 
+			       fdi (Oo.id self))
+
 	      | `Chunked ->
 		  if len > 0 then (
 		    (* Generate the chunk header: *)
@@ -343,6 +407,8 @@ object(self)
 
   method transfer_encoding = transfer_encoding
 
+  method body_size = real_length
+
   method front_token : front_token =
     if state = `Inhibited then raise Send_queue_empty;
     match front_token with
@@ -350,12 +416,17 @@ object(self)
 	  ( try
 	      let tok = Queue.take queue in
 	      front_token <- (tok :> front_token_opt);
+	      dlogr (fun () ->
+		       sprintf "FD %Ld resp-%d: response new front_token: %s"
+			 fdi (Oo.id self) (string_of_front_token_x tok));
 	      self # front_token
 	    with Queue.Empty ->
 	      raise Send_queue_empty
 	  )
       | `Resp_wire_action f ->
 	  front_token <- `None;
+	  dlogr (fun () -> sprintf "FD %Ld resp-%d: response Resp_wire_action" 
+		   fdi (Oo.id self));
 	  f();
 	  self # front_token
 
@@ -393,11 +464,15 @@ let send_static_response resp status hdr_opt body =
   resp # send (`Resp_status_line(code, text));
   resp # send (`Resp_header h);
   resp # send (`Resp_body(body, 0, String.length body));
-  resp # send `Resp_end;
+  resp # send `Resp_end
 ;;
 
 
 let send_file_response resp status hdr_opt fd length =
+  Netlog.Debug.track_fd
+    ~owner:"Nethttpd_kernel"
+    ~descr:"file response"
+    fd;
   let hdr =
     match hdr_opt with
       | None ->
@@ -435,6 +510,7 @@ let send_file_response resp status hdr_opt fd length =
 	      else (
 		resp # send `Resp_end;
 		fd_open := false;
+		Netlog.Debug.release_fd fd;
 		Unix.close fd;
 	      )
 	    with
@@ -444,8 +520,10 @@ let send_file_response resp status hdr_opt fd length =
 		  feed()
 	  )
       | `Processed | `Error | `Dropped ->
-	  if !fd_open then
+	  if !fd_open then (
+	    Netlog.Debug.release_fd fd;
 	    Unix.close fd;
+	  );
 	  fd_open := false
   in
   resp # send (`Resp_action feed)
@@ -466,6 +544,30 @@ type req_token =
     | `Timeout
     ]
 
+let string_of_req_token =
+  function
+    | `Req_header(((req_method, req_uri), req_proto), hd, resp) ->
+	sprintf "Req_header(%s %s)" req_method req_uri
+    | `Req_expect_100_continue ->
+	"Req_expect_100_continue"
+    | `Req_body(s,pos,len) ->
+	let n = min len 20 in
+	sprintf "Req_body_data(%s%s)"
+	  (String.escaped (String.sub s pos n))
+	  (if len > n then "..." else "")
+    | `Req_trailer tr ->
+	"Req_trailer"
+    | `Req_end ->
+	"Req_end"
+    | `Eof ->
+	"Eof"
+    | `Fatal_error e ->
+	sprintf "Fatal_error(%s)" (string_of_fatal_error e)
+    | `Bad_request_error (e, resp) ->
+	sprintf "Bad_request_error(%s)" (string_of_bad_request_error e)
+    | `Timeout -> 
+	"Timeout"
+
 exception Recv_queue_empty
 
 exception Buffer_exceeded
@@ -474,7 +576,7 @@ exception Buffer_exceeded
    *)
 
 exception Timeout
-  (* Internally used by HTTP implementation: socket blocks too long *)
+  (* Internally used by HTTP implementation: socket blocks for too long *)
 
 exception Fatal_error of fatal_error
   (* Internally used by HTTP implementation: Indicate fatal error *)
@@ -490,77 +592,103 @@ object
   method config_max_trailer_length : int
   method config_limit_pipeline_length : int
   method config_limit_pipeline_size : int
+  method config_announce_server : announcement
+  method config_suppress_broken_pipe : bool
 end
 
 
-let req_line_re = Netstring_pcre.regexp "([^ \r\n]+)[ \t]+([^ \r\n]+)[ \t]+([^ \r\n]+)\r?\n$";;
-  (* Note: \r at end is optional, because the class input_string might be changed
-   * in the future to also accept CRLF as line terminator.
-   *
-   * We don't support HTTP/0.9. One could recognize it easily because the third
-   * word is missing.
+let http_find_line_start s pos len =
+  try Mimestring.find_line_start s pos len
+  with Not_found -> raise Buffer_exceeded
+
+
+let http_find_double_line_start s pos len =
+  try Mimestring.find_double_line_start s pos len
+  with Not_found -> raise Buffer_exceeded
+
+
+let parse_req_line s pos len =
+  (* Parses a request line: "WORD WORD WORD\r\n", where \r is optional.
+     The words are separated by spaces or TABs. Once we did this with
+     a regexp, but this caused stack overflows in the regexp interpreter
+     for long request lines.
+
+     Raises Not_found if not parseable.
+
+     We don't support HTTP/0.9. One could recognize it easily because the third
+     word is missing.
    *)
+  let e = pos+len in
+  let rec next_sep p =
+    if p >= e then raise Not_found else
+      let c = s.[p] in
+      if c = ' ' || c = '\t' then p else
+	if c = '\r' || c = '\n' then raise Not_found
+	else next_sep(p+1) in
+  let rec next_end p =
+    if p >= e then raise Not_found else
+      let c = s.[p] in
+      if c = '\r' || c = '\n' then p 
+      else next_end(p+1) in
+  let rec skip_sep p =
+    if p >= e then raise Not_found else
+      let c = s.[p] in
+      if c = ' ' || c = '\t' 
+      then skip_sep(p+1)
+      else p in
+  let p1 = next_sep pos in
+  let q1 = skip_sep p1 in
+  let p2 = next_sep q1 in
+  let q2 = skip_sep p2 in
+  let p3 = next_end q2 in
+  if s.[p3] = '\n' then (
+    if p3+1 <> e then raise Not_found
+  ) else (
+    if s.[p3] <> '\r' then raise Not_found;
+    if p3+2 <> e then raise Not_found;
+    if s.[p3+1] <> '\n' then raise Not_found;
+  );
+  let w1 = String.sub s pos (p1-pos) in
+  let w2 = String.sub s q1 (p2-q1) in
+  let w3 = String.sub s q2 (p3-q2) in
+  (w1, w2, w3)
 
-let chunk_re = Netstring_pcre.regexp "([0-9a-fA-F]+)[ \t]*(;|\r?\n)";;
+  
+let is_hex =
+  function
+    | '0'..'9' | 'a'..'f' | 'A'..'F' -> true
+    | _ -> false
 
 
-
-let rec skip_empty_lines s pos len =
-  (* Skips over empty lines at [pos]. These lines are terminated by CR/LF or single LF *)
-  if len > 0 then
-    match s.[pos] with
-      | '\010' -> 
-	  skip_empty_lines s (pos+1) (len-1)
-      | '\013' ->
-	  if len > 1 && s.[pos+1] = '\010' then
-	    skip_empty_lines s (pos+2) (len-2)
-	  else
-	    pos
-      | _ ->
-	  pos
-  else
-    pos
-
-
-let rec find_line_end s pos len =
-  (* Returns the position after line terminator. These are CR/LF or single LF.
-   * Raises Buffer_exceeded if there is none.
+let parse_chunk_header s pos len =
+  (* Parses "HEXNUMBER OPTIONAL_SEMI_AND_IGNORED_EXTENSION CRLF",
+     or raises Not_found.
    *)
-  if len > 0 then
-    match s.[pos] with
-      | '\010' ->
-	  pos+1
-      | '\013' ->
-	  if len > 1 && s.[pos+1] = '\010' then
-	    pos+2
-	  else
-	    find_line_end s (pos+1) (len-1)
-      | _ ->
-	  find_line_end s (pos+1) (len-1)
-  else
-    raise Buffer_exceeded
+  let e = pos+len in
+  let rec skip_hex_number p =
+    if p >= e then raise Not_found else
+      let c = s.[p] in
+      if is_hex c then skip_hex_number(p+1)
+      else p in
+  let p1 = skip_hex_number pos in
+  if p1 = pos || p1 >= e then raise Not_found;
+  let c1 = s.[p1] in
+  if c1=';' then (
+    let p2 =
+      Mimestring.find_line_start s p1 (e - p1) in
+    if p2 <> e then raise Not_found
+  )
+  else (
+    if c1 = '\n' then (
+      if p1+1 <> e then raise Not_found
+    ) else (
+      if p1+2 <> e then raise Not_found;
+      if s.[p1+1] <> '\n' then raise Not_found
+    )
+  );
+  String.sub s pos (p1-pos)
 
 
-let rec find_double_line_end s pos len =
-  (* Returns the position after two adjacent line terminators. These are CR/LF or single LF.
-   * Raises Buffer_exceeded if this is not found.
-   *)
-  let pos' = find_line_end s pos len in   (* or Buffer_exceeded *)
-  let len' = len - (pos' - pos) in
-  if len' > 0 then
-    match s.[pos'] with
-      | '\010' ->
-	  pos'+1
-      | '\013' ->
-	  if len' > 1 && s.[pos'+1] = '\010' then
-	    pos'+2
-	  else
-	    find_double_line_end s pos' len'
-      | _ ->
-	  find_double_line_end s pos' len'
-  else
-    raise Buffer_exceeded
-
 type cont =
     [ `Continue of unit -> cont | `Restart | `Restart_with of unit -> cont ]
 
@@ -568,6 +696,8 @@ type cont =
 module StrSet = Set.Make(String)
 
 class http_protocol (config : #http_protocol_config) (fd : Unix.file_descr) =
+  let pa = Netsys_posix.create_poll_array 1 in
+  let fdi = Netsys.int64_of_file_descr fd in
 object(self)
   val mutable resp_queue = Queue.create()
     (* The queue of [http_response] objects. The first is currently being transmitted *)
@@ -608,7 +738,8 @@ object(self)
 
 
   initializer (
-    recv_cont <- self # accept_header 0
+    recv_cont <- self # accept_header 0;
+    Unix.set_nonblock fd
   )
 
   method cycle ?(block=0.0) () = 
@@ -624,13 +755,18 @@ object(self)
 
     with
       | Fatal_error e ->
+	  dlog (sprintf "FD %Ld: fatal error" fdi);
 	  self # abort e;
       | Timeout ->
+	  dlog (sprintf "FD %Ld: timeout" fdi);
 	  self # timeout()
       | Bad_request e ->
 	  (* Stop only the input side of the engine! *)
+	  dlog (sprintf "FD %Ld: bad request" fdi);
 	  self # stop_input_acceptor();
-	  let resp = new http_response_impl ~close:true (`Http((1,0),[])) in
+	  let resp = 
+	    new http_response_impl ~close:true fdi (`Http((1,0),[])) 
+	      config#config_announce_server in
 	  self # push_recv (`Bad_request_error(e, resp), 0);
 	  self # push_recv (`Eof, 0);
 	  resp # set_state `Queued;   (* allow response from now on *)
@@ -640,14 +776,49 @@ object(self)
     (* If d < 0 wait for undefinite time. If d >= 0 wait for a maximum of d seconds.
      * On expiration, raise [Timeout].
      *)
-    let l_in = if self#do_input then [fd] else [] in
-    let l_out = if self#do_output then [fd] else [] in
+    dlogr (fun () -> sprintf "FD %Ld: block %f" fdi d);
+    let f_input = self#do_input in
+    let f_output = self#do_output in
+    if not f_input && not f_output then raise Timeout;
+    Netsys_posix.set_poll_cell pa 0
+      { Netsys_posix.poll_fd = fd;
+	poll_req_events = Netsys_posix.poll_req_events f_input f_output false;
+	poll_act_events = Netsys_posix.poll_null_events()
+      };
     let t = Unix.gettimeofday() in
     try
-      let s_in, s_out, _ = Unix.select l_in l_out [] d in
-      if s_in = [] && s_out = [] then raise Timeout;
+      let n = Netsys_posix.poll pa 1 d in
+      if n = 0 then raise Timeout;
+      (* Check for error: *)
+      let c = Netsys_posix.get_poll_cell pa 0 in
+      let have_error = 
+	Netsys_posix.poll_err_result c.Netsys_posix.poll_act_events in
+      if have_error then (
+	(* Now find out which error. Unfortunately, a simple Unix.read on
+           the socket seems not to work.
+	 *)
+(*
+ -- interpreting POLL_HUP here is difficult. Generally, POLL_HUP can also
+    be set when the other side sends EOF, i.e. a harmless condition
+
+	if Netsys_posix.poll_hup_result c.Netsys_posix.poll_act_events then
+	  raise(Fatal_error `Broken_pipe);
+ *)
+	let code =
+	  Unix.getsockopt_int fd Unix.SO_ERROR in
+	let error =
+	  Netsys.unix_error_of_code code in
+	let exn_arg =
+	  match error with
+	    | Unix.EPIPE 
+	    | Unix.ECONNRESET -> `Broken_pipe
+	    | _ -> `Unix_error error in
+	raise(Fatal_error exn_arg)
+      )
+	
     with
 	Unix.Unix_error(Unix.EINTR,_,_) ->
+	  dlog (sprintf "FD %Ld: block: EINTR" fdi);
 	  if d < 0.0 then
 	    self # block d
 	  else (
@@ -677,7 +848,10 @@ object(self)
 	if recv_eof then (
 	  if need_linger then (
 	    (* Try to linger in the background... *)
-	    let n = Unix.read fd linger_buf 0 (String.length linger_buf) in
+	    dlogr
+	      (fun () ->
+		 sprintf "FD %Ld: lingering for remaining input" fdi);
+	    let n = Unix.recv fd linger_buf 0 (String.length linger_buf) [] in
 	       (* or Unix_error *)
 	    if n=0 then need_linger <- false  (* that's it! *)
 	  );
@@ -685,11 +859,17 @@ object(self)
 	)
 	else (
 	  if self # do_input then (
-	    let n = Netbuffer.add_inplace recv_buf (Unix.read fd) in  (* or Unix_error *)
+	    dlogr (fun () -> sprintf "FD %Ld: recv" fdi);
+	    let n = Netbuffer.add_inplace   (* or Unix_error *)
+	      recv_buf
+	      (fun s pos len -> Unix.recv fd s pos len []) in
 	    if n=0 then (
 	      recv_eof <- true;
 	      need_linger <- false;
-	    );
+	      dlog (sprintf "FD %Ld: got EOF" fdi);
+	    )
+	    else
+	      dlogr (fun () -> sprintf "FD %Ld: got %d bytes" fdi n);
 	    true
 	  )
 	  else
@@ -697,13 +877,17 @@ object(self)
 	)
       with
 	| Unix.Unix_error((Unix.EAGAIN | Unix.EWOULDBLOCK), _,_)->
+	    dlogr (fun () -> sprintf "FD %Ld: recv EWOULDBLOCK" fdi);
 	    false             (* socket not ready *)
 	| Unix.Unix_error(Unix.EINTR,_,_) ->
+	    dlogr (fun () -> sprintf "FD %Ld: recv EINTR" fdi);
 	    false             (* got signal *)
 	| Unix.Unix_error(Unix.ECONNRESET, _,_) ->
+	    dlogr (fun () -> sprintf "FD %Ld: recv ECONNRESET" fdi);
 	    self # abort `Broken_pipe;
 	    false
 	| Unix.Unix_error(e, _, _) ->
+	    dlogr (fun () -> sprintf "FD %Ld: recv ERROR" fdi);
 	    self # abort (`Unix_error e);
 	    false
     in
@@ -759,7 +943,7 @@ object(self)
     waiting_for_next_message <- true;
     let l = Netbuffer.length recv_buf in
     let s = Netbuffer.unsafe_buffer recv_buf in
-    let block_start = skip_empty_lines s pos (l - pos) in  (* (1) *)
+    let block_start = Mimestring.skip_line_ends s pos (l - pos) in  (* (1) *)
     try
       (* (2) *)
       if block_start = l || (block_start+1 = l && s.[block_start] = '\013') then (
@@ -769,7 +953,7 @@ object(self)
       (* (2a) *)
       let reqline_end =
 	try
-	  find_line_end s block_start (l - block_start)
+	  http_find_line_start s block_start (l - block_start)
 	with
 	    Buffer_exceeded ->
 	      IFDEF Testing THEN self # case "accept_header/reqline_ex" ELSE () END;
@@ -781,29 +965,29 @@ object(self)
       if reqline_end-block_start > config#config_max_reqline_length then
 	raise (Bad_request `Request_line_too_long);
       waiting_for_next_message <- false;
-      let line = String.sub s block_start (reqline_end - block_start) in
       let ((meth,uri),req_version) as request_line =
-	match Netstring_pcre.string_match req_line_re line 0 with
-	  | None ->
+	try
+	  let (meth, uri, proto_s) = 
+	    parse_req_line s block_start (reqline_end - block_start) in
+	    (* or Not_found *)
+	  IFDEF Testing THEN self # case "accept_header/4" ELSE () END;
+	  let proto = protocol_of_string proto_s in
+	  ( match proto with
+	      | `Http((1,_),_) -> ()
+	      | _ -> raise (Bad_request `Protocol_not_supported)
+	  );
+	  ((meth, uri), proto)
+	with
+	  | Not_found ->
 	      (* This is a bad request. Response should be "Bad Request" *)
 	      IFDEF Testing THEN self # case "accept_header/3" ELSE () END;
 	      raise (Bad_request `Bad_request_line)
-	  | Some m ->
-	      IFDEF Testing THEN self # case "accept_header/4" ELSE () END;
-	      let meth = Netstring_pcre.matched_group m 1 line in
-	      let uri = Netstring_pcre.matched_group m 2 line in
-	      let proto = protocol_of_string(Netstring_pcre.matched_group m 3 line) in
-	      ( match proto with
-		  | `Http((1,_),_) -> ()
-		  | _ -> raise (Bad_request `Protocol_not_supported)
-	      );
-	      ((meth, uri), proto)
       in
       (* (3) *)
       let config_max_header_length = config # config_max_header_length in
       let block_end =
 	try
-	  find_double_line_end s block_start (l - block_start)
+	  http_find_double_line_start s block_start (l - block_start)
 	with
 	    Buffer_exceeded -> 
 	      IFDEF Testing THEN self # case "accept_header/2" ELSE () END;
@@ -818,7 +1002,7 @@ object(self)
        * buffer.
        *)
       let ch = new Netchannels.input_string 
-		 ~pos:reqline_end ~len:(block_end - block_start) s in
+		 ~pos:reqline_end ~len:(block_end - reqline_end) s in
       let str = new Netstream.input_stream ch in
       (* TODO: This is quite expensive. Create a new netstream class for cheaper access 
        * in this case where we only read from a constant string.
@@ -836,7 +1020,9 @@ object(self)
 	      (try List.mem "close" (get_connection req_h) with Not_found -> false)
 	  | _ -> false in
       let suppress_body = (meth = "HEAD") in
-      let resp = new http_response_impl ~close ~suppress_body (snd request_line) in
+      let resp = 
+	new http_response_impl ~close ~suppress_body fdi (snd request_line)
+	  config#config_announce_server in
       self # push_recv 
 	(`Req_header (request_line, req_h, resp), block_end-block_start);
       Queue.push resp resp_queue;
@@ -971,30 +1157,30 @@ object(self)
     let l = Netbuffer.length recv_buf in
     let s = Netbuffer.unsafe_buffer recv_buf in
     try
-      let p = find_line_end s pos (l - pos) (* or Buffer_exceeded *) in
-      let chunk_header = Netbuffer.sub recv_buf pos (p-pos) in
-      match Netstring_pcre.string_match chunk_re chunk_header 0 with
-	| None ->
-	    IFDEF Testing THEN self # case "accept_body_chunked/invalid_ch" ELSE () END;
-	    raise(Bad_request (`Format_error "Invalid chunk"))
-	| Some m ->
-	    let hex_digits = Netstring_pcre.matched_group m 1 chunk_header in
-	    let chunk_length =
-	      try int_of_string("0x" ^ hex_digits)
-	      with Failure _ -> 
-		IFDEF Testing THEN self # case "accept_body_chunked/ch_large" ELSE () END;
-		raise(Bad_request (`Format_error "Chunk too large")) in
-	    (* Continue with chunk data or chunk end *)
-	    if chunk_length > 0 then (
-	      IFDEF Testing THEN self # case "accept_body_chunked/go_on" ELSE () END;
-	      `Continue(self # accept_body_chunked_contents req_h resp p chunk_length)
-	    )
-	    else (
-	      IFDEF Testing THEN self # case "accept_body_chunked/end" ELSE () END;
-	      `Continue(self # accept_body_chunked_end req_h resp p)
-	    )
+      let p = http_find_line_start s pos (l - pos) (* or Buffer_exceeded *) in
+      ( try
+	  let hex_digits = parse_chunk_header s pos (p-pos) in
+	    (* or Not_found *)
+	  let chunk_length =
+	    try int_of_string("0x" ^ hex_digits)
+	    with Failure _ -> 
+	      IFDEF Testing THEN self # case "accept_body_chunked/ch_large" ELSE () END;
+	      raise(Bad_request (`Format_error "Chunk too large")) in
+	  (* Continue with chunk data or chunk end *)
+	  if chunk_length > 0 then (
+	    IFDEF Testing THEN self # case "accept_body_chunked/go_on" ELSE () END;
+	    `Continue(self # accept_body_chunked_contents req_h resp p chunk_length)
+	  )
+	  else (
+	    IFDEF Testing THEN self # case "accept_body_chunked/end" ELSE () END;
+	    `Continue(self # accept_body_chunked_end req_h resp p)
+	  )
+	with Not_found ->
+	  IFDEF Testing THEN self # case "accept_body_chunked/invalid_ch" ELSE () END;
+	  raise(Bad_request (`Format_error "Invalid chunk"))
+      )
     with
-	Buffer_exceeded -> 
+      | Buffer_exceeded -> 
 	  IFDEF Testing THEN self # case "accept_body_chunked/exceeded" ELSE () END;
 	  if recv_eof then (
 	    IFDEF Testing THEN self # case "accept_body_chunked/eof" ELSE () END;
@@ -1061,7 +1247,7 @@ object(self)
 	    if recv_eof then (
 	      IFDEF Testing THEN self # case "accept_body_chunked_contents/e_eof" ELSE () END;
 	      raise(Bad_request `Unexpected_eof);
-	    )
+	    );
 	    Netbuffer.delete recv_buf 0 pos;
 	    IFDEF Testing THEN self # case "accept_body_chunked_contents/e_restart" ELSE () END;
 
@@ -1097,7 +1283,8 @@ object(self)
 	else (
 	  IFDEF Testing THEN self # case "accept_body_chunked_end/trailer" ELSE () END;
 	  (* Assume there is a trailer. *)
-	  let trailer_end = find_double_line_end s pos (l-pos) in (* or Buf_exceeded *)
+	  let trailer_end = http_find_double_line_start s pos (l-pos) in
+              (* or Buf_exceeded *)
 	  IFDEF Testing THEN self # case "accept_body_chunked_end/tr_found" ELSE () END;
 	  (* Now we are sure there is a trailer! *)
 	  if trailer_end - pos > config_max_trailer_length then (
@@ -1151,11 +1338,15 @@ object(self)
 	match resp # front_token with      (* or Send_queue_empty, Unix_error *)
 	  | `Resp_wire_data (s,pos,len) ->
 	      (* Try to write: *)
-	      let n = Unix.single_write fd s pos len in  (* or Unix.error *)
+	      dlogr (fun () -> sprintf "FD %Ld: send" fdi);
+	      let n = Unix.send fd s pos len [] in  (* or Unix.error *)
+	      dlogr (fun () -> sprintf "FD %Ld: sent %d bytes" fdi n);
 	      (* Successful. Advance by [n] *)
 	      resp # advance n;
 
 	  | `Resp_end ->
+	      dlogr
+		(fun () -> sprintf "FD %Ld: found Resp_end in the queue" fdi);
 	      pipeline_len <- pipeline_len - 1;
 	      resp # set_state `Processed;
 	      (* Check if we have to close the connection: *)
@@ -1234,11 +1425,13 @@ object(self)
      * - Keep receiving data for a while ("lingering"). In principle, until the client has 
      *   seen the half shutdown, but we do not know when. Apache lingers for 30 seconds.
      *)
+    dlogr (fun () -> sprintf "FD %Ld: shutdown" fdi);
     self # stop_input_acceptor();
     self # drop_remaining_responses();
     self # shutdown_sending()
 
   method private shutdown_sending() =
+    dlogr (fun () -> sprintf "FD %Ld: shutdown_sending" fdi);
     if not fd_down then (
       try
 	Unix.shutdown fd Unix.SHUTDOWN_SEND;
@@ -1276,6 +1469,7 @@ object(self)
   method timeout () =
     if waiting_for_next_message then (
       (* Indicate a "soft" timeout. Processing is nevertheless similar to [abort]: *)
+      dlogr (fun () -> sprintf "FD %Ld: soft timeout" fdi);
       need_linger <- false;
       self # shutdown();
       self # push_recv (`Timeout, 0);
@@ -1286,9 +1480,16 @@ object(self)
       
 
   method abort (err : fatal_error) = 
+    dlogr (fun () -> 
+	     sprintf "FD %Ld: abort %s" fdi (string_of_fatal_error err));
     need_linger <- false;
     self # shutdown();
-    self # push_recv (`Fatal_error err, 0);
+    let err' =
+      if err=`Broken_pipe && config#config_suppress_broken_pipe then
+	`Broken_pipe_ignore
+      else
+	err in
+    self # push_recv (`Fatal_error err', 0);
     self # push_recv (`Eof, 0)
 
   method fd = fd
@@ -1312,7 +1513,8 @@ object(self)
 end
 
 
-class lingering_close fd =
+class lingering_close ?(preclose = fun () -> ()) fd =
+  let fd_style = Netsys.get_fd_style fd in
 object(self)
   val start_time = Unix.gettimeofday()
   val timeout = 60.0
@@ -1323,14 +1525,16 @@ object(self)
     try
       if block then (
 	let now = Unix.gettimeofday() in
-	let sel_time = timeout -. (now -. start_time) in
-	let _ = Unix.select [ fd ] [] [] sel_time in
+	let sel_time = max (timeout -. (now -. start_time)) 0.0 in
+	if sel_time = 0.0 then raise Not_found;
+	ignore(Netsys.wait_until_readable fd_style fd sel_time);
 	()
       );
 
-      let n = Unix.read fd junk_buffer 0 (String.length junk_buffer) in
+      let n = Unix.recv fd junk_buffer 0 (String.length junk_buffer) [] in
       if n = 0 then (
 	lingering <- false;
+	preclose();
 	Unix.close fd
       )
     with
@@ -1338,11 +1542,64 @@ object(self)
 	  ()             (* socket not ready *)
       | Unix.Unix_error(Unix.EINTR,_,_) ->
 	  ()             (* got signal *)
-      | Unix.Unix_error(Unix.ECONNRESET, _,_) ->
+      | Unix.Unix_error(_, _,_) 
+      | Not_found ->  (* Any other error means we are done! *)
 	  lingering <- false;
+	  preclose();
 	  Unix.close fd
 
   method lingering = lingering
 
   method fd = fd
 end
+
+
+let default_http_protocol_config =
+  ( object
+      method config_max_reqline_length = 32768
+      method config_max_header_length = 65536
+      method config_max_trailer_length = 32768
+      method config_limit_pipeline_length = 5
+      method config_limit_pipeline_size = 65536
+      method config_announce_server = `Ocamlnet
+      method config_suppress_broken_pipe = false
+    end
+  )
+
+let override v opt =
+  match opt with
+    | None -> v
+    | Some x -> x
+
+class modify_http_protocol_config 
+        ?config_max_reqline_length
+        ?config_max_header_length
+        ?config_max_trailer_length
+        ?config_limit_pipeline_length
+        ?config_limit_pipeline_size
+        ?config_announce_server
+        ?config_suppress_broken_pipe 
+        (config : http_protocol_config) : http_protocol_config =
+  let config_max_reqline_length = 
+    override config#config_max_reqline_length config_max_reqline_length in
+  let config_max_header_length =
+    override config#config_max_header_length config_max_header_length in
+  let config_max_trailer_length =
+    override config#config_max_trailer_length config_max_trailer_length in
+  let config_limit_pipeline_length =
+    override config#config_limit_pipeline_length config_limit_pipeline_length in
+  let config_limit_pipeline_size =
+    override config#config_limit_pipeline_size config_limit_pipeline_size in
+  let config_announce_server =
+    override config#config_announce_server config_announce_server in
+  let config_suppress_broken_pipe =
+    override config#config_suppress_broken_pipe config_suppress_broken_pipe in
+object
+  method config_max_reqline_length = config_max_reqline_length
+  method config_max_header_length = config_max_header_length
+  method config_max_trailer_length = config_max_trailer_length
+  method config_limit_pipeline_length = config_limit_pipeline_length
+  method config_limit_pipeline_size = config_limit_pipeline_size
+  method config_announce_server = config_announce_server
+  method config_suppress_broken_pipe = config_suppress_broken_pipe
+end
