@@ -1,4 +1,4 @@
-(* $Id: nethttpd_services.ml,v 1.2 2005/07/25 22:36:41 stolpmann Exp $
+(* $Id: nethttpd_services.ml 1642 2011-07-20 20:46:25Z gerd $
  *
  *)
 
@@ -22,6 +22,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 
+module Debug = struct
+  let enable = ref false
+end
+
+let dlog = Netlog.Debug.mk_dlog "Nethttpd_services" Debug.enable
+let dlogr = Netlog.Debug.mk_dlogr "Nethttpd_services" Debug.enable
+
+let () =
+  Netlog.Debug.register_module "Nethttpd_services" Debug.enable
+
 open Nethttp
 open Nethttp.Header
 open Nethttpd_types
@@ -164,7 +174,7 @@ object(self)
       let new_env =
 	new redirected_environment 
 	  ~properties:new_properties
-	  ~in_channel:(env # input_ch) env in
+	  ~in_channel:(env # input_channel) env in
       (* Pass control over to the corresponding service: *)
       m_service # process_header new_env
 
@@ -317,7 +327,9 @@ object(self)
 	       * environment.
 	       *)
 	      if req_uri_norm <> req_path then (
-               let req_uri_esc = uripath_encode req_uri_norm in
+		let qs =  env#cgi_query_string in
+		let qm_qs = if qs = "" then "" else "?" ^ qs in
+		let req_uri_esc = uripath_encode req_uri_norm ^ qm_qs in
 		raise(Redirect_request(req_uri_esc, env # input_header)));
 	      (* Search the URI to match: *)
 	      ( match
@@ -419,8 +431,8 @@ end
 
 
 type std_activation_options =
-   { stdactv_processing : (string -> Netmime.mime_header -> Netcgi.argument_processing) option;
-     stdactv_operating_type : Netcgi.operating_type option;
+   { stdactv_processing : Netcgi.arg_store option;
+     stdactv_operating_type : Netcgi.output_type option;
    } 
 
 
@@ -438,7 +450,7 @@ type 'a dynamic_service =
       dyn_uri : string option;
       dyn_translator : string -> string;
       dyn_accept_all_conditionals : bool;
-  } constraint 'a = # Netcgi_types.cgi_activation
+  } constraint 'a = # Netcgi.cgi_activation
 
 let rec strip_prefix ~prefix l =
   match prefix, l with
@@ -456,29 +468,114 @@ let std_activation tag =
   match tag with
     | `Std_activation opts ->
 	(fun env ->
-	   new Netcgi.std_activation 
-	   ~env:(env :> Netcgi_env.cgi_environment)
-	   ?processing:opts.stdactv_processing
-	   ?operating_type:opts.stdactv_operating_type
-	   ())
+	   let out_type =
+	     match opts.stdactv_operating_type with
+	       | None -> `Direct ""
+	       | Some p -> p in
+	   let arg_store =
+	     match opts.stdactv_processing with
+	       | None -> (fun _ _ _ -> `Automatic)
+	       | Some f -> f in
+	   Netcgi_common.cgi_with_args 
+	     (new Netcgi_common.cgi)
+	     (env :> Netcgi.cgi_environment)
+	     out_type
+	     env#input_channel
+	     arg_store
+	)
     | `Std_activation_unbuffered ->
 	(fun env ->
-	   new Netcgi.std_activation 
-	   ~env:(env :> Netcgi_env.cgi_environment)
-	   ~operating_type:(`Direct "")
-	   ())
+	   Netcgi_common.cgi_with_args 
+	     (new Netcgi_common.cgi)
+	     (env :> Netcgi.cgi_environment)
+	     (`Direct "")
+	     env#input_channel
+	     (fun _ _ _ -> `Automatic)
+	)
     | `Std_activation_buffered ->
 	(fun env ->
-	   new Netcgi.std_activation 
-	   ~env:(env :> Netcgi_env.cgi_environment)
-	   ~operating_type:Netcgi.buffered_transactional_optype
-	   ())
+	   Netcgi_common.cgi_with_args 
+	     (new Netcgi_common.cgi)
+	     (env :> Netcgi.cgi_environment)
+	     Netcgi.buffered_transactional_outtype
+	     env#input_channel
+	     (fun _ _ _ -> `Automatic)
+	)
     | `Std_activation_tempfile ->
 	(fun env ->
-	   new Netcgi.std_activation 
-	   ~env:(env :> Netcgi_env.cgi_environment)
-	   ~operating_type:Netcgi.tempfile_transactional_optype
-	   ())
+	   Netcgi_common.cgi_with_args 
+	     (new Netcgi_common.cgi)
+	     (env :> Netcgi.cgi_environment)
+	     Netcgi.tempfile_transactional_outtype
+	     env#input_channel
+	     (fun _ _ _ -> `Automatic)
+	)
+
+
+class dynamic_out_channel enabled out_ch : Netchannels.out_obj_channel =
+  (* if enabled is set to false, output is suppressed *)
+object(self)
+  method output s p len =
+    if !enabled then
+      out_ch # output s p len
+    else
+      len
+  method flush() =
+    if !enabled then
+      out_ch # flush()
+  method close_out() =
+    if !enabled then
+      out_ch # close_out()
+  method pos_out =
+    out_ch # pos_out
+  method really_output s p len =
+    if !enabled then
+      out_ch # really_output s p len
+  method output_char c =
+    if !enabled then
+      out_ch # output_char c
+  method output_string s =
+    if !enabled then
+      out_ch # output_string s
+  method output_byte b =
+    if !enabled then
+      out_ch # output_byte b
+  method output_buffer b =
+    if !enabled then
+      out_ch # output_buffer b
+  method output_channel ?len ch =
+    if !enabled then
+      out_ch # output_channel ?len ch
+end
+
+
+class dynamic_env_wrapper (env:extended_environment) properties =
+  let in_channel = env#input_channel in
+  let out_enabled = ref true in
+  let out_channel = new dynamic_out_channel out_enabled env#output_ch in
+object(self)
+  inherit redirected_environment ~in_channel ~properties env as super
+
+  method send_output_header() =
+    (* Check for CGI-type redirection. In this case we have to suppress 
+       any output. (see also below)
+     *)
+    ( try
+	let loc = self # output_header_field "Location" in (* or Not_found *)
+	if loc = "" || loc.[0] <> '/' then raise Not_found;
+	dlogr (fun () -> 
+		 sprintf "env-%d dynamic_env_wrapper suppressing output"
+		   (Oo.id env));
+	out_enabled := false;  (* suppress output *)
+      with
+	  Not_found ->
+	    super # send_output_header()
+    )
+
+  method output_ch = out_channel
+  method out_channel = out_channel
+end
+
 
 let dynamic_service_impl spec =
 object(self)
@@ -494,14 +591,20 @@ object(self)
     Format.fprintf fmt "@]@ )"
 
   method process_header (env : extended_environment) =
+    dlogr (fun () ->
+	     sprintf "env-%d process_header dynamic_service"
+	       (Oo.id env));
     try
       let req_path_esc = env#cgi_script_name in
-      let req_path = 
+      let _req_path = 
 	try uripath_decode req_path_esc 
 	with Failure _ -> raise Not_found in
 
       let req_method = env # cgi_request_method in
-      let allowed = (env # config).Netcgi_env.permitted_http_methods in
+      let allowed = 
+	List.map 
+	  Netcgi_common.string_of_http_method 
+	  (env # config).Netcgi.permitted_http_methods in
       if not (List.mem req_method allowed) then (
 	let h = new Netmime.basic_mime_header [] in
 	set_allow h allowed;
@@ -515,6 +618,9 @@ object(self)
 	  raise(Standard_response(`Precondition_failed,None,None));
       );
 
+      dlogr (fun () ->
+	       sprintf "env-%d process_header dynamic_service accepts %s %s"
+		 (Oo.id env) req_method req_path_esc);
       (`Accept_body(self :> http_service_receiver) : http_service_reaction)
     with
       | Not_found ->
@@ -522,12 +628,14 @@ object(self)
       | Standard_response(status,hdr_opt,errmsg_opt) ->
 	  `Std_response(status,hdr_opt,errmsg_opt)
 
-  val mutable activation = None
-  val mutable fixed_env = None
+  val response_param = None
 
   method process_body env =
+    dlogr (fun () ->
+	     sprintf "env-%d process_body dynamic_service"
+	       (Oo.id env));
     (* Set PATH_INFO and PATH_TRANSLATED: *)
-    let env =
+    let props =
       match spec.dyn_uri with
 	| Some dyn_uri ->
 	    let req_path_esc = env#cgi_script_name in
@@ -551,35 +659,54 @@ object(self)
 		]
 		env#cgi_properties in
 
-	    new redirected_environment
-	      ~properties
-	      ~in_channel:(env # input_ch)
-	      env
-	    
-	| None -> env
-    in
+	    properties
+	| None -> 
+	    env#cgi_properties in
+    let fenv = new dynamic_env_wrapper env props in
+    let cgi = spec.dyn_activation fenv in
 
-    let cgi = spec.dyn_activation env in
-    activation <- Some cgi;
-    fixed_env <- Some env;
+    dlogr (fun () ->
+	     sprintf "env-%d process_body dynamic_service cgi=%d fixed_env=%d"
+	       (Oo.id env) (Oo.id cgi) (Oo.id fenv));
 
-    (self :> http_service_generator)
+    (* We cannot set here response_param directly because this object
+       is globally used by all incoming requests
+     *)
+    let self' =
+      {< response_param = Some (cgi,fenv) >} in
+    (self' :> http_service_generator)
 
   method generate_response env =
-    match (activation, fixed_env) with
-      | (Some cgi, Some env) ->
-	  spec.dyn_handler env cgi;
-	  (* Check for CGI-type redirection: *)
+
+    dlogr (fun () ->
+	     sprintf "env-%d generate_response dynamic_service"
+	       (Oo.id env));
+
+    match response_param with
+      | Some (cgi,fenv) ->
+	  dlogr (fun () ->
+		   sprintf "env-%d generate_response dynamic_service calling handler with cgi=%d fixed_env=%d"
+		     (Oo.id env) (Oo.id cgi) (Oo.id fenv));
+	  spec.dyn_handler fenv cgi;
+	  dlogr (fun () ->
+		   sprintf "env-%d generate_response dynamic_service \
+                            back from handler"
+		     (Oo.id env));
+
+	  (* Check for CGI-type redirection. In this case we have to do
+             the actual redirection (see also dynamic_env_wrapper)
+	   *)
 	  ( try
-	      let loc = env # output_header_field "Location" in (* or Not_found *)
-	      if loc <> "" && loc.[0] = '/' then (
-		env # output_header # set_fields [];  (* Reset output *)
-		raise(Redirect_response(loc, env # input_header));
-	      )
+	      let loc = 
+		fenv # output_header_field "Location" in (* or Not_found *)
+	      if loc = "" || loc.[0] <> '/' then raise Not_found;
+	      fenv # output_header # set_fields [];  (* Reset output *)
+	      raise(Redirect_response(loc, fenv # input_header))
 	    with
 		Not_found ->
-		  ()  (* no redirection *)
+		  ()
 	  )
+
       | _ ->
 	  failwith "Activation object is missing"
 end
@@ -589,9 +716,11 @@ let dynamic_service spec =
 
 type file_option =
     [ `Enable_gzip
+    | `Enable_cooked_compression
+    | `Override_compression_suffixes of (string * string) list
     | `Enable_index_file of string list
     | `Enable_listings of 	
-	extended_environment -> Netcgi_types.cgi_activation -> file_service -> unit
+	extended_environment -> Netcgi.cgi_activation -> file_service -> unit
     ]
 
 and file_service =
@@ -604,18 +733,27 @@ and file_service =
 
 let file_translator spec uri =
   let rem_slash s =
-    if s<>"" && s.[0] = '/' then
-      String.sub s 1 (String.length s - 1)
-    else
-      s in
+    let s1 =
+      if s<>"" && s.[0] = '/' then
+	String.sub s 1 (String.length s - 1)
+      else
+	s in
+    let s2 =
+      if s1 <> "" && s1.[String.length s1-1] = '/' then
+	String.sub s1 0 (String.length s1-1)
+      else
+	s1 in
+    s2 in
+  let concat p1 p2 =
+    if p2="" then p1 else Filename.concat p1 p2 in
   let rec translate pat_l l =
     match (pat_l, l) with
       | ([], [""]) ->
 	  spec.file_docroot
       | ([], path) ->
-	  Filename.concat spec.file_docroot (rem_slash(Neturl.join_path path))
+	  concat spec.file_docroot (rem_slash(Neturl.join_path path))
       | ([""], path) ->
-	  Filename.concat spec.file_docroot (rem_slash(Neturl.join_path path))
+	  concat spec.file_docroot (rem_slash(Neturl.join_path path))
       | (pat_dir :: pat_l', dir :: l') when pat_dir = dir ->
 	  translate pat_l' l'
       | _ ->
@@ -640,14 +778,14 @@ let file_translator spec uri =
 	let spec_uri_list = Neturl.norm_path (Neturl.split_path spec.file_uri) in
 	translate spec_uri_list uri_list
 
-let ext_re = Netstring_pcre.regexp ".*\\.([^.]+)$";;
+let ext_re = Netstring_str.regexp ".*\\.\\([^.]+\\)$";;
 
 let get_extension s =
-  match Netstring_pcre.string_match ext_re s 0 with
+  match Netstring_str.string_match ext_re s 0 with
     | None ->
 	None
     | Some m ->
-	Some(Netstring_pcre.matched_group m 1 s)
+	Some(Netstring_str.matched_group m 1 s)
 
 let merge_byte_ranges st ranges =
   (* Merge the byte [ranges] into a single range. Returns [Some (first,last)] if
@@ -682,6 +820,49 @@ let merge_byte_ranges st ranges =
   in
   merge ranges
 
+let w32_fix_trailing_slash s =
+  (* background: Win32 dislikes Unix.stat "directory/". It is unclear whether
+     this function is still required - file_translator is now changed so
+     that trailing slashes are normally not returned.
+   *)
+  if Sys.os_type = "Win32" then (
+    if s <> "" && s <> "/" && s.[ String.length s - 1 ] = '/' then
+      s ^ "."
+    else
+      s
+  )
+  else s
+
+
+let get_compression_suffixes file_options =
+  let rec find opts =
+    match opts with
+      | `Override_compression_suffixes l :: _ -> l
+      | _ :: opts' -> find opts'
+      | [] ->
+	  [ "gz", "gzip";
+	    "bz2", "bzip2";
+	    "Z", "compress"
+	  ] in
+  find file_options
+
+
+let rec search_cooked_file filename compression_suffixes =
+  match compression_suffixes with
+    | [] ->
+	[ "identity", filename ]
+    | (suffix, ce) :: suffixes' ->
+	try
+	  let filename_ext = filename ^ "." ^ suffix in
+	  let fd = Unix.openfile filename_ext [ Unix.O_RDONLY] 0 in
+	  (* or Unix_error *)
+	  Unix.close fd;
+	  (ce, filename_ext) :: search_cooked_file filename suffixes'
+	with
+	  | Unix.Unix_error(_,_,_) ->
+	      search_cooked_file filename suffixes'
+
+
 let file_service (spec : file_service) =
 object(self)
   method name = "file_service"
@@ -700,8 +881,12 @@ object(self)
     List.iter
       (function
 	 | `Enable_gzip       -> Format.fprintf fmt "@ enable_gzip"
+	 | `Enable_cooked_compression -> 
+	     Format.fprintf fmt "@ enable_cooked_compression"
 	 | `Enable_index_file _ -> Format.fprintf fmt "@ enable_index_file"
 	 | `Enable_listings _ -> Format.fprintf fmt "@ enable_listings"
+	 | `Override_compression_suffixes _ ->
+	     Format.fprintf fmt "@ override_compression_suffixes"
       )
       spec.file_options;
     Format.fprintf fmt "@]@ )";
@@ -714,7 +899,8 @@ object(self)
 	try uripath_decode req_path_esc with Failure _ -> raise Not_found in
       let filename =
 	file_translator spec req_path in (* or Not_found *)
-      let s = Unix.LargeFile.stat filename in   (* or Unix_error *)
+      let s = Unix.LargeFile.stat (w32_fix_trailing_slash filename) in
+      (* or Unix_error *)
       ( match s.Unix.LargeFile.st_kind with
 	  | Unix.S_REG ->
 	      self # serve_regular_file env filename s
@@ -753,19 +939,37 @@ object(self)
     if req_method <> "GET" && req_method <> "HEAD" then (
       let h = new Netmime.basic_mime_header [] in
       set_allow h [ "GET"; "HEAD"; "OPTIONS" ];
-      raise (Standard_response(`Method_not_allowed,Some h,(Some "Nethttpd: Method not allowed for file"))));
+      raise(Standard_response(`Method_not_allowed,Some h,
+			      (Some "Nethttpd: Method not allowed for file"))));
     (* Set [Accept-ranges] header: *)
     env # set_output_header_field "Accept-ranges" "bytes";
-    (* Figure out file extension and media type *)
+    (* Figure out file extension, content encoding (compression type)
+       and media type
+     *)
+    let media_type_of_ext ext =
+      try List.assoc ext spec.file_suffix_types
+      with Not_found -> spec.file_default_type in
+    let compression_suffixes =
+      get_compression_suffixes spec.file_options in
     let ext_opt = get_extension filename in
-    let media_type =
+    let content_encoding, media_type =
       match ext_opt with
-	| Some ext ->
-	    ( try List.assoc ext spec.file_suffix_types
-	      with Not_found -> spec.file_default_type
+	| Some ext when List.mem_assoc ext compression_suffixes ->
+	    let ce = List.assoc ext compression_suffixes in
+	    let filename1 = Filename.chop_extension filename in
+	    let ext1_opt = get_extension filename1 in
+	    ( match ext1_opt with
+		| None -> (ce, spec.file_default_type)
+		| Some ext1 -> (ce, media_type_of_ext ext1)
 	    )
-	| None -> spec.file_default_type in
+	| Some ext ->
+	    "identity", media_type_of_ext ext
+	| None ->
+	    "identity", spec.file_default_type in
     env # set_output_header_field "Content-type" media_type;
+    if content_encoding <> "identity" then
+      env # set_output_header_field "Content-Encoding" content_encoding;
+
     (* Generate the (weak) validator from the file statistics: *)
     let etag =
       `Weak (sprintf "%d-%Lu-%.0f"
@@ -837,9 +1041,9 @@ object(self)
 	| Not_found -> false, false
 	| Bad_header_field _ -> false, false in
     if (have_if_modified || have_if_none_match) && 
-       not (accept_if_modified || accept_if_none_match) 
+      not (accept_if_modified || accept_if_none_match) 
     then
-	 raise(Standard_response(`Not_modified,None,None));
+      raise(Standard_response(`Not_modified,None,None));
     (* Now the GET request is accepted! *)
     let partial_GET =
       try
@@ -892,26 +1096,25 @@ object(self)
 	| None ->
 	    (* Full GET *)
 	    (* Check whether there is a gzip-encoded complementary file *)
-	    let filename_gz = filename ^ ".gz" in
-	    let support_gzip =
-	      try
-		let fd = Unix.openfile filename_gz [ Unix.O_RDONLY] 0 in
-		(* or Unix_error *)
-		Unix.close fd;
-		true
-	      with
-		| Unix.Unix_error(_,_,_) -> false in
-	    let support_encodings =
-	      (if support_gzip then ["gzip"] else []) @ ["identity"] in
-	    let encoding = best_encoding env#input_header support_encodings in
+	    let encodings_and_files =
+	      if (List.mem `Enable_gzip spec.file_options ||
+		    List.mem `Enable_cooked_compression spec.file_options) 
+	      then
+		search_cooked_file filename compression_suffixes
+	      else
+		[ "identity", filename ] in
+	    let supported_encodings =
+	      List.map fst encodings_and_files in
+	    let encoding = best_encoding env#input_header supported_encodings in
 	    let h = env # output_header in
 	    ( match encoding with
 		| "identity" ->
 		    `File(`Ok, None, filename, 0L, s.Unix.LargeFile.st_size)
-		| "gzip" ->
-		    let st_gzip = Unix.LargeFile.stat filename_gz in
-		    h # update_field "Content-Encoding" "gzip";
-		    `File(`Ok, Some h, filename_gz, 0L, st_gzip.Unix.LargeFile.st_size)
+		| _ ->
+		    let fn = List.assoc encoding encodings_and_files in
+		    let st_gzip = Unix.LargeFile.stat fn in
+		    h # update_field "Content-Encoding" encoding;
+		    `File(`Ok, Some h, fn, 0L, st_gzip.Unix.LargeFile.st_size)
 		| _ -> assert false
 	    )
     )
@@ -943,24 +1146,23 @@ object(self)
       (* If [req_path] does not end with a slash, perform a redirection: *)
       if req_path <> "" && req_path.[ String.length req_path - 1 ] <> '/' then (
 	let h = new Netmime.basic_mime_header
-		  [ "Location", 
-		    sprintf "http://%s%s%s/"
-		      env#cgi_server_name
-		      ( match env#cgi_server_port with 
-			  | Some p -> ":" ^ string_of_int p
-			  | None -> "")
-		      env#cgi_request_uri ] in
+	  [ "Location", 
+	    sprintf "http://%s%s%s/"
+	      env#cgi_server_name
+	      ( match env#cgi_server_port with 
+		  | Some p -> ":" ^ string_of_int p
+		  | None -> "")
+	      env#cgi_request_uri ] in
 	raise(Standard_response(`Found, Some h, None));
       )
     );
-
     match (abs_index_file_opt, gen_listings) with
       | Some name, _ ->
 	  (* Ok, redirect to the file *)
 	  let req_path_esc = Neturl.split_path env#cgi_script_name in
 	  let name_esc = [ uripath_encode name ] in
 	  raise(Redirect_request(Neturl.join_path (req_path_esc @ name_esc), 
-			       env # input_header))
+				 env # input_header))
 
       | None, Some (`Enable_listings generator) ->
 	  (* If OPTIONS: Respond now *)
@@ -984,45 +1186,45 @@ object(self)
 	    } in
 	  let dyn_srv = dynamic_service dyn_spec in
 	  dyn_srv # process_header env
-(*
-	  let (listing, listing_hdr) = generator env spec filename in
-	  (* Generate the (weak) validator from the file statistics: *)
-	  let etag =
-	    `Weak (sprintf "%d-%Lu-%.0f"
-		     s.Unix.LargeFile.st_ino
-		     s.Unix.LargeFile.st_size
-		     s.Unix.LargeFile.st_mtime) in
-	  set_etag listing_hdr etag;
-	  (* Refuse If-match and If-unmodified-since: *)
-	  if env # multiple_input_header_field "If-match" <> [] then
-	    raise(Standard_response(`Precondition_failed, None, None));
-	  if env # multiple_input_header_field "If-unmodified-since" <> [] then
-	    raise(Standard_response(`Precondition_failed, None, None));
-	  (* Return contents: *)
-	  `Static(`Ok, Some listing_hdr, listing)
- *)
+	    (*
+	      let (listing, listing_hdr) = generator env spec filename in
+	    (* Generate the (weak) validator from the file statistics: *)
+	      let etag =
+	      `Weak (sprintf "%d-%Lu-%.0f"
+	      s.Unix.LargeFile.st_ino
+	      s.Unix.LargeFile.st_size
+	      s.Unix.LargeFile.st_mtime) in
+	      set_etag listing_hdr etag;
+	    (* Refuse If-match and If-unmodified-since: *)
+	      if env # multiple_input_header_field "If-match" <> [] then
+	      raise(Standard_response(`Precondition_failed, None, None));
+	      if env # multiple_input_header_field "If-unmodified-since" <> [] then
+	      raise(Standard_response(`Precondition_failed, None, None));
+	    (* Return contents: *)
+	      `Static(`Ok, Some listing_hdr, listing)
+	     *)
 
       | _ ->
 	  (* Listings are forbidden: *)
 	  `Std_response(`Forbidden, None, (Some "Nethttpd: Access to directories not configured") )
-     
+	    
 end
 
 
-let simple_listing ?(hide=[ "\\."; ".*~$" ]) env (cgi :Netcgi_types.cgi_activation) fs =
+let simple_listing ?(hide=[ "\\."; ".*~$" ]) env (cgi :Netcgi.cgi_activation) fs =
   let dirname = env # cgi_path_translated in
   let col_name = 30 in
   let col_mtime = 20 in
   let col_size = 10 in
-  let regexps = List.map (fun re -> Netstring_pcre.regexp re) hide in
+  let regexps = List.map (fun re -> Netstring_str.regexp re) hide in
   let req_path_esc = env#cgi_path_info in
   let req_path = uripath_decode req_path_esc in
-  let files = Sys.readdir dirname in
+  let files = Sys.readdir (w32_fix_trailing_slash dirname) in
   let xfiles =
     Array.map
       (fun name ->
 	 if List.exists 
-	      (fun re -> Netstring_pcre.string_match re name 0 <> None) regexps 
+	      (fun re -> Netstring_str.string_match re name 0 <> None) regexps 
 	 then
 	   `None
 	 else
@@ -1182,7 +1384,6 @@ let simple_listing ?(hide=[ "\\."; ".*~$" ]) env (cgi :Netcgi_types.cgi_activati
   cgi # output # commit_work()
 
 
-(*
 type ac_by_host_rule =
     [ `Allow of string list
     | `Deny of string list
@@ -1190,9 +1391,88 @@ type ac_by_host_rule =
 
 type 'a ac_by_host = ac_by_host_rule * 'a http_service
 
-let ac_by_host spec =
-  XXX
- *)
+let prepare_ac_by_host spec =
+  let resolve host =
+    try
+      [ Unix.inet_addr_of_string host ]
+    with
+      | _ ->
+	  ( try
+	      let h = Uq_resolver.get_host_by_name host in
+	      Array.to_list h.Unix.h_addr_list
+	    with
+	      | Uq_resolver.Host_not_found _ -> []
+	  )
+  in
+
+  match spec with
+    | `Allow hosts ->
+	let ipaddrs = List.flatten (List.map resolve hosts) in
+	`Allow_ip ipaddrs
+    | `Deny hosts ->
+	let ipaddrs = List.flatten (List.map resolve hosts) in
+	`Deny_ip ipaddrs
+
+let ac_by_host (spec, (srv : 'a http_service)) =
+  let spec' = prepare_ac_by_host spec in
+  ( object(self)
+      method name = "ac_by_host"
+      method def_term = `Ac_by_host (spec,srv)
+      method print fmt =
+	Format.fprintf fmt "ac_by_host(...)"
+      method process_header env =
+	let addr = env # remote_socket_addr in
+	let allowed =
+	  match spec' with
+	    | `Allow_ip ipaddrs ->
+		( match addr with
+		    | Unix.ADDR_INET(ia,_) ->
+			List.mem ia ipaddrs
+		    | _ ->
+			true
+		)
+	    | `Deny_ip ipaddrs -> 
+		( match addr with
+		    | Unix.ADDR_INET(ia,_) ->
+			not(List.mem ia ipaddrs)
+		    | _ ->
+			true
+		)
+	in
+	if allowed then
+	  srv # process_header env
+	else
+	  `Std_response(`Forbidden, None, (Some "Nethttpd: Access denied by host rule"))
+    end
+  )
 
 
-(* CHECK: HEAD handling *)
+let ws_re = Netstring_str.regexp "[ \r\t\n]+"
+
+let split_ws s =
+  Netstring_str.split ws_re s
+
+let read_media_types_file fname =
+  let f = open_in fname in
+  let l = ref [] in
+  try
+    while true do
+      let line = input_line f in
+      if line = "" || line.[0] <> '#' then (
+	let words = split_ws line in
+	match words with
+	  | [] -> ()
+	  | [ mtype ] -> ()
+	  | mtype :: suffixes ->
+	      l := (List.map (fun s -> (s,mtype)) (List.rev suffixes)) @ !l
+      )
+    done;
+    assert false
+  with
+    | End_of_file ->
+	close_in f;
+	List.rev !l
+    | error ->
+	close_in f;
+	raise error
+;;
