@@ -1,4 +1,4 @@
-(* $Id: nethttpd_types.mli,v 1.2 2005/07/25 22:36:41 stolpmann Exp $
+(* $Id: nethttpd_types.mli 1410 2010-02-14 19:44:28Z gerd $
  *
  *)
 
@@ -22,7 +22,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 
-(** Type definitions for the HTTP daemon, and an introduction
+(** Type definitions for the HTTP daemon
   *
   * {b Contents}
   *
@@ -30,7 +30,6 @@
   * - {!Nethttpd_types.environment}
   * - {!Nethttpd_types.service}
   * - {!Nethttpd_types.helpers}
-  * - {!Nethttpd_types.overview}
  *)
 
 (** Many types can also be found in the [Nethttp] module (part of netstring).
@@ -52,12 +51,21 @@ exception Standard_response of http_status * http_header option * string option
 
 (** {1:environment Environment} *)
 
+type output_state =
+    [ `Start
+    | `Sending
+    | `End
+    ]
+
+val string_of_output_state : output_state -> string
+  (** Debugging *)
+
 (** An extension of [cgi_environment] for use with the daemon. The methods
-  * retrieve the socket addresses are virtual.
+  * retrieving the socket addresses are virtual.
  *)
 class type virtual v_extended_environment =
 object
-  inherit Netcgi_env.cgi_environment
+  inherit Netcgi.cgi_environment
 
   method virtual server_socket_addr : Unix.sockaddr
   method virtual remote_socket_addr : Unix.sockaddr
@@ -65,6 +73,23 @@ object
       * of the current connection.
      *)
 
+  method cgi_request_uri : string
+    (** The full request URI. Identical to the CGI property "REQUEST_URI" *)
+
+  method log_props : (string * string) list -> unit
+    (** Remember this version of [cgi_properties] as the one sent to the
+        [config_log_access] function
+     *)
+
+  method input_channel : Netchannels.in_obj_channel
+    (** The input channel for reading the body of the request *)
+
+  method input_body_size : int64
+    (** so far known, or 0L *)
+
+  method request_body_rejected : bool
+    (** so far known, or false *)
+
   method send_file : Unix.file_descr -> int64 -> unit
     (** Sends the output header with a file as body. The file must already be open,
      * and positioned where the transmission begins. The number is the length
@@ -79,6 +104,13 @@ object
      * Only one transmission method must be invoked.
      *)
 
+  method virtual output_state : output_state ref
+    (** Reflects the state of the output generation:
+        - [`Start]: Nothing is generated yet
+        - [`Sending]: Output is already being sent
+        - [`End]: The response (for a single request) has been fully sent
+     *)
+
 end
 
 
@@ -90,17 +122,17 @@ object
   inherit v_extended_environment
   method server_socket_addr : Unix.sockaddr
   method remote_socket_addr : Unix.sockaddr
+  method output_state : output_state ref
 end
 
+
 (** {2 Construction of environments} *)
 
 class virtual empty_environment :
 object
   inherit v_extended_environment
-  val mutable config : Netcgi_env.cgi_config
-  val mutable in_state : Netcgi_env.input_state
-  val mutable out_state : Netcgi_env.output_state
-  val mutable protocol : Netcgi_env.protocol
+  val mutable config : Netcgi.config
+  val mutable protocol : Nethttp.protocol
   val mutable in_header : http_header
   val mutable out_header : http_header
   val mutable properties : (string * string) list
@@ -113,19 +145,19 @@ end
     * - [send_output_header]
     * - [send_file]
     * - [log_error]
+    * - [log_props]
     *
     * The virtual methods, of course, must be defined!
    *)
 
 class redirected_environment : 
-        ?in_state : Netcgi_env.input_state ->
         ?in_header : http_header ->
         ?properties : (string * string) list ->
         ?in_channel : Netchannels.in_obj_channel ->
         extended_environment ->
           extended_environment
   (** This class overlays the input-side containers of an existing environment.
-    * The output-side containers ([out_state], [out_header], and [out_channel])
+    * The output-side containers ([out_header], and [out_channel])
     * are physically identical with the existing environment.
     *
     * If one of the argument is not passed on class instantiation, the corresponding
@@ -166,11 +198,69 @@ val output_file_response : #extended_environment ->
     * The function raises [Sys_error] when the file cannot be read.
    *)
 
+(** {1 Generating error responses, logging} *)
+
+class type request_info =
+object
+  method server_socket_addr : Unix.sockaddr
+    (** The socket address of this server. May raise [Not_found] if
+        there is no such address *)
+  method remote_socket_addr : Unix.sockaddr
+    (** The socket address of the client. May raise [Not_found] if
+        there is no such address *)
+  method request_method : string
+    (** The method like [GET]. May raise [Not_found] *)
+  method request_uri : string
+    (** The URI of the client request. This is often without the
+        server designation, i.e. just [/path?query].
+        May raise [Not_found] *)
+  method input_header : Nethttp.http_header
+    (** The request header. May raise [Not_found] *)
+  method cgi_properties : (string * string) list
+    (** The distilled CGI properties *)
+  method input_body_size : int64
+    (** The size of the input body. May raise [Not_found] *)
+end
+
+
+class type full_info =
+object
+  inherit request_info
+  method response_status_code : int
+    (** The HTTP status code to response *)
+  method request_body_rejected : bool
+    (** Whether the request body was rejected *)
+  method output_header : Nethttp.http_header
+    (** The response header *)
+  method output_body_size : int64
+    (** The size of the output body. *)
+end
+
+
+class create_full_info : response_status_code:int ->
+                         request_body_rejected:bool ->
+                         output_header:Nethttp.http_header ->
+                         output_body_size:int64 ->
+                         request_info -> full_info
+  (** Creates a [full_info] object by adding to a [request_info] object *)
+
+
+class type error_response_params =
+object
+  inherit request_info
+
+  method response_status_code : int
+    (** The HTTP status code to response *)
+
+  method error_message : string
+    (** The error message explaining the detail that went wrong *)
+end
+
+
 class type min_config =
 object
-  method config_error_response : int -> string
-  method config_log_error : 
-    Unix.sockaddr option -> Unix.sockaddr option -> http_method option -> http_header option -> string -> unit
+  method config_error_response : error_response_params -> string
+  method config_log_error : request_info -> string -> unit
 end
   (** Minimal configuration needed for [output_std_response] *)
 
@@ -352,174 +442,3 @@ val update_alist : ('a * 'b) list -> ('a * 'b) list -> ('a * 'b) list
   (** [update_alist updl l]: Returns the alist with all elements of [updl]
     * and all elements of [l] that are not member of [updl].
    *)
-
-(**********************************************************************)
-
-(** {1:overview Overview over the HTTP daemon}
-
-This library implements an HTTP 1.1 server. Because it is a library and not
-a stand-alone server like Apache, it can be used in very flexible ways.
-The disadvantage is that the user of the library must do more to get
-a running program than just configuring the daemon.
-
-The daemon has five modules:
-
-- {!Nethttpd_types} is just a module with common type definitions used
-  by the other modules
-- {!Nethttpd_kernel} is the implementation of the HTTP protocol. If we
-  talk about the "kernel" we mean this module. The kernel has two 
-  interface sides: There is the "socket side" and the "message side"
-  that are connected by bidirectional data flow.
-  The task of the kernel is to decode input received by the socket side
-  and to deliver it to a consumer on the message side, and conversely
-  to encode input coming in through the message side and to send it
-  to the socket. The kernel is a quite low-level module; the socket
-  is accessed in a multiplexing-compatible style, and the messages
-  are sequences of tokens like "status line", "header", "body chunk"
-  and so on. Normally a user of the daemon does not program the kernel
-  directly. It is, however, possible to pass certain configuration
-  options to the kernel even if an encapsulation is used.
-- {!Nethttpd_reactor} is an encapsulation of the kernel with a nicer
-  interface. An instance of the reactor processes, like the kernel,
-  only a single HTTP connection. It is used as follows: The user
-  of the reactor pulls the arriving HTTP requests from the reactor,
-  processes them, and writes the responses back to the reactor. This
-  means that the requests are processed in a strictly sequential
-  way. The reactor hides the details of the HTTP protocol. The
-  reactor is able to perform socket input and output at the same time,
-  i.e. when the response is sent to the client the next request(s) can
-  already be read (pipelining). The reactor can be configured such that
-  buffering of requests and responses is avoided, even if large
-  messages are transferred. As mentioned, the reactor can only
-  deal with one connection at the same time. In order to serve
-  several connections, one must use multi-threading.
-- {!Nethttpd_engine} is another encapsulation of the kernel. It is
-  event-based, and this makes it possible that several instances
-  can work at the same time without using multi-threading. The user
-  of the engine is called back when the beginning of the next HTTP
-  request arrives and at certain other events. The user processes
-  the event, and generates the response. 
-  The engine is a
-  highly efficient implementation of an HTTP server, but there are
-  also drawbacks, so user may feel more comfortable with the reactor.
-  Especially, the engine needs large buffers for input and output
-  (there is an idea to use helper threads to avoid these buffers,
-  but this has not been implemented yet). Of course, the engine
-  also supports pipelining.
-- {!Nethttpd_services} has functions to compose complex service
-  functions from simpler ones. In particular, one can configure
-  name- or IP-based virtual hosting, one can bind services to
-  URLs, and one can define static file serving, directory listings,
-  and dynamic services. It is quite easy to turn a Netcgi program
-  into a dynamic service for Nethttpd.
-
-It is also important to mention what Nethttpd does not include:
-
-- There is no function to create the socket, and to accept connections.
-
-- There is no function to manage threads or subprocesses
-
-It is hoped to add this functionality later in a generic way (i.e.
-not only for Nethttpd).
-
-{2 Suggested strategy}
-
-First, look at {!Nethttpd_services}. This module allows the user
-to define the services of the web server. For example, the following
-code defines a single host with an URL space:
-
-{[
-let fs_spec =
-  { file_docroot = "/data/docroot";
-    file_uri = "/";
-    file_suffix_types = [ "txt", "text/plain";
-                          "html", "text/html" ];
-    file_default_type = "application/octet-stream";
-    file_options = [ `Enable_gzip;
-                     `Enable_listings simple_listing
-                   ]
-  }
-
-let srv =
-  host_distributor
-    [ default_host ~pref_name:"localhost" ~pref_port:8765 (),
-      uri_distributor
-        [ "*", (options_service());
-          "/files", (file_service fs_spec);
-          "/service", (dynamic_service
-                           { dyn_handler = process_request;
-                             dyn_activation = std_activation `Std_activation_buffered;
-                             dyn_uri = Some "/service";
-                             dyn_translator = file_translator fs_spec;
-                             dyn_accept_all_conditionals = false
-                           })
-        ]
-    ]
-]}
-
-The [/files] path is bound to a static service, i.e. the files found in
-the directory [/data/docroot] can be accessed over the web. The record
-[fs_spec] configures the static service.
-
-The [/service] path is bound to a dynamic service, i.e. the requests
-are processed by the user-defined function [process_request]. This function
-is very similar to the request processors used in Netcgi.
-
-The symbolic [*] path is only bound for the [OPTIONS] method. This is
-recommended, because clients can use this method to find out the
-capabilities of the server.
-
-Second, select an encapsulation. As mentioned, the reactor is much simpler
-to use, but you must take a multi-threaded approach to serve multiple
-connections simultaneously. The engine is more efficient, but may use
-more memory (unless it is only used for static pages).
-
-Third, write the code to create the socket and to accept connections.
-For the reactor, you should do this in a multi-threaded way (but
-multi-processing is also possible). For the engine, you should do this in
-an event-based way.
-
-Now, just call {!Nethttpd_reactor.process_connection} or 
-{!Nethttpd_engine.process_connection}, and pass the socket descriptor
-as argument. These functions do all the rest.
-
-The Ocamlnet source tarball includes examples for several approaches.
-Especially look at [file_reactor.ml], [file_mt_reactor.ml], and 
-[file_engine.ml].
-
-{2 Configuration}
-
-One of the remaining questions is: How to set all these configuration 
-options.
-
-The user configures the daemon by passing a configuration object.
-This object has a number of methods that usually return constants, but
-there are also a few functions, e.g.
-
-{[
-  let config : http_reactor_config =
-    object
-      method config_timeout_next_request = 15.0
-      method config_timeout = 300.0
-      method config_reactor_synch = `Write
-      method config_cgi = Netcgi_env.default_config
-      method config_error_response n = "<html>Error " ^ string_of_int n ^ "</html>"
-      method config_log_error _ _ _ _ msg =
-        printf "Error log: %s\n" msg
-      method config_max_reqline_length = 256
-      method config_max_header_length = 32768
-      method config_max_trailer_length = 32768
-      method config_limit_pipeline_length = 5
-      method config_limit_pipeline_size = 250000
-    end 
-]}
-
-Some of the options are interpreted by the encapsulation, and some by the
-kernel. The object approach has been taken, because it can be arranged that
-the layers of the daemon correspond to a hierarchy of class types.
-
-The options are documented in the modules where the class types are
-defined. Some of them are difficult to understand. In doubt, it is
-recommended to just copy the values found in the examples, because these
-are quite reasonable for typical usage scenarios.
- *)
