@@ -1,4 +1,4 @@
-(* $Id: netmcore_process.ml 1579 2011-04-12 20:16:22Z gerd $ *)
+(* $Id: netmcore_process.ml 1837 2013-02-12 16:46:35Z gerd $ *)
 
 open Netmcore
 
@@ -25,6 +25,11 @@ let join jp pid =
     | None -> None
     | Some enc -> Some(Obj.obj (Encap.unwrap enc))
 
+let join_nowait jp pid =
+  match Netmcore.join_nowait jp pid with
+    | None -> None
+    | Some enc -> Some(Obj.obj (Encap.unwrap enc))
+
 let release_fork_point fp =
   release fp
 
