@@ -1,4 +1,4 @@
-(* $Id: netplex_container.ml 1718 2012-02-21 14:59:45Z gerd $ *)
+(* $Id: netplex_container.ml 1867 2013-07-21 12:43:13Z gerd $ *)
 
 open Netplex_types
 open Netplex_ctrl_aux
@@ -155,7 +155,10 @@ object(self)
       sockserv # processor # container_run esys
     with
       | error ->
+          let bt = Printexc.get_backtrace() in
 	  self # log `Crit ("run: Exception " ^ Netexn.to_string error);
+          if Printexc.backtrace_status() then
+            self # log `Crit ("run: Backtrace: " ^ bt);
 	  let b = sockserv # processor # global_exception_handler error in
 	  if b then self # protect_run()
 
