@@ -1,4 +1,4 @@
-(* $Id: netplex_log.mli 1405 2010-02-12 01:02:27Z gerd $ *)
+(* $Id: netplex_log.mli 1692 2012-02-05 18:44:00Z gerd $ *)
 
 (** Loggers *)
 
@@ -180,7 +180,8 @@ val level_of_string : string -> level
   * The format string may include variable
   * parts in the syntax [$name] or [${name}]. The following variable
   * specifications are defined:
-  * - [timestamp]: the time in standard format
+  * - [timestamp]: the time in standard format (as set in
+      {!Netlog.current_formatter})
   * - [timestamp:<format>] the time in custom format where [<format>] is a
   *   {!Netdate} format string
   * - [timestamp:unix]: the time in seconds since the epoch
