@@ -1,4 +1,4 @@
-(* $Id: netbuffer.mli 1135 2007-07-31 14:58:58Z gerd $
+(* $Id: netbuffer.mli 1476 2010-08-31 00:05:15Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -62,6 +62,12 @@ val add_substring : t -> string -> int -> int -> unit
 val add_char : t -> char -> unit
     (** [add_char nb c]: Adds a single char at the end of the buffer *)
 
+val add_char_2 : t -> char -> char -> unit
+    (** [add_char_2 nb c1 c2]: Adds two chars at the end of the buffer *)
+
+val add_char_4 : t -> char -> char -> char -> char -> unit
+    (** [add_char_4 nb c1 c2 c3 c4]: Adds four chars at the end of the buffer *)
+
 val add_inplace : ?len:int -> t -> (string -> int -> int -> int) -> int
     (** [add_inplace nb f]: Calls the function [f] to add bytes to the
      * netbuffer [nb]. The arguments of [f] are the buffer, the position
@@ -79,6 +85,21 @@ val add_inplace : ?len:int -> t -> (string -> int -> int -> int) -> int
 val add_buffer : t -> t -> unit
   (** [add_buffer nb1 nb2]: Adds the contents of [nb2] to the end of [nb1] *)
 
+val area_for_additions : ?len:int -> t -> (string * int * int)
+val advance : t -> int -> unit
+  (** These two functions work together, so that the effect of [add_inplace]
+      can be obtained in two steps. First, the user calls
+      {[
+        let (s,pos,len) = area_for_additions nb
+      ]}
+      to get the area where to put new data of length [n], with [n <= len].
+      After this the data is made valid by
+      {[
+        advance n
+      ]}
+   *)
+
+
 (** {2 Inserting strings} *)
 
 val insert_string : t -> int -> string -> unit
@@ -151,6 +172,18 @@ val index_from : t -> int -> char -> int
      * returned. Otherwise, [Not_found] is raised.
      *)
 
+(** {2 Memory} *)
+
+val blit_to_memory : t -> int -> Netsys_mem.memory -> int -> int -> unit
+    (** [blit_to_memory nb srcpos dest destpos len]: Copies the [len] bytes at
+     *  position [srcpos] from [nb] to the membuffer [dest] at position
+	[destpos].
+     *)
+
+val add_sub_memory : t -> Netsys_mem.memory -> int -> int -> unit
+  (** Same as [add_sub_string], but gets data from a memory buffer *)
+
+
 (** {2 Miscelleneous} *)
 
 val unsafe_buffer : t -> string
