@@ -1,4 +1,4 @@
-(* $Id: netconversion.mli 1084 2007-02-20 12:36:17Z gerd $
+(* $Id: netconversion.mli 1816 2012-11-19 02:47:02Z gerd $
  * ----------------------------------------------------------------------
  *)
 
@@ -596,7 +596,6 @@ val from_unicode : charset -> int -> int
    * - PRE_EVAL(charset)
    *)
 
-
 val available_input_encodings : unit -> encoding list
   (** Returns the list of all available encodings that can be used for
    * input strings. The list reflects the set of loadable/linked [Netmapping]
@@ -610,6 +609,22 @@ val available_output_encodings : unit -> encoding list
    * modules.
    *)
 
+val user_encoding : unit -> encoding option
+  (** Determines the preferred user encoding:
+
+      - Unix: This is the character set from the current locale
+      - Win32: This is derived from the current ANSI code page
+
+      If an error occurs while determining the result, the value
+      [None] is returned.
+   *)
+
+val win32_code_pages : (int * encoding) list
+  (** Mapping between Win32 code page numbers and Ocamlnet encodings.
+      This is incomplete. The official list:
+      http://msdn.microsoft.com/en-us/library/dd317756%28v=VS.85%29.aspx
+   *)
+
 
 
 (**********************************************************************)
@@ -1235,7 +1250,34 @@ val ustring_map :
    *   (default: 0)
    * @param range_len The byte length of the substring to map
    *   (default: byte length of the input string minus [range_pos])
-   *) 
+   *)
+
+val ustring_to_lower : encoding -> ?range_pos:int -> ?range_len:int ->
+                       string -> string
+  (** Converts the input string to lowercase.
+
+      The [encoding], [range_pos], and [range_len] arguments work
+      as for [ustring_map]. The exception [Malformed_code] is raised
+      when illegal byte sequences are found.
+   *)
+
+val ustring_to_upper : encoding -> ?range_pos:int -> ?range_len:int ->
+                       string -> string
+  (** Converts the input string to uppercase.
+
+      The [encoding], [range_pos], and [range_len] arguments work
+      as for [ustring_map]. The exception [Malformed_code] is raised
+      when illegal byte sequences are found.
+   *)
+
+val ustring_to_title : encoding -> ?range_pos:int -> ?range_len:int ->
+                       string -> string
+  (** Converts the input string to titlecase.
+
+      The [encoding], [range_pos], and [range_len] arguments work
+      as for [ustring_map]. The exception [Malformed_code] is raised
+      when illegal byte sequences are found.
+   *)
 
 val ustring_sub :
        encoding ->
@@ -1288,6 +1330,20 @@ val ustring_compare :
    *   referring to the following string argument
    *)
 
+val code_cmp : int -> int -> int
+  (** A compare function for [ustring_compare]: Normal string comparison:
+      This function compares by code point
+   *)
+
+val ci_code_cmp : int -> int -> int
+  (** A compare function for [ustring_compare]: Case-insensitive comparison:
+      This function compares by the lowercase code point if it exists, 
+      and the untransformed code point otherwise.
+
+      NB. This bases on the lowercase transformation that maps one char
+      to only one char, and not to many.
+   *)
+
 val uarray_of_ustring : 
     encoding -> 
     ?range_pos:int -> ?range_len:int ->
