@@ -1,11 +1,9 @@
-(* $Id: netencoding.ml 1058 2006-12-17 19:52:30Z gerd $
+(* $Id: netencoding.ml 1652 2011-08-03 21:50:30Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
 
 
-module Str = Netstring_pcre
-
 module Base64 = struct
   let b64_pattern plus slash =
     [| 'A'; 'B'; 'C'; 'D'; 'E'; 'F'; 'G'; 'H'; 'I'; 'J'; 'K'; 'L'; 'M';
@@ -925,17 +923,17 @@ module Url = struct
 
 
   let url_encoding_re =
-    Str.regexp "[^A-Za-z0-9_.!*-]";;
+    Netstring_str.regexp "[^A-Za-z0-9_.!*-]";;
 
   let url_decoding_re =
-    Str.regexp "\\+|%..|%.|%";;
+    Netstring_str.regexp "\\+\\|%..\\|%.\\|%";;
 
 
   let encode ?(plus = true) s =
-    Str.global_substitute
+    Netstring_str.global_substitute
       url_encoding_re
       (fun r _ ->
-	 match Str.matched_string r s with
+	 match Netstring_str.matched_string r s with
 	     " " when plus -> "+"
 	   | x ->
 	       let k = Char.code(x.[0]) in
@@ -944,19 +942,24 @@ module Url = struct
       s ;;
 
 
-  let decode ?(plus = true) s =
-    let l = String.length s in
-    Str.global_substitute
+  let decode ?(plus = true) ?(pos=0) ?len s =
+    let s_l = String.length s in
+    let s1 = 
+      if pos = 0 && len=None then s else 
+	let len = match len with Some n -> n | None -> s_l in
+	String.sub s pos len in
+    let l = String.length s1 in
+    Netstring_str.global_substitute
       url_decoding_re
       (fun r _ ->
-	 match Str.matched_string r s with
+	 match Netstring_str.matched_string r s1 with
 	   | "+" -> if plus then " " else "+"
 	   | _ ->
-	       let i = Str.match_beginning r in
-	       (* Assertion: s.[i] = '%' *)
+	       let i = Netstring_str.match_beginning r in
+	       (* Assertion: s1.[i] = '%' *)
 	       if i+2 >= l then failwith "Netencoding.Url.decode";
-	       let c1 = s.[i+1] in
-	       let c2 = s.[i+2] in
+	       let c1 = s1.[i+1] in
+	       let c2 = s1.[i+2] in
 	       begin
 		 try
 		   let k1 = of_hex1 c1 in
@@ -967,10 +970,10 @@ module Url = struct
 		       failwith "Netencoding.Url.decode"
 	       end
       )
-      s ;;
+      s1 ;;
 
   let url_split_re =
-    Str.regexp "[&=]";;
+    Netstring_str.regexp "[&=]";;
 
   let mk_url_encoded_parameters nv_pairs =
     String.concat "&"
@@ -987,28 +990,199 @@ module Url = struct
   let dest_url_encoded_parameters parstr =
     let rec parse_after_amp tl =
       match tl with
-	  Str.Text name :: Str.Delim "=" :: Str.Text value :: tl' ->
+	  Netstring_str.Text name :: Netstring_str.Delim "=" :: Netstring_str.Text value :: tl' ->
 	    (decode name, decode value) :: parse_next tl'
-	| Str.Text name :: Str.Delim "=" :: Str.Delim "&" :: tl' ->
+	| Netstring_str.Text name :: Netstring_str.Delim "=" :: Netstring_str.Delim "&" :: tl' ->
 	    (decode name, "") :: parse_after_amp tl'
-	| Str.Text name :: Str.Delim "=" :: [] ->
+	| Netstring_str.Text name :: Netstring_str.Delim "=" :: [] ->
 	    [decode name, ""]
 	| _ ->
 	    failwith "Netencoding.Url.dest_url_encoded_parameters"
     and parse_next tl =
       match tl with
 	  [] -> []
-	| Str.Delim "&" :: tl' ->
+	| Netstring_str.Delim "&" :: tl' ->
 	    parse_after_amp tl'
 	| _ ->
 	    failwith "Netencoding.Url.dest_url_encoded_parameters"
     in
-    let toklist = Str.full_split url_split_re parstr in
+    let toklist = Netstring_str.full_split url_split_re parstr in
     match toklist with
 	[] -> []
       | _ -> parse_after_amp toklist
   ;;
 
+(* Alternative version. It is commented out because it causes problems
+   in Neturl.
+ *)
+
+(*
+
+  (* The Ocamlnet3 version is from Christophe Troestler. 
+     See netcgi.mli for the full copyright message.
+
+     This module is a drop-in replacement of [Netencoding.Url].  Not
+     only it is much faster (about 10 times) but it also avoids the
+     dependency on PCRE (less dependencies are good, especially on
+     windows or if you need to install by hand).
+   *)
+
+  (* Decoding -------------------------------------------------- *)
+  exception Hex_of_char
+
+  let hex_of_char =
+    let code_a = Char.code 'a' - 10
+    and code_A = Char.code 'A' - 10 in
+    function
+    | '0' .. '9' as c -> Char.code c - Char.code '0'
+    | 'a' .. 'f' as c -> Char.code c - code_a
+    | 'A' .. 'F' as c -> Char.code c - code_A
+    | _ -> raise Hex_of_char
+
+  (* Overwrite the part of the range [s.[i0 .. up-1]] with the decoded
+     string.  Returns [i] such that [s.[i0 .. i]] is the decoded
+     string.  Invalid '%XX' are left unchanged.  *)
+  let rec decode_range_loop plus i0 i up s =
+    if i0 >= up then i else begin
+      match String.unsafe_get s i0 with
+      | '+' ->
+	  String.unsafe_set s i plus;
+	  decode_range_loop plus (succ i0) (succ i) up s
+      | '%' when i0 + 2 < up ->
+          let i1 = succ i0 in
+          let i2 = succ i1 in
+          let i0_next =
+            try
+              let v = hex_of_char(String.unsafe_get s i1) lsl 4
+                + hex_of_char(String.unsafe_get s i2) in
+	      String.unsafe_set s i (Char.chr v);
+	      succ i2
+            with Hex_of_char ->
+	      String.unsafe_set s i '%';
+	      i1 in
+	  decode_range_loop plus i0_next (succ i) up s
+      | c ->
+	  String.unsafe_set s i c;
+	  decode_range_loop plus (succ i0) (succ i) up s
+    end
+
+  (* We do not strip heading and trailing spaces of key-value data
+     because it does not conform the specs.  However certain browsers
+     do it, so the user should not rely on them.  See e.g.
+     https://bugzilla.mozilla.org/show_bug.cgi?id=114997#c6 *)
+
+  let decode ?(plus=true) ?(pos=0) ?len s =
+    let real_len = 
+      match len with 
+	| None -> String.length s - pos
+	| Some l -> l in
+    let s = String.sub s pos real_len in
+    let up = decode_range_loop (if plus then ' ' else '+') 0 0 real_len s in
+    if up <> real_len then String.sub s 0 up else s
+
+
+  (* Query parsing -------------------------------------------------- *)
+
+  (* It is ASSUMED that the range is valid i.e., [0 <= low] and [up <=
+     String.length s].  *)
+  let decode_range s low up =
+    if low >= up then "" else
+      let len = up - low in
+      let s = String.sub s low len in
+      let up = decode_range_loop ' ' 0 0 len s in
+      if up <> len then String.sub s 0 up else s
+
+  (* Split the query string [qs] into a list of pairs (key,value).
+     [i0] is the initial index of the key or value, [i] the current
+     index and [up-1] the last index to scan. *)
+  let rec get_key qs i0 i up =
+    if i >= up then [(decode_range qs i0 up, "")] else
+      match String.unsafe_get qs i with
+      | '=' -> get_val qs (i+1) (i+1) up (decode_range qs i0 i)
+      | '&' ->
+	  (* key but no val *)
+	  (decode_range qs i0 i, "") :: get_key qs (i+1) (i+1) up
+      | _ ->
+	  get_key qs i0 (i+1) up
+  and get_val qs i0 i up key =
+    if i >= up then [(key, decode_range qs i0 up)] else
+      match String.unsafe_get qs i with
+      | '&' -> (key, decode_range qs i0 i) :: get_key qs (i+1) (i+1) up
+      | _ -> get_val qs i0 (i+1) up key
+
+
+  let dest_url_encoded_parameters qs =
+    if qs = "" then [] else get_key qs 0 0 (String.length qs)
+
+
+
+  (* Encoding -------------------------------------------------- *)
+
+  let hex = [| '0'; '1'; '2'; '3'; '4'; '5'; '6'; '7'; '8'; '9';
+	       'A'; 'B'; 'C'; 'D'; 'E'; 'F' |]
+  let char_of_hex i = Array.(*unsafe_*)get hex i
+
+  let encode_wrt is_special s0 =
+    let len = String.length s0 in
+    let encoded_length = ref len in
+    for i = 0 to len - 1 do
+      if is_special(String.unsafe_get s0 i) then
+	encoded_length := !encoded_length + 2
+    done;
+    let s = String.create !encoded_length in
+    let rec do_enc i0 i = (* copy the encoded string in s *)
+      if i0 < len then begin
+	let s0i0 = String.unsafe_get s0 i0 in
+	(* It is important to check first that [s0i0] is special in
+	   case [' '] is considered as such a character. *)
+	if is_special s0i0 then begin
+          let c = Char.code s0i0 in
+          let i1 = succ i in
+          let i2 = succ i1 in
+          String.unsafe_set s i '%';
+          String.unsafe_set s i1 (char_of_hex (c lsr 4));
+          String.unsafe_set s i2 (char_of_hex (c land 0x0F));
+          do_enc (succ i0) (succ i2)
+	end
+	else if s0i0 = ' ' then begin
+	  String.unsafe_set s i '+';
+          do_enc (succ i0) (succ i)
+	end
+	else begin
+          String.unsafe_set s i s0i0;
+          do_enc (succ i0) (succ i)
+	end
+      end in
+    do_enc 0 0;
+    s
+
+
+  (* Unreserved characters consist of all alphanumeric chars and the
+     following limited set of punctuation marks and symbols: '-' | '_' |
+     '.' | '!' | '~' | '*' | '\'' | '(' | ')'.  According to RFC 2396,
+     they should not be escaped unless the context requires it. *)
+  let special_rfc2396 = function
+    | ';' | '/' | '?' | ':' | '@' | '&' | '=' | '+' | '$' | ',' (* Reserved *)
+    | '\000' .. '\031' | '\127' .. '\255' (* Control chars and non-ASCII *)
+    | '<' | '>' | '#' | '%' | '"'         (* delimiters *)
+    | '{' | '}' | '|' | '\\' | '^' | '[' | ']' | '`' (* unwise *)
+    | '~' (* causes sometimes problems *)
+	-> true
+    | _ -> false
+  (* ' ' must also be encoded but its encoding '+' takes a single char. *)
+
+  let encode ?(plus=true) s =
+    let is_special =
+      if plus then special_rfc2396
+      else (fun c -> special_rfc2396 c || c = ' ') in
+    encode_wrt is_special s
+
+ *)
+
+  let mk_url_encoded_parameters params =
+    String.concat "&"
+      (List.map (fun (name, value) -> encode name ^ "=" ^ encode value) params)
+
 end
 
 
@@ -1316,52 +1490,64 @@ module Html = struct
   let unsafe_chars_html4 = "<>\"&\000\001\002\003\004\005\006\007\008\011\012\014\015\016\017\018\019\020\021\022\023\024\025\026\027\028\029\030\031\127" ;;
 
 
+  let regexp_ht = Hashtbl.create 7
+  let regexp_ht_mutex = !Netsys_oothr.provider # create_mutex()
+
+  let regexp_set s =
+    Netsys_oothr.serialize
+      regexp_ht_mutex
+      (fun () ->
+	 try
+	   Hashtbl.find regexp_ht s
+	 with
+	   | Not_found ->
+	       let re = Netstring_str.regexp (Netstring_str.quote_set s) in
+	       if Hashtbl.length regexp_ht < 100 then  (* avoid leak *)
+		 Hashtbl.replace regexp_ht s re;
+	       re
+      )
+      ()
+
+
   (* The functions [encode_quickly] and [encode_ascii] are special cases of 
    * [encode] that can be implemented by regular expressions.
    *)
 
-  let no_alphanum_re = Netstring_pcre.regexp "[^a-zA-Z0-9]";;
-  let null_re = Netstring_pcre.regexp "\\\\\\000";;
-      (* matches: backslash NUL *)
-
   let encode_quickly ~prefer_name ~unsafe_chars () =
     (* Preconditions: in_enc = out_enc, and the encoding must be a single-byte,
      * ASCII-compatible encoding.
      *)
-    let unsafe_re_str = 
-      "[" ^ Netstring_pcre.global_replace no_alphanum_re "\\$&" unsafe_chars ^
-      "]" in
-    let unsafe_re_str' =
-      Netstring_pcre.global_replace null_re "\\000" unsafe_re_str in
-      (* Replace backslash-NUL by PCRE null *)
-    let unsafe_re = 
-      Netstring_pcre.regexp unsafe_re_str' in
-    Netstring_pcre.global_substitute
-      unsafe_re
-      (fun r s ->
-	 let t = Netstring_pcre.matched_string r s in
-	 let p = Char.code (t.[0]) in    (* p is an ASCII code point *)
-	 let name = rev_etable.(p) in
-	 if prefer_name && name <> "" then
-	   name
-	 else
-	   "&#" ^ string_of_int p ^ ";"
-      )
+    if unsafe_chars = "" then
+      (fun s -> s)
+    else
+      let unsafe_re = 
+	regexp_set unsafe_chars in
+      Netstring_str.global_substitute
+	unsafe_re
+	(fun r s ->
+	   let t = Netstring_str.matched_string r s in
+	   let p = Char.code (t.[0]) in    (* p is an ASCII code point *)
+	   let name = rev_etable.(p) in
+	   if prefer_name && name <> "" then
+	     name
+	   else
+	     "&#" ^ string_of_int p ^ ";"
+	)
   ;;
 
+  let msb_set = (
+    let s = String.create 128 in
+    for k = 0 to 127 do s.[k] <- Char.chr (128+k) done;
+    s
+  )
 
   let encode_ascii ~in_enc ~prefer_name ~unsafe_chars () =
     (* Preconditions: out_enc = `Enc_usascii, and in_enc must be a single-byte,
      * ASCII-compatible encoding.
      *)
-    let unsafe_re_str = 
-      "[" ^ Netstring_pcre.global_replace no_alphanum_re "\\$&" unsafe_chars ^ 
-      "\128-\255]" in
-    let unsafe_re_str' =
-      Netstring_pcre.global_replace null_re "\\000" unsafe_re_str in
-      (* Replace backslash-NUL by PCRE null *)
+    let unsafe_chars1 = unsafe_chars ^ msb_set in
     let unsafe_re = 
-      Netstring_pcre.regexp unsafe_re_str' in
+      regexp_set unsafe_chars1 in
     (* unicode_of.[q] = p: the code point q+128 of in_enc is the same as the
      * Unicode code point p
      *)
@@ -1377,10 +1563,10 @@ module Html = struct
 	  Netconversion.Malformed_code -> 
 	    unicode_of.(i) <- (-1)
     done;
-    Netstring_pcre.global_substitute
+    Netstring_str.global_substitute
       unsafe_re
       (fun r s ->
-	 let t = Netstring_pcre.matched_string r s in
+	 let t = Netstring_str.matched_string r s in
 	 (* p is the code point in the encoding ~in_enc; p' is the Unicode
 	  * code point:
 	  *)
@@ -1467,9 +1653,11 @@ module Html = struct
   type entity_set = [ `Html | `Xml | `Empty ];;
 
   let eref_re = 
-    Pcre.regexp "\\&(\\#([0-9]+);|([a-zA-Z]+)\\;)" ;;
-
-  (* TODO: Decode hex numerals, too *)
+    Netstring_str.regexp "&\\(\
+                            #\\([0-9]+\\);\\|\
+                            #[xX]\\([0-9a-fA-F]+\\);\\|\
+                            \\([a-zA-Z]+\\);\
+                          \\)" ;;
 
   let total_enc =
     (* every byte must have a corresponding Unicode code point, i.e. the
@@ -1490,6 +1678,34 @@ module Html = struct
       | _ -> false
   ;;
 
+  let hex_digit_of_char c =
+    match c with
+       '0'..'9' -> Char.code c - 48
+      | 'A'..'F' -> Char.code c - 55
+      | 'a'..'f' -> Char.code c - 87
+      | _ -> assert false
+
+  let hex_of_string s =
+    let n = ref 0 in
+    for i = 0 to String.length s - 1 do
+      let d = hex_digit_of_char s.[i] in
+      n := (!n lsl 4) lor d
+    done;
+    !n
+
+
+  let search_all re s pos =
+    let rec search p acc =
+      match 
+	try Some(Netstring_str.search_forward re s p) with Not_found -> None
+      with
+	| Some (k,r) ->
+	    search (k+1) ( (k,r) :: acc )
+	| None ->
+	    List.rev acc in
+    search pos []
+      
+
   let decode 
         ~in_enc
         ~out_enc
@@ -1534,50 +1750,61 @@ module Html = struct
 	Netconversion.recode_string ~in_enc ~out_enc ~subst
     in
     (fun s ->
-       (* Find all occurrences of &name; or &#num; *)
-       let occurences = 
-	 try Pcre.exec_all ~rex:eref_re s with Not_found -> [| |] in
+       (* Find all occurrences of &name; or &#num; or &#xnum; *)
+       let occurrences = search_all eref_re s 0  in
        (* Collect the resulting string in a buffer *)
        let buf = Buffer.create 250 in
        let n = ref 0 in
-       for k = 0 to Array.length occurences - 1 do
-	 let (n0,n1) = Pcre.get_substring_ofs (occurences.(k)) 0 in
-	 if n0 > !n then
-	   Buffer.add_string buf (recode_str (String.sub s !n (n0 - !n)));
-	 (* TODO: avoid String.sub *)
-	 let occurence = occurences.(k) in
-	 let replacement =
-	   let num = 
-	     try Pcre.get_substring occurence 2 with Not_found -> "" in
-	   (* Note: Older versions of Pcre return "" when the substring
-	    * did not match, newer versions raise Not_found  
-	    *)
-	   if num <> "" then begin
-	     let n = int_of_string num in
-	     makechar n
-	   end
-	   else begin
-	     let name = 
-	       try Pcre.get_substring occurence 3 with Not_found -> "" in
-	     (* Note: Older versions of Pcre return "" when the substring
-	      * did not match, newer versions raise Not_found  
-	      *)
-	     assert(name <> "");
-	     lookup_entity name
-	   end
-	 in
-	 Buffer.add_string buf replacement;
-	 n := n1;
-       done;
+       List.iter
+	 (fun (n0,r) ->
+	    let n1 = Netstring_str.match_end r in
+	    if n0 > !n then
+	      Buffer.add_string buf (recode_str (String.sub s !n (n0 - !n)));
+	    (* TODO: avoid String.sub *)
+	    let replacement =
+	      let num = 
+		try Netstring_str.matched_group r 2 s with Not_found -> "" in
+	      (* Note: Older versions of Pcre return "" when the substring
+	       * did not match, newer versions raise Not_found  
+	       *)
+	      if num <> "" then begin
+		let n = int_of_string num in
+		makechar n
+	      end
+	      else begin
+		let xnum = 
+		  try Netstring_str.matched_group r 3 s with Not_found -> "" in
+		(* Note: Older versions of Pcre return "" when the substring
+		 * did not match, newer versions raise Not_found  
+		 *)
+		if xnum <> "" then begin
+		  let n = hex_of_string xnum in
+		  makechar n
+		end
+		else begin
+		  let name = 
+		    try Netstring_str.matched_group r 4 s with Not_found -> "" in
+		  (* Note: Older versions of Pcre return "" when the substring
+	           * did not match, newer versions raise Not_found  
+	           *)
+		  assert(name <> "");
+		  lookup_entity name
+		end
+	      end
+	    in
+	    Buffer.add_string buf replacement;
+	    n := n1;
+	 )
+	 occurrences;
        let n0 = String.length s in
        if n0 > !n then
 	 Buffer.add_string buf (recode_str (String.sub s !n (n0 - !n)));
-	 (* TODO: avoid String.sub *)
+       (* TODO: avoid String.sub *)
        (* Return *)
        Buffer.contents buf
     )
   ;;
-
+     
   let decode_to_latin1 =
     decode ~in_enc:`Enc_iso88591 ~out_enc:`Enc_iso88591 
            ~lookup:(fun s -> "&" ^ s ^ ";")
