@@ -1,4 +1,4 @@
-(* $Id: netencoding.mli 1003 2006-09-24 15:17:15Z gerd $
+(* $Id: netencoding.mli 1285 2009-10-20 13:43:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -212,9 +212,12 @@ module Url :
      *)
 
 
-    val decode : ?plus:bool -> string -> string
+    val decode : ?plus:bool -> ?pos:int -> ?len:int -> string -> string
 	(** Option [plus]: Whether '+' is converted to space. The default
 	 * is true. If false, '+' is returned as it is.
+         *
+         * The optional arguments [pos] and [len] may restrict the string
+         * to process to this substring.
 	 *)
     val encode : ?plus:bool -> string -> string
 	(** Option [plus]: Whether spaces are converted to '+'. The default
