@@ -1,4 +1,4 @@
-(* $Id: nethttp.mlp 1787 2012-06-27 14:28:10Z gerd $ 
+(* $Id: nethttp.mlp 1875 2013-08-13 20:41:36Z gerd $ 
  * ----------------------------------------------------------------------
  * Nethttp: Basic definitions for the HTTP protocol
  *)
@@ -209,6 +209,17 @@ let http_status_of_int =
   | 505 -> `Http_version_not_supported
   | _ -> raise Not_found
   
+let base_code code =
+  if (code >= 100) && (code < 200)
+  then 100
+  else
+    if (code >= 200) && (code < 300)
+    then 200
+    else
+      if (code >= 300) && (code < 400)
+      then 300
+      else if (code >= 400) && (code < 500) then 400 else 500
+  
 type http_method = (string * string)
 
 (** Method name, URI *)
@@ -457,19 +468,19 @@ module Cookie =
       in
         (for i = 0 to len - 1 do
            (match String.unsafe_get s0 i with
-            | '\"' | '\\' | '\n' | '\r' -> incr encoded_length
+            | '"' | '\\' | '\n' | '\r' -> incr encoded_length
             | '\000' .. '\031' -> decr encoded_length
             | (* ignore *) _ -> ())
          done;
          let s = String.create (!encoded_length + 2)
          in
-           (String.unsafe_set s 0 '\"';
+           (String.unsafe_set s 0 '"';
             let j = ref 1
             in
               (* Ignore these control chars, useless for comments *)
               (for i = 0 to len - 1 do
                  (match String.unsafe_get s0 i with
-                  | ('\"' | '\\' as c) ->
+                  | ('"' | '\\' as c) ->
                       (String.unsafe_set s !j '\\';
                        incr j;
                        String.unsafe_set s !j c;
@@ -487,7 +498,7 @@ module Cookie =
                   | '\000' .. '\031' -> ()
                   | c -> (String.unsafe_set s !j c; incr j))
                done;
-               String.unsafe_set s !j '\"';
+               String.unsafe_set s !j '"';
                s)))
       
     (* [gen_cookie c] returns a buffer containing an attribute suitable
@@ -586,7 +597,7 @@ module Cookie =
                     if i < len
                     then
                       ((match String.unsafe_get s i with
-                        | ('\"' | '\\' as c) -> String.unsafe_set s j c
+                        | ('"' | '\\' as c) -> String.unsafe_set s j c
                         | 'n' -> String.unsafe_set s j '\n'
                         | 'r' -> String.unsafe_set s j '\r'
                         | 't' -> String.unsafe_set s j '\t'
@@ -668,7 +679,7 @@ module Cookie =
         (match String.unsafe_get cs i with
          | ' ' | '\t' | '\n' | '\r' -> (* skip linear white space *)
              skip_value_space cs (i + 1) len key cl
-         | '\"' -> get_quoted_value cs (i + 1) (i + 1) len key cl
+         | '"' -> get_quoted_value cs (i + 1) (i + 1) len key cl
          | _ -> get_value cs i i len key cl)
     and get_value cs i0 i len key cl =
       if i >= len
@@ -688,7 +699,7 @@ module Cookie =
       else
         (match String.unsafe_get cs i with
          | '\\' -> get_quoted_value cs i0 (i + 2) len key cl
-         | '\"' ->
+         | '"' ->
              let cl = add_key_val key (unescape_range cs i0 i) cl
              in skip_to_next cs (i + 1) len cl
          | _ -> get_quoted_value cs i0 (i + 1) len key cl)
