@@ -1,4 +1,4 @@
-(* $Id: nethttp.mli 1285 2009-10-20 13:43:39Z gerd $
+(* $Id: nethttp.mli 1875 2013-08-13 20:41:36Z gerd $
  * ----------------------------------------------------------------------
  * Nethttp: Basic definitions for the HTTP protocol
  *)
@@ -150,6 +150,24 @@ val string_of_http_status : http_status -> string
   (** Returns the informational text for a status value *)
   (* See also Netcgi.status_line *)
 
+val base_code : int -> int
+  (** Allows to handle unknown status codes that are untranslatable by
+      [http_status_of_int]:
+       - for a code 100 to 199 the value 100 is returned
+       - for a code 200 to 299 the value 200 is returned
+       - for a code 300 to 399 the value 300 is returned
+       - for a code 400 to 499 the value 400 is returned
+       - for all other codes 500 is returned
+
+      E.g.
+      {[
+  let st =
+    try Nethttp.http_status_of_int code
+    with Not_found ->
+      Nethttp.http_status_of_int (Nethttp.base_code code)
+      ]}
+   *)
+
 type http_method = string * string
   (** Method name, URI *)
 
@@ -455,10 +473,10 @@ module Header : sig
       * acceptable, "" will be returned.
       * The supported media types should be sorted such that the best type
       * is mentioned first.
-      *
-      * Known bug: The rule that media ranges are sorted by degree of
-      * "specificness" is not implemented, e.g. that "text/html" is
-      * preferred over other "text/*" when both have the same "q" value.
+      * Of several media types with equal quality the one mentioned first in the
+      * list of supported types is chosen. In case several types in the Accept:
+      * header match the same type in the list of supported types, the most
+      * specific type is chosen.
       *)
 
   val set_accept : #http_header -> (string *
