@@ -1,4 +1,4 @@
-(* $Id: neturl.ml 1062 2006-12-17 20:17:36Z gerd $
+(* $Id: neturl.ml 1662 2011-08-29 23:05:06Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -123,6 +123,10 @@ let ualpha = [ 'A'; 'B'; 'C'; 'D'; 'E'; 'F'; 'G'; 'H'; 'I'; 'J'; 'K'; 'L'; 'M';
 
 let digit = [ '0'; '1'; '2'; '3'; '4'; '5'; '6'; '7'; '8'; '9' ]
 
+let hex_digit = [ 'a'; 'b'; 'c'; 'd'; 'e'; 'f';
+		  'A'; 'B'; 'C'; 'D'; 'E'; 'F';
+		] @ digit
+
 let safe = [ '$'; '-'; '_'; '.'; '+' ]
 
 let extra = [ '!'; '*'; '\''; '('; ')'; ',' ]
@@ -175,14 +179,22 @@ let host_cats =
     (lalpha @ ualpha @ digit @ ['.'; '-'])
     []
 ;;
-
     (* host_cats: character categorization to _check_ whether the host name
      * is formed only by legal characters.
      * Especially '%' is not allowed here!
-     *
-     * CHECK: IPv6
+     * IPv6 addresses are checked separately.
      *)
 
+let ipv6_cats =
+  make_cats
+    (hex_digit @ [ ':' ])
+    [ ]
+
+let ipv6_sep_cats =
+  make_cats
+    (hex_digit @ [ ':' ])
+    [ ']' ]
+
 let port_cats =
   make_cats
     digit
@@ -256,7 +268,6 @@ let other_cats_from_syntax syn =
      *)
 
 
-
 let extract_url_scheme s = 
   let l = String.length s in
   let k = scan_url_part s 0 l scheme_cats false in
@@ -588,6 +599,13 @@ let url_syntax_of_url url = url.url_syntax
 ;;
 
 
+let host_of_addr ip =
+  match Netsys.domain_of_inet_addr ip with
+    | Unix.PF_INET -> Unix.string_of_inet_addr ip
+    | Unix.PF_INET6 -> "[" ^ Unix.string_of_inet_addr ip ^ "]"
+    | _ -> assert false
+
+
 let modify_url
       ?syntax
       ?(encoded = false)
@@ -596,7 +614,9 @@ let modify_url
       ?user_param
       ?password
       ?host
+      ?addr
       ?port
+      ?socksymbol
       ?path
       ?param
       ?query
@@ -649,7 +669,36 @@ let modify_url
       p
   in
 
+  let check_host s =
+    let l = String.length s in
+    if String.length s >= 2 && s.[0] = '[' then (
+      if s.[l-1] <> ']' then raise Malformed_URL;
+      let ipv6 = String.sub s 1 (l-2) in
+      check_string (Some ipv6) ipv6_cats
+    )
+    else
+      check_string (Some s) host_cats
+  in
+
   (* Create the modified record: *)
+  let url_host_0 =
+    match addr with
+      | Some a -> Some(host_of_addr a)
+      | None ->
+	  ( match host with
+	      | Some h -> Some h
+	      | None -> url.url_host
+	  ) in
+  let url_port_0 =
+    match port with
+      | Some p -> Some p
+      | None -> url.url_port in
+  let (url_host, url_port) =
+    match (socksymbol : Netsockaddr.socksymbol option) with
+      | Some(`Inet(ip,p)) -> (Some(host_of_addr ip), Some p)
+      | Some(`Inet_byname(h,p)) -> (Some h, Some p)
+      | Some _ -> failwith "Neturl: Unacceptable socksymbol"
+      | None -> (url_host_0, url_port_0) in
   let url' =
     { 
       url_syntax   = new_syntax;
@@ -660,8 +709,8 @@ let modify_url
 			     None -> url.url_user_param
 			   | Some p -> enc_list p);
       url_password = if password = None then url.url_password else enc password;
-      url_host     = if host     = None then url.url_host     else host;
-      url_port     = if port     = None then url.url_port     else port;
+      url_host     = url_host;
+      url_port     = url_port;
       url_path     = (match path with
 			  None -> url.url_path
 			| Some p -> enc_list p);
@@ -685,7 +734,10 @@ let modify_url
   check_string url'.url_user            login_cats;
   check_string_list url'.url_user_param login_cats ';';
   check_string url'.url_password        login_cats;
-  check_string url'.url_host            host_cats;
+  ( match url'.url_host with
+      | None -> ()
+      | Some s -> check_host s
+  );
   (match url'.url_port with 
        None -> ()
      | Some p -> if p < 0 || p > 65535 then raise Malformed_URL
@@ -758,7 +810,9 @@ let make_url
       ?user_param
       ?password
       ?host
+      ?addr
       ?port
+      ?socksymbol
       ?path
       ?param
       ?query
@@ -778,7 +832,9 @@ let make_url
     ?user_param
     ?password
     ?host
+    ?addr
     ?port
+    ?socksymbol
     ?path
     ?param
     ?query
@@ -995,6 +1051,36 @@ let url_fragment   ?(encoded=false) url = decode_if encoded url.url_fragment;;
 let url_other      ?(encoded=false) url = decode_if encoded url.url_other;;
 
 
+let url_addr url =
+  match url.url_host with
+    | None -> raise Not_found
+    | Some h ->
+	let l = String.length h in
+	if l >= 2 && h.[0] = '[' && h.[l-1] = ']' then
+	  let a = String.sub h 1 (l-2) in
+	  try Unix.inet_addr_of_string a
+	  with _ -> raise Not_found
+	else (
+	  try Unix.inet_addr_of_string h
+	  with _ -> raise Not_found
+	)
+
+let url_socksymbol url dp =
+  let p =
+    match url.url_port with
+      | None -> dp
+      | Some p -> p in
+  try
+    let a = url_addr url in
+    `Inet(a, p)
+  with
+    | Not_found ->
+	(match url.url_host with
+	   | None -> raise Not_found
+	   | Some h -> `Inet_byname(h, p)
+	)
+
+
 let string_of_url url =
   if not (url.url_validity) then
     failwith "Neturl.string_of_url: URL not flagged as valid";
@@ -1063,22 +1149,39 @@ let url_of_string url_syntax s =
   let l = String.length s in
   let recognized x = x <> Url_part_not_recognized in
 
-  let rec collect_words terminators eof_char cats k =
+  let rec collect_words ?(ipv6=false) terminators eof_char cats k =
     (* Collect words as recognized by 'cats', starting at position 'k' in
      * 's'. Collection stops if one the characters listed in 'terminators'
      * is found. If the end of the string is reached, it is treated as
      * 'eof_char'.
+     * 
+     * if ipv6: words "[ipv6addr]" are also recognized.
      *)
-    let k' = scan_url_part s k l cats url_syntax.url_accepts_8bits in  
-             (* or raise Malformed_URL *)
-    let word, sep =
-      String.sub s k (k'-k), (if k'<l then s.[k'] else eof_char) in
+    let word, sep, k_end =
+      if ipv6 && k < l && s.[k] = '[' then (
+	let k' = scan_url_part s (k+1) l ipv6_sep_cats false in
+	if k' >= l then raise Malformed_URL;
+	if s.[k'] <> ']' then raise Malformed_URL;
+	let word, sep =
+	  String.sub s k (k'+1-k), (if k'+1<l then s.[k'+1] else eof_char) in
+	if sep <> eof_char then (
+	  if cats.(Char.code sep) <> Separator then raise Malformed_URL
+	);
+	(word, sep, k'+1)
+      )
+      else (
+	let k' = scan_url_part s k l cats url_syntax.url_accepts_8bits in  
+	(* or raise Malformed_URL *)
+	let word, sep =
+	  String.sub s k (k'-k), (if k'<l then s.[k'] else eof_char) in
+	(word, sep, k')
+      ) in
     if List.mem sep terminators then
-      [word, sep], k'
+      [word, sep], k_end
     else
-      let word_sep_list', k'' = 
-	collect_words terminators eof_char cats (k'+1) in
-      ((word, sep) :: word_sep_list'), k''
+      let word_sep_list', k_end' = 
+	collect_words ~ipv6 terminators eof_char cats (k_end+1) in
+      ((word, sep) :: word_sep_list'), k_end'
   in
 
   (* Try to extract the scheme name: *)
@@ -1101,8 +1204,8 @@ let url_of_string url_syntax s =
     if recognized url_syntax.url_enable_host  &&
        k1 + 2 <= l  &&  s.[k1]='/'  && s.[k1+1]='/' then begin
 
-      let word_sep_list, k' = collect_words [ '/'; '?'; '#' ] '/' login_cats (k1+2) 
-      in
+      let word_sep_list, k' = 
+	collect_words ~ipv6:true [ '/'; '?'; '#' ] '/' login_cats (k1+2) in
           (* or raise Malformed_URL *)
 
       let int x =
@@ -1281,13 +1384,14 @@ let url_of_string url_syntax s =
 
 
 
-let problem_re = Pcre.regexp "[ <>\"{}|\\\\^\\[\\]`]"
+let problem_re = Netstring_str.regexp "[ <>\"{}|\\^`]"
+let problem_hash_re = Netstring_str.regexp "[ <>\"{}|\\^`#]"
 
-let fixup_url_string =
-  Netstring_pcre.global_substitute 
-    problem_re
+let fixup_url_string ?(escape_hash=false) =
+  Netstring_str.global_substitute 
+    (if escape_hash then problem_hash_re else problem_re)
     (fun m s ->
-       sprintf "%%%02x" (Char.code s.[Netstring_pcre.match_beginning m]))
+       sprintf "%%%02x" (Char.code s.[Netstring_str.match_beginning m]))
 ;;
 
   
@@ -1526,10 +1630,10 @@ let print_url url =
 ;;
 
 
-let backslash_re = Netstring_pcre.regexp "\\\\";;
-let drive_letter_re = Netstring_pcre.regexp "^([A-Za-z]):/";;
-let drive_letter_re' = Netstring_pcre.regexp "^/([A-Za-z]):/";;
-let unc_path_re = Netstring_pcre.regexp "^//([^/]+)(/|$)";;
+let backslash_re = Netstring_str.regexp "\\\\";;
+let drive_letter_re = Netstring_str.regexp "^\\([A-Za-z]\\):/";;
+let drive_letter_re' = Netstring_str.regexp "^/\\([A-Za-z]\\):/";;
+let unc_path_re = Netstring_str.regexp "^//\\([^/]+\\)\\(/|$\\)";;
 
 
 let os_type = Sys.os_type;;
@@ -1542,15 +1646,15 @@ let classify_path p =
 	else
 	  `Relative p
     | "Win32" ->
-	let p' = Netstring_pcre.global_replace backslash_re "/" p in
-	( match Netstring_pcre.string_match drive_letter_re p' 0 with
+	let p' = Netstring_str.global_replace backslash_re "/" p in
+	( match Netstring_str.string_match drive_letter_re p' 0 with
 	      Some m ->
 		`Absolute_local ("/" ^ p')
 	    | None ->
-		( match Netstring_pcre.string_match unc_path_re p' 0 with
+		( match Netstring_str.string_match unc_path_re p' 0 with
 		      Some m ->
-			let host = Netstring_pcre.matched_group m 1 p' in
-			let host_e = Netstring_pcre.group_end m 1 in
+			let host = Netstring_str.matched_group m 1 p' in
+			let host_e = Netstring_str.group_end m 1 in
 			let path = String.sub p' host_e (String.length p' - host_e) in
 			let path = if path = "" then "/" else path in
 			`Absolute_remote(host,path)
@@ -1562,17 +1666,17 @@ let classify_path p =
 		)
 	)
     | "Cygwin" ->
-	let p' = Netstring_pcre.global_replace backslash_re "/" p in
-	( match Netstring_pcre.string_match drive_letter_re p' 0 with
+	let p' = Netstring_str.global_replace backslash_re "/" p in
+	( match Netstring_str.string_match drive_letter_re p' 0 with
 	      Some m ->
-		let letter = Netstring_pcre.matched_group m 1 p' in
+		let letter = Netstring_str.matched_group m 1 p' in
 		let rest = String.sub p' 2 (String.length p' - 2) in
 		`Absolute_local("/cygdrive/" ^ letter ^ rest)
 	    | None ->
-		( match Netstring_pcre.string_match unc_path_re p' 0 with
+		( match Netstring_str.string_match unc_path_re p' 0 with
 		      Some m ->
-			let host = Netstring_pcre.matched_group m 1 p' in
-			let host_e = Netstring_pcre.group_end m 1 in
+			let host = Netstring_str.matched_group m 1 p' in
+			let host_e = Netstring_str.group_end m 1 in
 			let path = String.sub p' host_e (String.length p' - host_e) in
 			let path = if path = "" then "/" else path in
 			`Absolute_remote(host,path)
@@ -1610,9 +1714,9 @@ let file_url_of_local_path ?(getcwd = Sys.getcwd) p =
       | `Relative_drive r ->
 	  ( match classify_path (getcwd()) with
 		`Absolute_local l ->
-		  ( match Netstring_pcre.string_match drive_letter_re' l 0 with
+		  ( match Netstring_str.string_match drive_letter_re' l 0 with
 			Some m ->
-			  let letter = Netstring_pcre.matched_group m 1 l in
+			  let letter = Netstring_str.matched_group m 1 l in
 			  `Absolute_local("/" ^ letter ^ ":" ^ r)
 		      | None ->
 			  assert false
@@ -1638,7 +1742,7 @@ let file_url_of_local_path ?(getcwd = Sys.getcwd) p =
 ;;
 
 
-let drive_letter_comp_re = Netstring_pcre.regexp "^([A-Za-z])(:|\\|)";;
+let drive_letter_comp_re = Netstring_str.regexp "^\\([A-Za-z]\\)\\(:|\\|\\)";;
 
 let local_path_of_file_url u =
   let local_path p =
@@ -1651,9 +1755,9 @@ let local_path_of_file_url u =
 	  (* There must be a drive letter: *)
 	  ( match p with
 		("" :: drive :: rest) ->
-		  (match Netstring_pcre.string_match drive_letter_comp_re drive 0 with
+		  (match Netstring_str.string_match drive_letter_comp_re drive 0 with
 		       Some m ->
-			 let letter = Netstring_pcre.matched_group m 1 drive in
+			 let letter = Netstring_str.matched_group m 1 drive in
 			 let rest = if rest = [] then [""] else rest in
 			 join_path((letter ^ ":") :: rest)
 		     | None ->
@@ -1666,9 +1770,9 @@ let local_path_of_file_url u =
 	  (* Recognize drive letters: *)
 	  ( match p with
 		("" :: drive :: rest) ->
-		  (match Netstring_pcre.string_match drive_letter_comp_re drive 0 with
+		  (match Netstring_str.string_match drive_letter_comp_re drive 0 with
 		       Some m ->
-			 let letter = Netstring_pcre.matched_group m 1 drive in
+			 let letter = Netstring_str.matched_group m 1 drive in
 			 join_path("" :: "cygdrive" :: letter :: rest)
 		     | None ->
 			 join_path p
