@@ -1,12 +1,12 @@
 (*
- * $Id: xdr.ml 1562 2011-03-07 16:13:14Z gerd $
+ * $Id: xdr.ml 1702 2012-02-14 21:01:59Z gerd $
  *)
 
 (* This is an XDR implementation.
  * See RFC 1014
  *)
 
-open Rtypes;;
+open Netnumber
 open Printf
 
 exception Propagate of string;;
@@ -143,6 +143,7 @@ let equal_sets l1 l2 =
 ;;
 *)
 
+
 (**********************************************************************)
 (* definition of XDR types and type systems                           *)
 (**********************************************************************)
@@ -179,6 +180,10 @@ type xdr_type_term =
   | X_param of string
   | X_rec of (string * xdr_type_term)      (* define a recursive type *)
   | X_refer of string                      (* refer to a recursive type *)
+  | X_direct of xdr_type_term * 
+                (string -> int ref -> int -> exn) *
+                (exn -> string -> int ref -> unit) *
+                (exn -> int)
 ;;
 
 
@@ -188,8 +193,12 @@ module StringSet = Set.Make(String)
 
 type xdr_type0 =
   { mutable term   : xdr_term;
-    mutable params : StringSet.t
+    mutable params : StringSet.t;
       (* "params" is normally only non-empty in the top node *)
+    mutable min_size : int
+      (* min_size: the minimum number of bytes every element of the array
+	 will take in XDR form. This does not include any inner parameters.
+       *)
   }
 and xdr_term =
     T_int
@@ -205,7 +214,7 @@ and xdr_term =
   | T_string of uint4
   | T_mstring of string * uint4
   | T_array_fixed of xdr_type0 * uint4
-  | T_array of       xdr_type0 * uint4
+  | T_array of       xdr_type0 * (* max size: *) uint4
   | T_struct of (string * xdr_type0) array
   | T_union_over_int of
       (int4, xdr_type0) Hashtbl.t * xdr_type0 option
@@ -220,6 +229,10 @@ and xdr_term =
   | T_param of string
   | T_rec of (string * xdr_type0)
   | T_refer of (string * xdr_type0)
+  | T_direct of xdr_type0 * 
+                (string -> int ref -> int -> exn) *
+                (exn -> string -> int ref -> unit) *
+                (exn -> int)
 ;;
 
 type xdr_type =
@@ -261,6 +274,7 @@ let t_name =
     | T_param _ -> "T_param"
     | T_rec _ -> "T_rec"
     | T_refer _ -> "T_refer"
+    | T_direct _ -> "T_direct"
 	
 
 let x_bool =
@@ -292,6 +306,9 @@ let x_array_max t =
 (* definition of XDR values                                           *)
 (**********************************************************************)
 
+type xdr_value_version =
+    [ `V1 | `V2 | `V3 | `V4 | `Ocamlrpcgen ]
+
 type xdr_value =
     XV_int of int4
   | XV_uint of uint4
@@ -313,6 +330,7 @@ type xdr_value =
   | XV_union_over_enum_fast of (int * xdr_value)
   | XV_array_of_string_fast of string array
   | XV_mstring of Xdr_mstring.mstring
+  | XV_direct of exn * int
 ;;
 
 let xv_true = XV_enum_fast 1 (* "TRUE" *);;
@@ -487,6 +505,24 @@ let () =
 exception Xdr_failure of string
 
 
+let safe_add x y = (* exported *)
+  (* pre: x >= 0 && y >= 0 *)
+  let s = x + y in
+  if s < 0 then (* can only happen on 32 bit platforms *)
+    raise(Xdr_failure "int overflow while computing size");
+  s
+
+let safe_mul x y = (* exported *)
+  (* pre: x >= 0 && y >= 0 *)
+  if x=0 || y=0 then
+    0
+  else
+    let n = max_int / y in
+    if x > n then
+      raise(Xdr_failure "int overflow while computing size");
+    x * y
+
+
 (**********************************************************************)
 (* check if XDR types are well-formed                                 *)
 (**********************************************************************)
@@ -507,7 +543,8 @@ let rec validate_xdr_type_i1
    * raise Not_found on any error
    *)
 
-  let mktype tm = { term = tm; params = StringSet.empty } in
+  let mktype tm = { term = tm; params = StringSet.empty; min_size = (-1) } in
+  (* min_size is calculated in a second pass *)
 
   match t with
     X_int    -> mktype T_int
@@ -522,7 +559,7 @@ let rec validate_xdr_type_i1
       if all_distinct e_names && all_distinct e_values then
 	let ea =
 	  Array.map
-	    (fun (n,i) -> (n, Rtypes.int32_of_int4 i))
+	    (fun (n,i) -> (n, Netnumber.int32_of_int4 i))
 	    (Array.of_list e) in
 	Array.sort (fun (_,i) (_,i') -> compare i i') ea;
 	mktype (T_enum ea)
@@ -532,8 +569,12 @@ let rec validate_xdr_type_i1
   | X_opaque n          -> mktype (T_opaque n)
   | X_string n          -> mktype (T_string n)
   | X_mstring (name,n)  -> mktype (T_mstring (name,n))
-  | X_array_fixed (s,n) -> mktype (T_array_fixed(validate_xdr_type_i1 r b s, n))
-  | X_array (s,n)       -> mktype (T_array      (validate_xdr_type_i1 r b s, n))
+  | X_array_fixed (s,n) -> 
+      let nL = int64_of_uint4 n in
+      if nL > 0x3fff_ffff_ffffL then
+	raise (Propagate "Bad fixed array: bound too high");
+      mktype (T_array_fixed(validate_xdr_type_i1 r b s, n))
+  | X_array (s,n)       -> mktype (T_array (validate_xdr_type_i1 r b s, n))
   | X_struct s ->
       let s_names, s_types = List.split s in
       if all_distinct s_names then
@@ -616,9 +657,10 @@ let rec validate_xdr_type_i1
       let t' = validate_xdr_type_i1 r ((name,node)::b) s in
       node.term <- T_rec (name, t');
       node
-
   | X_refer name ->
       mktype (T_refer (name, List.assoc name b))
+  | X_direct(s, read, write, size) ->
+      mktype (T_direct (validate_xdr_type_i1 r b s, read, write, size))
 ;;
 
 
@@ -661,6 +703,8 @@ let rec find_params (t:xdr_type0) : StringSet.t =
                       u
   | T_rec (_,t') ->
       find_params t'
+  | T_direct(t',_,_,_) ->
+      find_params t'
   | _ ->
       StringSet.empty
 ;;
@@ -723,8 +767,123 @@ let rec elim_rec t = (* get rid of T_rec and T_refer *)
 	elim_rec t'
     | T_refer(n,t') ->
 	t'
+    | T_direct(t',read,write,size) ->
+	{ t with term = T_direct(elim_rec t', read, write, size) }
 
 
+let rec calc_min_size t =
+
+  let ( ++ ) x y =
+    (* pre: x >= 0 && y >= 0 *)
+    let s = x + y in
+    if s < 0 then (* can only happen on 32 bit platforms *)
+      raise(Propagate("Minimum size of type exceeds limit"));
+    s in
+
+
+  let calc_for_union u_snd default =
+    ( match default with
+	| None -> ()
+	| Some d -> calc_min_size d
+    );
+    List.iter (fun t' -> calc_min_size t') u_snd;
+    let l =
+      (match default with
+	 | None -> []
+	 | Some d -> [d]
+      ) @ u_snd in
+    assert(l <> []);
+    4 ++
+      (List.fold_left
+	 (fun acc x ->
+	    min acc x.min_size
+	 )
+	 ((List.hd l).min_size)
+	 (List.tl l)
+      )
+  in
+
+  let hashtbl_vals h =
+    Hashtbl.fold (fun _ v acc -> v :: acc) h [] in
+
+  let optarray_elems a =
+    Array.fold_left
+      (fun acc x_opt ->
+	 match x_opt with
+	   | None -> acc
+	   | Some x -> x :: acc
+      )
+      []
+      a in
+
+  if t.min_size < 0 then (
+    t.min_size <- 0;   (* for stopping recursions *)
+    ( match t.term with
+	  T_int    -> t.min_size <- 4
+	| T_uint   -> t.min_size <- 4
+	| T_hyper  -> t.min_size <- 8
+	| T_uhyper -> t.min_size <- 8
+	| T_float  -> t.min_size <- 4
+	| T_double -> t.min_size <- 8
+	| T_void   -> t.min_size <- 0
+	| T_enum e -> t.min_size <- 4
+	| T_opaque_fixed n -> 
+	    let nL = int64_of_uint4 n in
+	    let min_size =
+	      if nL=0L then 0 
+	      else Int64.to_int(Int64.succ (Int64.div (Int64.pred nL) 4L)) in
+	    t.min_size <- min_size
+	| T_opaque n -> t.min_size <- 4
+	| T_string n -> t.min_size <- 4
+	| T_mstring (name,n) -> t.min_size <- 4
+	| T_array_fixed (s,n) -> 
+	    calc_min_size s;
+	    if s.min_size = 0 then
+	      raise(Propagate "Array elements must not have length 0");
+	    let nL = int64_of_uint4 n in
+	    let n_max = max_int / s.min_size in
+	    if nL > Int64.of_int n_max then
+	      raise(Propagate "Minimum size of type exceeds limit");
+	    let iL = Int64.of_int s.min_size in
+	    t.min_size <- Int64.to_int (Int64.mul nL iL)
+	| T_array (s,n) -> 
+	    calc_min_size s;
+	    if s.min_size = 0 then
+	      raise(Propagate "Array elements must not have length 0");
+	    t.min_size <- 4
+	| T_struct s ->
+	    Array.iter (fun (_,t') -> calc_min_size t') s;
+	    t.min_size <-
+	      (Array.fold_left
+		 (fun acc (_,x) ->
+		    acc ++ x.min_size
+		 )
+		 0
+		 s
+	      )
+	| T_union_over_int (u, default) ->
+	    t.min_size <- calc_for_union (hashtbl_vals u) default
+	| T_union_over_uint (u, default) ->
+	    t.min_size <- calc_for_union (hashtbl_vals u) default
+	| T_union_over_enum (e,u,default) ->
+	    t.min_size <- calc_for_union (optarray_elems u) default
+	| T_param p ->
+	    (* not optimal, but we do not know it better at this point *)
+	    t.min_size <- 0
+	| T_direct(t',_,_,_) ->
+	    calc_min_size t';
+	    t.min_size <- t'.min_size
+	| T_rec (_,t') ->
+	    calc_min_size t';
+	    t.min_size <- t'.min_size
+	| T_refer (r,t') ->
+	    calc_min_size t';
+	    t.min_size <- t'.min_size;
+	    (* eprintf "%s: " r*)
+    );
+    (* eprintf "min_size(%s) = %d\n" (t_name t.term) t.min_size*)
+  )
+
 
 let rec validate_xdr_type (t:xdr_type_term) : xdr_type =
   let r n =
@@ -735,6 +894,8 @@ let rec validate_xdr_type (t:xdr_type_term) : xdr_type =
     let pl = find_params t0' in
     t0'.params <- pl;
     let t1' = elim_rec t0' in
+    calc_min_size t0';
+    calc_min_size t1';
     (t0', t1')
   with
     Not_found ->
@@ -785,6 +946,8 @@ let validate_xdr_type_system (s:xdr_type_term_system) : xdr_type_system =
 	      let pl = find_params t0' in
 	      t0'.params <- pl;
 	      let t1' = elim_rec t0' in
+	      calc_min_size t0';
+	      calc_min_size t1';
 	      (t0',t1')
 	    with
 	      Not_found -> failwith "Xdr.validate_xdr_type_system: unspecified error"
@@ -820,7 +983,7 @@ let rec xdr_type_term0 (t:xdr_type0) : xdr_type_term =
   | T_uhyper -> X_uhyper
   | T_enum l -> X_enum (Array.to_list
 			  (Array.map
-			     (fun (n,i) -> (n,Rtypes.int4_of_int32 i))
+			     (fun (n,i) -> (n,Netnumber.int4_of_int32 i))
 			     l))
   | T_float  -> X_float
   | T_double -> X_double
@@ -855,6 +1018,8 @@ let rec xdr_type_term0 (t:xdr_type0) : xdr_type_term =
 	  )
       in
       X_union_over_enum (xdr_type_term0 e_term, u', conv_option d)
+  | T_direct (t', read, write, size) -> 
+      X_direct (xdr_type_term0 t',read, write, size)
   | _ ->
       assert false
 ;;
@@ -873,7 +1038,7 @@ let xdr_type_term_system (s:xdr_type_system) : xdr_type_term_system =
 (* expand X_type members relative to given systems                    *)
 (**********************************************************************)
 
-(* The implemantation of "expanded_xdr_type_term" repeats many phrases
+(* The implementation of "expanded_xdr_type_term" repeats many phrases
  * that have been defined for "validate_xdr_type" in a very similar
  * way.
  * TODO: Currently many checks have been left out
@@ -942,6 +1107,8 @@ let rec expanded_xdr_type_term (s:xdr_type_term_system) (t:xdr_type_term)
       r [] s
   | X_rec (n, t') ->
       X_rec (n, expanded_xdr_type_term s t')
+  | X_direct (t',read, write, size) ->
+      X_direct ((expanded_xdr_type_term s t'), read, write, size)
   | _ ->
       t
 ;;
@@ -951,6 +1118,8 @@ let expanded_xdr_type (s:xdr_type_system) (t:xdr_type_term) : xdr_type =
   try
     let t0 = validate_xdr_type_i1 (expand_X_type s) [] t in
     let t1 = elim_rec t0 in
+    calc_min_size t0;
+    calc_min_size t1;
     (t0,t1)
   with
     Not_found -> failwith "Xdr.expanded_xdr_type: unspecified error"
@@ -1000,7 +1169,7 @@ let get_string_decoration_size x_len n =
   (* header field plus padding *)
   let x_len_u = uint4_of_int x_len in
   let x_len_mod_4 = x_len land 3 in
-  if Rtypes.le_uint4 x_len_u n then begin
+  if Netnumber.le_uint4 x_len_u n then begin
     (if x_len_mod_4 = 0
      then 4 
      else 8 - x_len_mod_4
@@ -1011,6 +1180,16 @@ let get_string_decoration_size x_len n =
       (Xdr_failure "string is longer than allowed")
 
 
+let sizefn_string n x =
+  let x_len = String.length x in
+  get_string_decoration_size x_len n + x_len
+
+
+let sizefn_mstring n x =
+  let x_len = x#length in
+  get_string_decoration_size x_len n + x_len
+
+
 let pack_size
       (v:xdr_value)
       (t:xdr_type0)
@@ -1040,10 +1219,10 @@ let pack_size
 	  int_of_uint4 n
       | T_opaque n ->
 	  let x = dest_xv_opaque v in
-	  get_string_size x n
+	  sizefn_string n x
       | T_string n ->
 	  let x = dest_xv_string v in
-	  get_string_size x n
+	  sizefn_string n x
       | T_mstring(_,n) ->
 	  (* for an mstring we only count the length field plus padding *)
 	  let x = dest_xv_mstring v in
@@ -1052,7 +1231,7 @@ let pack_size
       | T_array_fixed (t',n) ->
 	  get_array_size v t' n (fun m n -> m=n)
       | T_array (t',n) ->
-	  4 + get_array_size v t' n Rtypes.le_uint4
+	  4 + get_array_size v t' n Netnumber.le_uint4
       | T_struct s ->
 	  let v_array = map_xv_struct_fast0 t v in
 	  let sum = ref 0 in
@@ -1110,6 +1289,11 @@ let pack_size
 	  get_size v t'
       | T_refer (n, t') ->
 	  get_size v t'
+      | T_direct(t', _, _, _) ->
+	  ( match v with
+	      | XV_direct(_,size) -> size
+	      | _ -> get_size v t'
+	  )
 
   and get_array_size v t' n cmp =  (* w/o array header *)
     (* TODO: optimize arrays of types with fixed repr length *)
@@ -1124,7 +1308,7 @@ let pack_size
 	    !s
 	  )
 	  else
-	    raise (Xdr_failure "array is longer than allowed")
+	    raise (Xdr_failure "array length mismatch")
       | XV_array_of_string_fast x ->
 	  ( match t'.term with
 	      | T_string sn ->
@@ -1132,26 +1316,35 @@ let pack_size
 		  if cmp m n then (
 		    let sum = ref 0 in
 		    Array.iter
-		      (fun s -> sum := !sum ++ get_string_size s sn)
+		      (fun s -> sum := !sum ++ sizefn_string sn s)
 		      x;
 		    !sum
 		  )
 		  else 
-		    raise (Xdr_failure "array is longer than allowed")
+		    raise (Xdr_failure "array length mismatch")
+	      | T_direct(t1, _, _, _) ->
+		  get_array_size v t1 n cmp
 	      | _ -> 
 		  raise Dest_failure
 	  )
       | _ ->
 	  raise Dest_failure
 
-  and get_string_size x n =
-    let x_len = String.length x in
-    x_len + get_string_decoration_size x_len n
-
   in
   get_size v t
 
 
+let print_string_padding l buf pos =
+  let n = 4-(l land 3) in
+  if n < 4 then begin
+    let p = !pos in
+    if n >= 1 then String.unsafe_set buf p '\000';
+    if n >= 2 then String.unsafe_set buf (p + 1) '\000';
+    if n >= 3 then String.unsafe_set buf (p + 2) '\000';
+    pos := p + n
+  end
+
+
 let rec pack_mstring 
       (v:xdr_value)
       (t:xdr_type0)
@@ -1183,53 +1376,42 @@ let rec pack_mstring
     )
   in
 
-  let print_string_padding l =
-    let n = 4-(l land 3) in
-    if n < 4 then begin
-      let l0 = !buf_pos in
-      if n >= 1 then String.unsafe_set buf l0 '\000';
-      if n >= 2 then String.unsafe_set buf (l0 + 1) '\000';
-      if n >= 3 then String.unsafe_set buf (l0 + 2) '\000';
-      buf_pos := l0 + n
-    end
-  in
-
   let print_string s l =
     String.unsafe_blit s 0 buf !buf_pos l;
     buf_pos := !buf_pos + l;
-    print_string_padding l
+    print_string_padding l buf buf_pos
   in
 
   let rec pack v t =
     match t.term with
 	T_int ->
 	  let x = dest_xv_int v in
-	  Rtypes.write_int4_unsafe buf !buf_pos x;
+	  Netnumber.BE.write_int4_unsafe buf !buf_pos x;
 	  buf_pos := !buf_pos + 4
       | T_uint ->
 	  let x = dest_xv_uint v in
-	  Rtypes.write_uint4_unsafe buf !buf_pos x;
+	  Netnumber.BE.write_uint4_unsafe buf !buf_pos x;
 	  buf_pos := !buf_pos + 4
       | T_hyper ->
 	  let x = dest_xv_hyper v in
-	  Rtypes.write_int8_unsafe buf !buf_pos x;
+	  Netnumber.BE.write_int8_unsafe buf !buf_pos x;
 	  buf_pos := !buf_pos + 8
       | T_uhyper ->
 	  let x = dest_xv_uhyper v in
-	  Rtypes.write_uint8_unsafe buf !buf_pos x;
+	  Netnumber.BE.write_uint8_unsafe buf !buf_pos x;
 	  buf_pos := !buf_pos + 8
       | T_enum e ->
 	  let i = map_xv_enum_fast0 t v in
-	  Rtypes.write_int4_unsafe buf !buf_pos (int4_of_int32 i);
+	  Netnumber.BE.write_int4_unsafe buf !buf_pos (int4_of_int32 i);
 	  buf_pos := !buf_pos + 4
       | T_float ->
 	  let x = dest_xv_float v in
-	  let s = fp4_as_string x in
+	  let s = Netnumber.BE.fp4_as_string x in
 	  String.unsafe_blit s 0 buf !buf_pos 4;
 	  buf_pos := !buf_pos + 4
       | T_double ->
 	  let x = dest_xv_double v in
-	  let s = fp8_as_string x in
+	  let s = Netnumber.BE.fp8_as_string x in
 	  String.unsafe_blit s 0 buf !buf_pos 8;
 	  buf_pos := !buf_pos + 8
       | T_opaque_fixed n ->
@@ -1238,23 +1420,23 @@ let rec pack_mstring
       | T_opaque n ->
 	  let x = dest_xv_opaque v in
 	  let x_len = String.length x in
-	  Rtypes.write_uint4_unsafe buf !buf_pos (uint4_of_int x_len);
+	  Netnumber.BE.write_uint4_unsafe buf !buf_pos (uint4_of_int x_len);
 	  buf_pos := !buf_pos + 4;
 	  print_string x x_len
       | T_string n ->
 	  let x = dest_xv_string v in
 	  let x_len = String.length x in
-	  Rtypes.write_uint4_unsafe buf !buf_pos (uint4_of_int x_len);
+	  Netnumber.BE.write_uint4_unsafe buf !buf_pos (uint4_of_int x_len);
 	  buf_pos := !buf_pos + 4;
 	  print_string x x_len
       | T_mstring(_,n) ->
 	  let x = dest_xv_mstring v in
 	  let x_len = x#length in
-	  Rtypes.write_uint4_unsafe buf !buf_pos (uint4_of_int x_len);
+	  Netnumber.BE.write_uint4_unsafe buf !buf_pos (uint4_of_int x_len);
 	  buf_pos := !buf_pos + 4;
 	  save_buf();
 	  result := x :: !result;
-	  print_string_padding x_len
+	  print_string_padding x_len buf buf_pos
       | T_array_fixed (t',n) ->
 	  pack_array v t' n false
       | T_array (t',n) ->
@@ -1276,7 +1458,7 @@ let rec pack_mstring
 		      Some d -> d
 		    | None   -> raise (Xdr_failure "T_union_over_int")
 	  in
-	  Rtypes.write_int4_unsafe buf !buf_pos i;
+	  Netnumber.BE.write_int4_unsafe buf !buf_pos i;
 	  buf_pos := !buf_pos + 4;
 	  pack x t'
       | T_union_over_uint (u,default) ->
@@ -1290,7 +1472,7 @@ let rec pack_mstring
 		      Some d -> d
 		    | None   -> raise (Xdr_failure "T_union_over_uint")
 	  in
-	  Rtypes.write_uint4_unsafe buf !buf_pos i;
+	  Netnumber.BE.write_uint4_unsafe buf !buf_pos i;
 	  buf_pos := !buf_pos + 4;
 	  pack x t'
       | T_union_over_enum (et,u,default) ->
@@ -1304,7 +1486,7 @@ let rec pack_mstring
 		      | None -> raise (Xdr_failure "T_union_over_enum")
 		  )
 	  in
-	  Rtypes.write_int4_unsafe buf !buf_pos (int4_of_int32 i);
+	  Netnumber.BE.write_int4_unsafe buf !buf_pos (int4_of_int32 i);
 	  buf_pos := !buf_pos + 4;
 	  pack x t'
       | T_void ->
@@ -1327,6 +1509,15 @@ let rec pack_mstring
 	  pack v t'
       | T_refer (n, t') ->
 	  pack v t'
+      | T_direct(t', _, write, _) ->
+	  ( match v with
+	      | XV_direct(x,xv_size) ->
+		  let old = !buf_pos in
+		  write x buf buf_pos;
+(* Printf.eprintf "old=%d new=%d size=%d\n" old !buf_pos size; *)
+		  assert(!buf_pos = old + xv_size);
+	      | _ -> pack v t'
+	  )
 
   and pack_array v t' n have_array_header =
     match v with
@@ -1342,18 +1533,20 @@ let rec pack_mstring
 		  Array.iter
 		    (fun s ->
 		       let s_len = String.length s in
-		       Rtypes.write_uint4_unsafe
+		       Netnumber.BE.write_uint4_unsafe
 			 buf !buf_pos (uint4_of_int s_len);
 		       buf_pos := !buf_pos + 4;
 		       print_string s s_len
 		    )
 		    x
+	      | T_direct(t1,_,_,_) ->
+		  pack_array v t1 n have_array_header
 	      | _ -> raise Dest_failure
 	  )
       | _ -> raise Dest_failure
 
   and pack_array_header x_len =
-    Rtypes.write_uint4_unsafe buf !buf_pos (uint4_of_int x_len);
+    Netnumber.BE.write_uint4_unsafe buf !buf_pos (uint4_of_int x_len);
     buf_pos := !buf_pos + 4;
   in
   pack v t;
@@ -1362,185 +1555,22 @@ let rec pack_mstring
 ;;
 
 
-(*
-let pack_buf
-      ?(rm = false)
-      (v:xdr_value)
-      (t:xdr_type0)
-      (get_param:string->xdr_type)
-    : string =
+let write_string_fixed n x buf pos = (* exported *)
+  let x_len = String.length x in
+  if x_len <> n then
+    raise (Xdr_failure "fixed string has bad length");
+  String.unsafe_blit x 0 buf !pos x_len;
+  pos := !pos + x_len;
+  print_string_padding x_len buf pos
+  
 
-  let size = 
-    (if rm then 4 else 0) ++
-      pack_size v t get_param in  (* all sanity checks are done here! *)
-
-  let buf = String.create size in
-  let buf_len = ref 0 in
-
-  if rm then (
-    buf.[0] <- '\000';
-    buf.[1] <- '\000';
-    buf.[2] <- '\000';
-    buf.[3] <- '\000';
-    buf_len := 4
-  );
-
-  let print_string s l =  (* assert(l=String.length s) *)
-    let n = 4-(l land 3) in
-    if n < 4 then begin
-      String.unsafe_blit s 0 buf !buf_len l;
-      let l0 = !buf_len + l in
-      if n >= 1 then String.unsafe_set buf l0 '\000';
-      if n >= 2 then String.unsafe_set buf (l0 + 1) '\000';
-      if n >= 3 then String.unsafe_set buf (l0 + 2) '\000';
-      buf_len := l0 + n
-    end
-    else begin
-      String.unsafe_blit s 0 buf !buf_len l;
-      buf_len := !buf_len + l
-    end
-  in
-
-  let rec pack v t =
-    match t.term with
-	T_int ->
-	  let x = dest_xv_int v in
-	  Rtypes.write_int4_unsafe buf !buf_len x;
-	  buf_len := !buf_len + 4
-      | T_uint ->
-	  let x = dest_xv_uint v in
-	  Rtypes.write_uint4_unsafe buf !buf_len x;
-	  buf_len := !buf_len + 4
-      | T_hyper ->
-	  let x = dest_xv_hyper v in
-	  Rtypes.write_int8_unsafe buf !buf_len x;
-	  buf_len := !buf_len + 8
-      | T_uhyper ->
-	  let x = dest_xv_uhyper v in
-	  Rtypes.write_uint8_unsafe buf !buf_len x;
-	  buf_len := !buf_len + 8
-      | T_enum e ->
-	  let i = map_xv_enum_fast0 t v in
-	  Rtypes.write_int4_unsafe buf !buf_len (int4_of_int32 i);
-	  buf_len := !buf_len + 4
-      | T_float ->
-	  let x = dest_xv_float v in
-	  let s = fp4_as_string x in
-	  String.unsafe_blit s 0 buf !buf_len 4;
-	  buf_len := !buf_len + 4
-      | T_double ->
-	  let x = dest_xv_double v in
-	  let s = fp8_as_string x in
-	  String.unsafe_blit s 0 buf !buf_len 8;
-	  buf_len := !buf_len + 8
-      | T_opaque_fixed n ->
-	  let x = dest_xv_opaque v in
-	  print_string x (String.length x)
-      | T_opaque n ->
-	  let x = dest_xv_opaque v in
-	  let x_len = String.length x in
-	  Rtypes.write_uint4_unsafe buf !buf_len (uint4_of_int x_len);
-	  buf_len := !buf_len + 4;
-	  print_string x x_len
-      | T_string n ->
-	  let x = dest_xv_string v in
-	  let x_len = String.length x in
-	  Rtypes.write_uint4_unsafe buf !buf_len (uint4_of_int x_len);
-	  buf_len := !buf_len + 4;
-	  print_string x x_len
-      | T_array_fixed (t',n) ->
-	  pack_array v t' n false
-      | T_array (t',n) ->
-	  pack_array v t' n true
-      | T_struct s ->
-	  let v_array = map_xv_struct_fast0 t v in
-	  Array.iteri
-	    (fun k v_component ->
-	       pack v_component (snd s.(k)))
-	    v_array
-      | T_union_over_int (u,default) ->
-	  let i,x = dest_xv_union_over_int v in
-	  let t' =
-      	    try
-	      Hashtbl.find u i
-	    with
-		Not_found ->
-		  match default with
-		      Some d -> d
-		    | None   -> raise Not_found
-	  in
-	  Rtypes.write_int4_unsafe buf !buf_len i;
-	  buf_len := !buf_len + 4;
-	  pack x t'
-      | T_union_over_uint (u,default) ->
-	  let i,x = dest_xv_union_over_uint v in
-	  let t' =
-      	    try
-	      Hashtbl.find u i
-	    with
-		Not_found ->
-		  match default with
-		      Some d -> d
-		    | None   -> raise Not_found
-	  in
-	  Rtypes.write_uint4_unsafe buf !buf_len i;
-	  buf_len := !buf_len + 4;
-	  pack x t'
-      | T_union_over_enum (et,u,default) ->
-	  let k,i,x = map_xv_union_over_enum_fast0 t v in
-	  let t' =
-	    match u.(k) with
-		Some u_t -> u_t
-	      | None     ->
-		  ( match default with
-			Some d -> d
-		      | None -> raise Not_found
-		  )
-	  in
-	  Rtypes.write_int4_unsafe buf !buf_len (int4_of_int32 i);
-	  buf_len := !buf_len + 4;
-	  pack x t'
-      | T_void ->
-	  ()
-      | T_param n ->
-	  let t' = get_param n in
-	  pack v (snd t')
-      | T_rec (n, t') ->
-	  pack v t'
-      | T_refer (n, t') ->
-	  pack v t'
-
-  and pack_array v t' n have_array_header =
-    match v with
-      | XV_array x ->  (* generic *)
-	  if have_array_header then pack_array_header (Array.length x);
-	  Array.iter
-	    (fun v' -> pack v' t')
-	    x
-      | XV_array_of_string_fast x ->
-	  ( match t'.term with
-	      | T_string n ->
-		  if have_array_header then pack_array_header (Array.length x);
-		  Array.iter
-		    (fun s ->
-		       let s_len = String.length s in
-		       Rtypes.write_uint4_unsafe
-			 buf !buf_len (uint4_of_int s_len);
-		       buf_len := !buf_len + 4;
-		       print_string s s_len
-		    )
-		    x
-	      | _ -> raise Dest_failure
-	  )
-      | _ -> raise Dest_failure
-
-  and pack_array_header x_len =
-    Rtypes.write_uint4_unsafe buf !buf_len (uint4_of_int x_len);
-    buf_len := !buf_len + 4;
-  in
-  pack v t;
-  buf
- *)
+let write_string x buf pos = (* exported *)
+  let x_len = String.length x in
+  Netnumber.BE.write_uint4_unsafe buf !pos (uint4_of_int x_len);
+  pos := !pos + 4;
+  String.unsafe_blit x 0 buf !pos x_len;
+  pos := !pos + x_len;
+  print_string_padding x_len buf pos
 
 
 let value_matches_type
@@ -1592,9 +1622,9 @@ let pack_xdr_value
     with
       | Dest_failure ->
 	  raise(Xdr_failure "Xdr.pack_xdr_value [2]: XDR type mismatch")
-      | Rtypes.Cannot_represent _ ->
+      | Netnumber.Cannot_represent _ ->
 	  raise(Xdr_failure "Xdr.pack_xdr_value [3]: integer not representable")
-      | Rtypes.Out_of_range ->
+      | Netnumber.Out_of_range ->
 	  raise(Xdr_failure "Xdr.pack_xdr_value [4]: index out of range")
       | Failure s ->
 	  raise(Xdr_failure ("Xdr.pack_xdr_value [5]: " ^ s))
@@ -1628,12 +1658,14 @@ let pack_xdr_value_as_string
       Xdr_mstring.concat_mstrings mstrings
     with
       | Dest_failure ->
+(*let bt = Printexc.get_backtrace() in
+eprintf "Backtrace: %s\n" bt; *)
 	  raise(Xdr_failure
 		  "Xdr.pack_xdr_value_as_string [2]: XDR type mismatch")
-      | Rtypes.Cannot_represent _ ->
+      | Netnumber.Cannot_represent _ ->
 	  raise(Xdr_failure
 		  "Xdr.pack_xdr_value_as_string [3]: integer not representable")
-      | Rtypes.Out_of_range ->
+      | Netnumber.Out_of_range ->
 	  raise(Xdr_failure
 		  "Xdr.pack_xdr_value_as_string [4]: index out of range")
       | Failure s ->
@@ -1659,11 +1691,11 @@ let pack_xdr_value_as_mstrings
       | Dest_failure ->
 	  raise(Xdr_failure
 		  "Xdr.pack_xdr_value_as_mstring [2]: XDR type mismatch")
-      | Rtypes.Cannot_represent _ ->
+      | Netnumber.Cannot_represent _ ->
 	  raise
 	    (Xdr_failure
 	       "Xdr.pack_xdr_value_as_mstring [3]: integer not representable")
-      | Rtypes.Out_of_range ->
+      | Netnumber.Out_of_range ->
 	  raise(Xdr_failure
 		  "Xdr.pack_xdr_value_as_mstring [4]: index out of range")
       | Failure s ->
@@ -1720,7 +1752,7 @@ let hex_dump_s s pos len =
   Buffer.contents b
  *)
 
-let read_string str k k_end n =
+let read_string_fixed n str k k_end = (* exported *)
   let k0 = !k in
   let m = if n land 3 = 0 then n else n+4-(n land 3) in
   if k0 > k_end - m then raise_xdr_format_too_short ();
@@ -1729,6 +1761,17 @@ let read_string str k k_end n =
   k := k0 + m;
   s
 
+let read_string n str k k_end = (* exported *)
+  let k0 = !k in
+  k := k0 + 4;
+  if !k > k_end then raise_xdr_format_too_short();
+  let m = Netnumber.BE.read_uint4_unsafe str k0 in
+    (* Test: n < m as unsigned int32: *)
+  if Netnumber.lt_uint4 n m then
+    raise_xdr_format_maximum_length ();
+  read_string_fixed (int_of_uint4 m) str k k_end
+  
+
 let empty_mf = Hashtbl.create 1
 
 let rec unpack_term
@@ -1737,6 +1780,7 @@ let rec unpack_term
     ?(fast = false)
     ?(prefix = false)
     ?(mstring_factories = empty_mf)
+    ?(xv_version = if fast then `Ocamlrpcgen else `V1)
     (str:string)
     (t:xdr_type0)
     (get_param:string->xdr_type)
@@ -1747,6 +1791,13 @@ let rec unpack_term
      parameters
    *)
 
+  let xv_version =
+    if xv_version = `Ocamlrpcgen then `V4 else xv_version in
+
+  let v2 = (xv_version <> `V1) in      (* meaning: at least v2 *)
+  let v3 = v2 && (xv_version <> `V2) in
+  let v4 = v3 && (xv_version <> `V3) in
+
   let len =
     match len with
 	None -> String.length str - pos
@@ -1762,21 +1813,21 @@ let rec unpack_term
   let rec read_fp4 k0 =
     if k0 + 4 > k_end then raise_xdr_format_too_short();
     k := !k + 4;
-    Rtypes.read_fp4 str k0
+    Netnumber.BE.read_fp4 str k0
   in
 
   let rec read_fp8 k0 =
     if k0 + 8 > k_end then raise_xdr_format_too_short();
     k := !k + 8;
-    Rtypes.read_fp8 str k0
+    Netnumber.BE.read_fp8 str k0
   in
 
   let rec read_enum e k0 =
     k := k0 + 4;
     if !k > k_end then raise_xdr_format_too_short();
-    let i = Rtypes.int32_of_int4(Rtypes.read_int4_unsafe str k0) in
+    let i = Netnumber.int32_of_int4(Netnumber.BE.read_int4_unsafe str k0) in
     let j = find_enum e i in   (* returns array position, or Xdr_format *)
-    if fast then
+    if v2 then
       XV_enum_fast j
     else
       XV_enum(fst(e.(j)))
@@ -1785,11 +1836,11 @@ let rec unpack_term
   let rec read_string_or_opaque n k0 =
     k := k0 + 4;
     if !k > k_end then raise_xdr_format_too_short();
-    let m = Rtypes.read_uint4_unsafe str k0 in
+    let m = Netnumber.BE.read_uint4_unsafe str k0 in
     (* Test: n < m as unsigned int32: *)
-    if Rtypes.lt_uint4 n m then
+    if Netnumber.lt_uint4 n m then
       raise_xdr_format_maximum_length ();
-    read_string str k k_end (int_of_uint4 m)
+    read_string_fixed (int_of_uint4 m) str k k_end
   in
 
   let rec read_mstring name n k0 =
@@ -1802,9 +1853,9 @@ let rec unpack_term
 	) in
     k := k0 + 4;
     if !k > k_end then raise_xdr_format_too_short();
-    let m = Rtypes.read_uint4_unsafe str k0 in
+    let m = Netnumber.BE.read_uint4_unsafe str k0 in
     (* Test: n < m as unsigned int32: *)
-    if Rtypes.lt_uint4 n m then
+    if Netnumber.lt_uint4 n m then
       raise_xdr_format_maximum_length ();
     let m = int_of_uint4 m in
     let p = if m land 3 = 0 then m else m+4-(m land 3) in
@@ -1815,16 +1866,22 @@ let rec unpack_term
   in
 
   let rec unpack_array t' p =
+    (* Estimate the maximum p *)
+(* eprintf "unpack_array: t' = %s\n" (t_name t'.term);*)
+    assert(t'.min_size > 0);
+    let p_max = (k_end - !k) / t'.min_size in
+    if p > p_max then 
+      raise_xdr_format_too_short();
     match t'.term with
       | T_string n ->
-	  let n' = Rtypes.logical_int32_of_uint4 n in
+	  let n' = Netnumber.logical_int32_of_uint4 n in
 	  let a = Array.create p "" in
 	  let k' = 
 	    Netsys_xdr.s_read_string_array_unsafe str !k (k_end - !k) n' a in
 	  if k' = (-1) then raise_xdr_format_too_short();
 	  if k' = (-2) then raise_xdr_format_maximum_length ();
 	  k := k';
-	  if fast then
+	  if v3 then
 	    XV_array_of_string_fast a
 	  else
 	    XV_array(Array.map (fun s -> XV_string s) a)
@@ -1842,19 +1899,19 @@ let rec unpack_term
       T_int ->
 	k := k0 + 4;
 	if !k > k_end then raise_xdr_format_too_short();
-	XV_int (Rtypes.read_int4_unsafe str k0)
+	XV_int (Netnumber.BE.read_int4_unsafe str k0)
     | T_uint ->
 	k := k0 + 4;
 	if !k > k_end then raise_xdr_format_too_short();
-	XV_uint (Rtypes.read_uint4_unsafe str k0)
+	XV_uint (Netnumber.BE.read_uint4_unsafe str k0)
     | T_hyper ->
 	k := k0 + 8;
 	if !k > k_end then raise_xdr_format_too_short();
-	XV_hyper (Rtypes.read_int8_unsafe str k0)
+	XV_hyper (Netnumber.BE.read_int8_unsafe str k0)
     | T_uhyper ->
 	k := !k + 8;
 	if k0 > k_end then raise_xdr_format_too_short();
-	XV_uhyper (read_uint8_unsafe str k0)
+	XV_uhyper (Netnumber.BE.read_uint8_unsafe str k0)
     | T_enum e ->
 	read_enum e k0
     | T_float ->
@@ -1862,7 +1919,7 @@ let rec unpack_term
     | T_double ->
 	XV_double (read_fp8 k0)
     | T_opaque_fixed n ->
-	XV_opaque (read_string str k k_end (int_of_uint4 n))
+	XV_opaque (read_string_fixed (int_of_uint4 n) str k k_end)
     | T_opaque n ->
 	XV_opaque (read_string_or_opaque n k0)
     | T_string n ->
@@ -1874,12 +1931,12 @@ let rec unpack_term
 	unpack_array t' p
     | T_array (t',n) ->
 	k := k0 + 4;
-	let m = Rtypes.read_uint4 str k0 in
-	if Rtypes.lt_uint4 n m then
+	let m = Netnumber.BE.read_uint4 str k0 in
+	if Netnumber.lt_uint4 n m then
 	  raise_xdr_format_maximum_length ();
 	unpack_array t' (int_of_uint4 m)
     | T_struct s ->
-	if fast then
+	if v2 then
 	  XV_struct_fast
 	    ( Array.map
 		(fun (name,t') -> unpack t')
@@ -1910,7 +1967,7 @@ let rec unpack_term
 		assert( !k <= k_end );
 		let (v, p) = 
 		  unpack_term
-		    ~fast ~mstring_factories dec_s (snd t')
+		    ~mstring_factories ~xv_version dec_s (snd t')
 		    (fun _ -> assert false)
 		    (fun _ -> None) in
 		v
@@ -1918,12 +1975,19 @@ let rec unpack_term
     | T_rec (_, t')
     | T_refer (_, t') ->
 	unpack t'
+    | T_direct(t', read, _, _) ->
+	if v4 then
+	  let k0 = !k in
+	  let xv = read str k k_end in
+	  XV_direct(xv, !k-k0)
+	else
+	  unpack t'
     | _ ->
 	assert false
 
   and unpack_union_over_int u default k0 =
     k := k0 + 4;
-    let n = Rtypes.read_int4 str k0 in
+    let n = Netnumber.BE.read_int4 str k0 in
     let t' =
       try
 	Hashtbl.find u n
@@ -1937,7 +2001,7 @@ let rec unpack_term
 
   and unpack_union_over_uint u default k0 =
     k := k0 + 4;
-    let n = Rtypes.read_uint4 str k0 in
+    let n = Netnumber.BE.read_uint4 str k0 in
     let t' =
       try
 	Hashtbl.find u n
@@ -1951,7 +2015,7 @@ let rec unpack_term
 
   and unpack_union_over_enum e u default k0 =
     k := k0 + 4;
-    let i = Rtypes.int32_of_int4 (Rtypes.read_int4 str k0) in
+    let i = Netnumber.int32_of_int4 (Netnumber.BE.read_int4 str k0) in
     let j = find_enum e i  (* returns array position, or Xdr_format *) in
     let t' =
       match u.(j) with
@@ -1963,7 +2027,7 @@ let rec unpack_term
 		    raise_xdr_format_undefined_descriminator()
 	    )
     in
-    if fast then
+    if v2 then
       XV_union_over_enum_fast(j, unpack t')
     else
       let name = fst(e.(j)) in
@@ -1990,7 +2054,7 @@ let rec unpack_term
 
 
 let unpack_xdr_value
-    ?pos ?len ?fast ?prefix ?mstring_factories ?(decode=[])
+    ?pos ?len ?fast ?prefix ?mstring_factories ?xv_version ?(decode=[])
     (str:string)
     ((_,t):xdr_type)
     (p:(string * xdr_type) list)
@@ -2000,7 +2064,7 @@ let unpack_xdr_value
      List.for_all (fun (n,t') -> StringSet.is_empty (fst t').params) p then
 
     fst(unpack_term 
-	  ?pos ?len ?fast ?prefix ?mstring_factories
+	  ?pos ?len ?fast ?prefix ?mstring_factories ?xv_version
 	  str t
 	  (fun n -> List.assoc n p)
 	  (fun n -> try Some(List.assoc n decode) with Not_found -> None)
@@ -2012,7 +2076,7 @@ let unpack_xdr_value
 
 
 let unpack_xdr_value_l
-    ?pos ?len ?fast ?prefix ?mstring_factories ?(decode=[])
+    ?pos ?len ?fast ?prefix ?mstring_factories ?xv_version ?(decode=[])
     (str:string)
     ((_,t):xdr_type)
     (p:(string * xdr_type) list)
@@ -2022,7 +2086,7 @@ let unpack_xdr_value_l
      List.for_all (fun (n,t') -> StringSet.is_empty (fst t').params) p then
 
     unpack_term
-      ?pos ?len ?fast ?prefix ?mstring_factories 
+      ?pos ?len ?fast ?prefix ?mstring_factories ?xv_version
       str t
       (fun n -> List.assoc n p)
       (fun n -> try Some(List.assoc n decode) with Not_found -> None)
