@@ -1,4 +1,4 @@
-(* $Id: xdr.mli 1562 2011-03-07 16:13:14Z gerd $ *)
+(* $Id: xdr.mli 1709 2012-02-16 01:17:04Z gerd $ *)
 
 (** External Data Representation *)
 
@@ -105,6 +105,22 @@ open Rtypes;;
  * to the preferred factory for managed strings (needs to be passed to the
  * XDR unpacker). Values for managed strings are objects of type
  * {!Xdr_mstring.mstring}.
+ *
+ * The term [X_direct(t,read,write,size)] is generated by ocamlrpcgen at 
+ * positions
+ * suitable for direct mapping. In this case, the XDR byte representation is
+ * directly mapped to the final Ocaml value bypassing the intermediate
+ * representation defined in this module. [t] is the mapped type. The
+ * function [read] is called as [read s cursor len] in order to map the
+ * XDR bytes at [!cursor] in [s]. The [cursor] must be advanced to the
+ * end position. [len] is the number of valid bytes in [s] (i.e. [s.(len-1)]
+ * is the last one). The result of [read] is an arbitrary exception which
+ * needs to be postprocessed by ocamlrpcgen-generated code. The function
+ * [write] is called as [write x s cursor]: The value encapsulated in
+ * exception [x] is written to string [s] at position [cursor]. Like for
+ * [read] the cursor is advanced by the number of written bytes. There
+ * must be enough space in [s]. The function [size] can be used to determine
+ * the number of written bytes beforehand.
  *)
 
 type xdr_type_term =
@@ -133,6 +149,10 @@ type xdr_type_term =
   | X_param of string
   | X_rec of (string * xdr_type_term)      (* define a recursive type *)
   | X_refer of string                      (* refer to a recursive type *)
+  | X_direct of xdr_type_term * 
+                (string -> int ref -> int -> exn) *
+                (exn -> string -> int ref -> unit) *
+                (exn -> int)
 ;;
 
 type xdr_type;;
@@ -215,7 +235,15 @@ type xdr_value =
           type of [X_string]
        *)
   | XV_mstring of Xdr_mstring.mstring
- 
+  (* New in 3.5: *)
+  | XV_direct of exn * int
+      (* The variant [XV_direct(exn,size)] should only be used by
+	 code generated with ocamlrpcgen. It is only allowed at positions
+	 in the term with type [X_direct]. It is assumed that the value
+	 is available as exception [exn] (in any form). [size] is the number
+	 of bytes in XDR format. 
+       *) 
+
   (* TODO: arrays of int, uint, hyper, uhyper, opaque, float, double *)
 
 val xv_true : xdr_value
@@ -404,23 +432,44 @@ val pack_xdr_value_as_mstrings :
  *)
 
 
+type xdr_value_version =
+    [ `V1 | `V2 | `V3 | `V4 | `Ocamlrpcgen ]
+  (** Selects which version of [xdr_value] is returned by [unpack_xdr_value].
+      During the development of Ocamlnet several incompatible changes were
+      made, but by selecting a certain version these changes can be hidden
+      from the caller.
+
+      - [`V1]: This version refers to the original [xdr_value] definition,
+        which only included [XV_int], [XV_uint], [XV_hyper], [XV_uhyper],
+        [XV_enum], [XV_float], [XV_double], [XV_opaque], [XV_string],
+        [XV_array], [XV_struct], [XV_union_over_int], [XV_union_over_uint],
+        [XV_union_over_enum], and [XV_void].
+      - [`V2]: This version is available since the [rpc-0.4] distribution,
+        and added the tags [XV_enum_fast], [XV_struct_fast], and
+        [XV_union_over_enum_fast].
+      - [`V3]: In Ocamlnet-3.0 the tag [XV_array_of_string_fast] was added.
+      - [`V4]: In Ocamlnet-3.5 the tag [XV_direct] was added.
+      - [`Ocamlrpcgen]: This refers to the version that must be used if the
+        returned [xdr_value] is processed by code generated with [ocamlrpcgen].
+
+      The default is still [`V1], for ultimate backward compatibility.
+      The switch [fast=true] selects [`Ocamlrpcgen] (it was always used
+      for this purpose, despite its name).
+   *)
 
 val unpack_xdr_value : ?pos:int -> ?len:int -> ?fast:bool -> ?prefix:bool ->
                        ?mstring_factories:Xdr_mstring.named_mstring_factories->
+                       ?xv_version:xdr_value_version ->
                        ?decode:(string * decoder) list ->
                        string -> xdr_type -> (string * xdr_type) list ->
                        xdr_value
 val unpack_xdr_value_l : ?pos:int -> ?len:int -> ?fast:bool -> ?prefix:bool ->
-                        ?mstring_factories:Xdr_mstring.named_mstring_factories->
+                       ?mstring_factories:Xdr_mstring.named_mstring_factories-> 
+                         ?xv_version:xdr_value_version ->
                          ?decode:(string * decoder) list ->
                          string -> xdr_type -> (string * xdr_type) list ->
                          (xdr_value * int)
-  (** [fast]: whether to prefer the new "fast" value representation
-   * (default: false).
-   * The [fast] flag {b must} be set to unpack for types that have been
-   * created with [ocamlrpcgen].
-   *
-   * [prefix]: whether it is ok that the string is longer than the message
+  (** [prefix]: whether it is ok that the string is longer than the message
    *   (default: false)
    *
    * [mstring_factories]: when a [T_mstring(name,_)] type is found, the
@@ -428,6 +477,14 @@ val unpack_xdr_value_l : ?pos:int -> ?len:int -> ?fast:bool -> ?prefix:bool ->
    * fails under the name ["*"]. If there is no such
    * factory, unpacking fails! (Default: empty table.)
    *
+   * [xv_version]: Selects a certain version of the returned [xdr_value]
+   * terms. See {!Xdr.xdr_value_version} for details. Set this to
+   * [`Ocamlrpcgen] if you decode the [xdr_value] with ocamlrpcgen-generated
+   * decoders.
+   *
+   * [fast]: Setting this to true is a deprecated way to set
+   * [xv_version=`Ocamlrpcgen].
+   *
    * The variant [unpack_xdr_value_l] returns not only the decoded value,
    * but also the actual length in bytes.
    *
@@ -442,3 +499,24 @@ val get_string_decoration_size : int -> Rtypes.uint4 -> int
   (* Returns the size of the "string decoration", i.e. the size of the
      header (always 4 bytes) plus the size of the padding.
    *)
+
+val sizefn_string : Rtypes.uint4 -> string -> int
+  (* Return the packed size of a string *)
+
+val sizefn_mstring : Rtypes.uint4 -> Xdr_mstring.mstring -> int
+  (* Return the packed size of an mstring *)
+
+val write_string_fixed : int -> string -> string -> int ref -> unit
+val write_string : string -> string -> int ref -> unit
+  (* [write_* s1 s2 p]: write the string s1 as XDR to s2 at position p *)
+
+val read_string_fixed : int -> string -> int ref -> int -> string
+val read_string : Rtypes.uint4 -> string -> int ref -> int -> string
+
+val raise_xdr_format_too_short : unit -> 'a
+val raise_xdr_format_value_not_included : unit -> 'a
+val raise_xdr_format_maximum_length : unit -> 'a
+val raise_xdr_format_undefined_descriminator : unit -> 'a
+
+val safe_add : int -> int -> int
+val safe_mul : int -> int -> int
