@@ -1,6 +1,6 @@
 /* Config for win32 */
 
-#undef HAVE__EXIT
+#define HAVE__EXIT
 #undef HAVE_SYSCONF
 #undef HAVE_GETPGID
 #undef HAVE_SETPGID
@@ -11,3 +11,6 @@
 #undef HAVE_GETSID
 #undef HAVE_SETREUID
 #undef HAVE_SETREGID
+#undef HAVE_FSYNC
+#undef HAVE_FDATASYNC
+#undef HAVE_POLL
