@@ -1,4 +1,4 @@
-/* $Id: netsys_c.c 1790 2012-07-17 14:14:14Z gerd $
+/* $Id: netsys_c.c 1886 2013-08-30 11:02:36Z gerd $
  * ----------------------------------------------------------------------
  */
 
@@ -444,11 +444,21 @@ static int at_flags_table[] = {
 #ifndef O_RSYNC
 #define O_RSYNC 0
 #endif
+#ifndef O_CLOEXEC
+#define NEED_CLOEXEC_EMULATION
+#define O_CLOEXEC 0
+#endif
 
 static int open_flag_table[] = {
   O_RDONLY, O_WRONLY, O_RDWR, O_NONBLOCK, O_APPEND, O_CREAT, O_TRUNC, O_EXCL, 
-  O_NOCTTY, O_DSYNC, O_SYNC, O_RSYNC, 0 /* O_SHARE_DELETE */
+  O_NOCTTY, O_DSYNC, O_SYNC, O_RSYNC, 0 /* O_SHARE_DELETE */, O_CLOEXEC
 };
+
+#ifdef NEED_CLOEXEC_EMULATION
+static int open_cloexec_table[] = {
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
+};
+#endif
 #endif
 
 CAMLprim value netsys_openat(value dirfd, value path, value flags, value perm)
@@ -467,6 +477,13 @@ CAMLprim value netsys_openat(value dirfd, value path, value flags, value perm)
     leave_blocking_section();
     stat_free(p);
     if (ret == -1) uerror("openat", path);
+#if defined(NEED_CLOEXEC_EMULATION) && defined(FD_CLOEXEC)
+    if (convert_flag_list(flags, open_cloexec_table) != 0) {
+        int flags = fcntl(fd, F_GETFD, 0);
+        if (flags == -1 || fcntl(fd, F_SETFD, flags | FD_CLOEXEC) == -1)
+          uerror("openat", path);
+    }
+#endif
     CAMLreturn (Val_int(ret));
 #else
     invalid_argument("Netsys_posix.openat not available");
