@@ -1,4 +1,4 @@
-/* $Id: netsys_c_mem.c 1789 2012-06-27 14:31:32Z gerd $ */
+/* $Id: netsys_c_mem.c 1826 2013-01-13 17:57:16Z gerd $ */
 
 #include "netsys_c.h"
 #include "netsys_c_htab.h"
@@ -12,6 +12,8 @@
 #endif
 #endif
 
+#undef DEBUG
+
 /**********************************************************************/
 /* Bigarray helpers                                                   */
 /**********************************************************************/
@@ -174,14 +176,13 @@ CAMLprim value netsys_map_file(value fdv,
 {
 #if defined(HAVE_MMAP) && defined(HAVE_SYSCONF) && !defined(_WIN32)
     int fd, shared;
-    off_t pos, savepos, eofpos, basize0;
+    off_t pos, eofpos, basize0;
           /* Att: pos might be 64 bit even on 32 bit systems! */
     struct stat st;
     void *addr, *eff_addr;
     intnat size;
     uintnat basize;
     int64 pos0;
-    char c;
     uintnat pagesize, delta;
 
     /* Avoid here seeking at all costs. On some systems, shared memory
@@ -673,9 +674,11 @@ int netsys_init_value_1(struct htab *t,
 			int enable_bigarrays, 
 			int enable_customs,
 			int enable_atoms,
+                        int enable_cc,
 			int simulation,
 			void *target_addr,
 			struct named_custom_ops *target_custom_ops,
+                        value cc,
 			int color,
 			intnat *start_offset,
 			intnat *bytelen
@@ -705,6 +708,14 @@ int netsys_init_value_1(struct htab *t,
     void *nativeint_target_ops;
     void *bigarray_target_ops;
 
+#ifdef DEBUG
+    fprintf(stderr,
+            "dest=%lx dest_end=%lx orig=%lx target=%lx en_b=%d en_c=%d en_a=%d sim=%d color=%d\n",
+            (intnat) dest, (intnat) dest_end, Long_val(orig), (intnat) target_addr,
+            enable_bigarrays, enable_customs, enable_atoms, simulation,
+            color);
+#endif
+
     copy = 0;
 
     dest_cur = dest;
@@ -712,11 +723,21 @@ int netsys_init_value_1(struct htab *t,
 
     if (dest_cur >= dest_end && !simulation) return (-4);   /* out of space */
 
-    if (!Is_block(orig)) return (-2);
+    if (!Is_block(orig)) {
+#ifdef DEBUG
+        fprintf(stderr, "DEBUG: Unboxed value\n");
+#endif
+        return (-2);
+    };
 
     orig_addr = (void *) orig;
     code = netsys_queue_add(q, orig_addr);
-    if (code != 0) return code;
+    if (code != 0) {
+#ifdef DEBUG
+        fprintf(stderr, "DEBUG: From netsys_queue_add: %d\n", code);
+#endif
+        return code;
+    }
 
     /* initialize *_target_ops */
     bigarray_target_ops = NULL;
@@ -747,12 +768,22 @@ int netsys_init_value_1(struct htab *t,
 
     code = netsys_queue_take(q, &work_addr);
     while (code != (-3)) {
-	if (code != 0) return code;
+	if (code != 0) {
+#ifdef DEBUG
+            fprintf(stderr, "DEBUG: From netsys_queue_take: %d\n", code);
+#endif
+            return code;
+        }
 
 	/* fprintf(stderr, "work_addr=%lx\n", (unsigned long) work_addr); */
 
 	code = netsys_htab_lookup(t, work_addr, &copy_addr);
-	if (code != 0) return code;
+	if (code != 0) {
+#ifdef DEBUG
+            fprintf(stderr, "DEBUG: From netsys_htab_lookup: %d\n", code);
+#endif
+            return code;
+        };
 
 	if (copy_addr == NULL) {
 	    /* The address is unknown, so copy the value */
@@ -770,12 +801,20 @@ int netsys_init_value_1(struct htab *t,
 		case Closure_tag:
 		case Lazy_tag:
 		case Forward_tag:
+#ifdef DEBUG                    
+                    fprintf(stderr, "DEBUG: Unsupported tag\n");
+#endif
 		    return (-2);   /* unsupported */
 		}
 
 		work_words = Wosize_hp(work_header);
 		if (work_words == 0) {
-		    if (!enable_atoms) return (-2);
+		    if (!enable_atoms) {
+#ifdef DEBUG
+                        fprintf(stderr, "DEBUG: Unsupported atom\n");
+#endif
+                        return (-2);
+                    }
 		    if (enable_atoms == 1) goto next;
 		};
 		
@@ -798,16 +837,41 @@ int netsys_init_value_1(struct htab *t,
 		/* Add the association (work_addr -> copy_addr) to t: */
 
 		code = netsys_htab_add(t, work_addr, copy_addr);
-		if (code < 0) return code;
+		if (code < 0) {
+#ifdef DEBUG
+                    fprintf(stderr, "DEBUG: From netsys_htab_add: %d\n", code);
+#endif
+                    return code;
+                }
 
 		/* Add the sub values of work_addr to q: */
 
 		for (i=0; i < work_words; ++i) {
 		    value field = Field(work, i);
 		    if (Is_block (field)) {
+                        if (enable_cc) {
+                            value cc_rest, cc_pair;
+                            intnat start, end;
+                            cc_rest = cc;
+                            while (Is_block(cc_rest)) {
+                                cc_pair = Field(cc_rest, 0);  /* List.hd */
+                                start = Long_val(Field(cc_pair,0)) << 1;
+                                end = Long_val(Field(cc_pair,1)) << 1;
+                                if (field >= start && field < end)
+                                    goto cont_for;
+                                cc_rest = Field(cc_rest, 1);  /* List.tl */
+                            }
+                        };
 			code = netsys_queue_add(q, (void *) field);
-			if (code != 0) return code;
+			if (code != 0) {
+#ifdef DEBUG
+                            fprintf(stderr, "DEBUG: From netsys_queue_add: %d\n", code);
+#endif
+                            return code;
+                        }
 		    }
+                cont_for:
+                    continue;
 		}
 	    }
 	    else {
@@ -819,6 +883,9 @@ int netsys_init_value_1(struct htab *t,
 		/* Check for bigarrays and other custom blocks */
 		switch (work_tag) {
 		case Abstract_tag:
+#ifdef DEBUG
+                    fprintf(stderr, "DEBUG: Unsupported abstract tag\n");
+#endif
 		    return(-2);
 		case String_tag:
 		    do_copy = 1; break;
@@ -836,7 +903,12 @@ int netsys_init_value_1(struct htab *t,
 			if (id[0] == '_') {
 			    switch (id[1]) {
 			    case 'b':
-				if (!enable_bigarrays) return (-2);
+				if (!enable_bigarrays) {
+#ifdef DEBUG
+                                    fprintf(stderr, "DEBUG: Unsupported bigarray\n");
+#endif
+                                    return (-2);
+                                };
 				if (strcmp(id, "_bigarray") == 0) {
 				    caml_id = 'b';
 				    break;
@@ -844,17 +916,29 @@ int netsys_init_value_1(struct htab *t,
 			    case 'i': /* int32 */
 			    case 'j': /* int64 */
 			    case 'n': /* nativeint */
-				if (!enable_customs) return (-2);
+				if (!enable_customs) {
+#ifdef DEBUG
+                                    fprintf(stderr, "DEBUG: Unsupported custom value\n");
+#endif
+                                    return (-2);
+                                };
 				if (id[2] == 0) {
 				    caml_id = id[1];
 				    break;
 				}
 			    default:
+#ifdef DEBUG
+                                fprintf(stderr, "DEBUG: Unknown custom _value\n");
+#endif
 				return (-2);
 			    }
 			}
-			else
+			else {
+#ifdef DEBUG
+                            fprintf(stderr, "DEBUG: Unknown custom value\n");
+#endif
 			    return (-2);
+                        }
 		    }
 		}; /* switch */
 
@@ -892,7 +976,12 @@ int netsys_init_value_1(struct htab *t,
 		    }
 		    
 		    code = netsys_htab_add(t, work_addr, copy_addr);
-		    if (code < 0) return code;
+		    if (code < 0) {
+#ifdef DEBUG
+                        fprintf(stderr, "DEBUG: From netsys_htab_add: %d\n", code);
+#endif
+                        return code;
+                    }
 		}
 
 		if (do_bigarray) {
@@ -928,6 +1017,9 @@ int netsys_init_value_1(struct htab *t,
 		       16M on 32 bit systems). The special representation
 		       is an Abstract_tag with zero length, followed
 		       by the real length (in words)
+
+                       NB. This convention is also understood by the
+                       GC algorithm in Netmcore_heap.
 		    */
 		    
 		    if (enable_bigarrays == 2) {
@@ -961,7 +1053,7 @@ int netsys_init_value_1(struct htab *t,
 			b_copy->data = data_copy;
 			b_copy->proxy = NULL;
 
-			/* If the copy is in our own buffer, it is
+			/* as the copy resides in our own buffer, it is
 			   now externally managed.
 			*/
 			b_copy->flags = 
@@ -979,14 +1071,19 @@ int netsys_init_value_1(struct htab *t,
     next:
 	code = netsys_queue_take(q, &work_addr);
     } /* while */
-    
+#ifdef DEBUG
+    fprintf(stderr, "End pass 1\n");    
+#endif
+
     /* Second pass. The copied blocks still have fields pointing to the
        original blocks. We fix that now by iterating once over the copied
        memory block.
     */
 
     if (!simulation) {
-	/* fprintf(stderr, "second pass\n"); */
+#ifdef DEBUG
+        fprintf(stderr, "second pass\n");
+#endif
 	dest_ptr = dest;
 	while (dest_ptr < dest_cur) {
 	    copy_header1 = *((header_t *) dest_ptr);
@@ -1001,7 +1098,12 @@ int netsys_init_value_1(struct htab *t,
 			/* It is a pointer. Try to fix it up. */
 			code = netsys_htab_lookup(t, (void *) field,
 						  &fixup_addr);
-			if (code != 0) return code;
+			if (code != 0) { 
+#ifdef DEBUG
+                            fprintf(stderr, "DEBUG: From netsys_htab_lookup (pass2): %d\n", code);
+#endif
+                            return code;
+                        }
 
 			if (fixup_addr != NULL)
 			    Field(copy,i) = 
@@ -1028,7 +1130,7 @@ int netsys_init_value_1(struct htab *t,
 }
 
 
-static int init_value_flags[] = { 1, 2, 4, 8 };
+static int init_value_flags[] = { 1, 2, 4, 8, 16, 32 };
 
 static struct htab *stat_tab = NULL;
 static struct nqueue *stat_queue = NULL;
@@ -1096,7 +1198,8 @@ value netsys_init_value(value memv,
 			value orig,  
 			value flags,
 			value targetaddrv,
-			value target_custom_ops
+			value target_custom_ops,
+                        value cc
 			)
 {
     int code;
@@ -1108,15 +1211,33 @@ value netsys_init_value(value memv,
     char *mem_end;
     intnat off;
     struct named_custom_ops *ops, *old_ops, *next_ops;
-    
+
+#ifdef DEBUG
+    fprintf(stderr,"Start netsys_init_value\n");    
+#endif
     code = prep_stat_tab();
-    if (code != 0) goto exit;
+    if (code != 0) {
+#ifdef DEBUG
+        fprintf(stderr, "DEBUG Bad prep_stat_tab\n");
+#endif
+        goto exit;
+    }
 
     code = prep_stat_queue();
-    if (code != 0) goto exit;
+    if (code != 0) {
+#ifdef DEBUG
+        fprintf(stderr, "DEBUG Bad prep_stat_queue\n");
+#endif
+        goto exit;
+    }
 
     off = Long_val(offv);
-    if (off % sizeof(void *) != 0) { code=(-2); goto exit; }
+    if (off % sizeof(void *) != 0) { 
+#ifdef DEBUG
+        fprintf(stderr, "DEBUG Bad alignment\n");
+#endif
+        code=(-2); goto exit; 
+    }
 
     cflags = caml_convert_flag_list(flags, init_value_flags);
     targetaddr = (void *) (Nativeint_val(targetaddrv) + off);
@@ -1145,10 +1266,14 @@ value netsys_init_value(value memv,
     code = netsys_init_value_1(stat_tab, stat_queue, mem_data, mem_end, orig, 
 			       (cflags & 1) ? 2 : 0, 
 			       (cflags & 2) ? 1 : 0, 
-			       (cflags & 4) ? 2 : 0,
+			       (cflags & 4) ? 2 : ((cflags & 32) ? 1 : 0),
+                               (cflags & 16) ? 1 : 0,
 			       cflags & 8,
-			       targetaddr, ops, 0,
+			       targetaddr, ops, cc, 0,
 			       &start_offset, &bytelen);
+#ifdef DEBUG
+    fprintf(stderr, "Done netsys_init_value code=%d\n", code);
+#endif
     if (code != 0) goto exit;
 
     unprep_stat_tab();
@@ -1175,6 +1300,9 @@ value netsys_init_value(value memv,
     case (-1):
 	unix_error(errno, "netsys_init_value", Nothing);
     case (-2):
+#ifdef DEBUG
+        fprintf(stderr, "Lib err\n");
+#endif
 	failwith("Netsys_mem.init_value: Library error");
     case (-4):
 	caml_raise_constant(*caml_named_value("Netsys_mem.Out_of_space"));
@@ -1188,7 +1316,7 @@ value netsys_init_value_bc(value * argv, int argn)
 {
     return
 	netsys_init_value(argv[0], argv[1], argv[2], argv[3],
-			  argv[4], argv[5]);
+			  argv[4], argv[5], argv[6]);
 }
 
 
@@ -1243,8 +1371,9 @@ value netsys_copy_value(value flags, value orig)
 			       (cflags & 1) ? 1 : 0,  /* enable_bigarrays */
 			       (cflags & 2) ? 1 : 0,  /* enable_customs */
 			       1, /* enable_atoms */
+                               0, /* enable_cc */
 			       1, /* simulate */
-			       NULL, NULL, 0, &start_offset, &bytelen);
+			       NULL, NULL, 0, 0, &start_offset, &bytelen);
     if (code != 0) goto exit;
 
     /* fprintf (stderr, "done counting bytelen=%ld\n", bytelen); */
@@ -1310,8 +1439,9 @@ value netsys_copy_value(value flags, value orig)
 			       (cflags & 1) ? 1 : 0,  /* enable_bigarrays */
 			       (cflags & 2) ? 1 : 0,  /* enable_customs */
 			       1, /* enable_atoms */
+                               0, /* enable_cc */
 			       0, /* simulate */
-			       dest, &bigarray_ops, color,
+			       dest, &bigarray_ops, 0, color,
 			       &start_offset, &bytelen);
     if (code != 0) goto exit;
 
@@ -1382,3 +1512,19 @@ value netsys_get_custom_ops (value v)
 
     CAMLreturn(r);
 }
+
+
+value netsys_is_bigarray(value v) {
+    struct custom_operations *custom_ops;
+    CAMLparam1(v);
+    CAMLlocal1(r);
+
+    if (Is_block(v) && Tag_val(v) == Custom_tag) {
+	custom_ops = Custom_ops_val(v);
+        r = Val_bool(strcmp(custom_ops->identifier, "_bigarray")==0);
+    }
+    else
+        r = Val_bool(0);
+
+    CAMLreturn(r);
+}
