@@ -1,4 +1,4 @@
-/* $Id: netsys_c_spawn.c 1694 2012-02-06 17:14:51Z gerd $ */
+/* $Id: netsys_c_spawn.c 1749 2012-03-15 16:18:30Z gerd $ */
 
 #include "netsys_c.h"
 
@@ -511,6 +511,7 @@ CAMLprim value netsys_posix_spawn_nat(value v_pg,
     value return_value;
     int code;
     short flags;
+    int   use_fork_exec;
 
     pid_t pid;
     char **sub_argv;
@@ -519,6 +520,7 @@ CAMLprim value netsys_posix_spawn_nat(value v_pg,
     int cleanup_sub_env;;
 
     posix_spawn_file_actions_t fd_actions;
+    int n_fd_actions;
     posix_spawnattr_t attr;
     int cleanup_fd_actions;
     int cleanup_attr;
@@ -553,6 +555,7 @@ CAMLprim value netsys_posix_spawn_nat(value v_pg,
     uerror_function = "<uninit>";
     flags = 0;
     fd_known = 0;
+    use_fork_exec = 0;
 
     nofile = sysconf(_SC_OPEN_MAX);
 
@@ -583,6 +586,7 @@ CAMLprim value netsys_posix_spawn_nat(value v_pg,
 
     /* Init fd_actions */
     code = posix_spawn_file_actions_init(&fd_actions);
+    n_fd_actions = 0;
     if (code != 0) 
 	MAIN_ERROR(code, "netsys_posix_spawn/posix_spawn_file_actions_init");
     cleanup_fd_actions = 1;
@@ -680,15 +684,6 @@ CAMLprim value netsys_posix_spawn_nat(value v_pg,
 	v_fd_actions_hd = Field(v_fd_actions_l, 0);
 	v_fd_actions_l  = Field(v_fd_actions_l, 1);
 	switch(Tag_val(v_fd_actions_hd)) {
-	case 0: /* Fda_close */
-	    fd1 = Int_val(Field(v_fd_actions_hd, 0));
-	    if (fd1 != fd_known) {
-		code = posix_spawn_file_actions_addclose(&fd_actions, fd1);
-		if (code != 0)
-		    MAIN_ERROR(code, 
-			       "netsys_posix_spawn/psfa_addclose [1]");
-	    };
-	    break;
 	case 1: /* Fda_close_ignore */
 	    /* We translate this into a dup2 + close */
 	    fd1 = Int_val(Field(v_fd_actions_hd, 0));
@@ -698,10 +693,22 @@ CAMLprim value netsys_posix_spawn_nat(value v_pg,
 		    MAIN_ERROR(code, 
 			       "netsys_posix_spawn/psfa_adddup2 [1]");
 		
+		n_fd_actions++;
 		code = posix_spawn_file_actions_addclose(&fd_actions, fd1);
 		if (code != 0)
 		    MAIN_ERROR(code, 
 			       "netsys_posix_spawn/psfa_addclose [2]");
+		n_fd_actions++;
+	    };
+	    break;
+	case 0: /* Fda_close */
+	    fd1 = Int_val(Field(v_fd_actions_hd, 0));
+	    if (fd1 != fd_known) {
+		code = posix_spawn_file_actions_addclose(&fd_actions, fd1);
+		if (code != 0)
+		    MAIN_ERROR(code, 
+			       "netsys_posix_spawn/psfa_addclose [1]");
+		n_fd_actions++;
 	    };
 	    break;
 	case 2:  /* Fda_close_except */
@@ -716,12 +723,14 @@ CAMLprim value netsys_posix_spawn_nat(value v_pg,
 			    MAIN_ERROR
 				(code, 
 				 "netsys_posix_spawn/psfa_actions_adddup2 [2]");
+			n_fd_actions++;
 			code = posix_spawn_file_actions_addclose
 			    (&fd_actions, k);
 			if (code != 0)
 			    MAIN_ERROR
 				(code, 
 				 "netsys_posix_spawn/psfa_addclose [3]");
+			n_fd_actions++;
 		    }
 		}
 	    }
@@ -735,12 +744,24 @@ CAMLprim value netsys_posix_spawn_nat(value v_pg,
 	    code = posix_spawn_file_actions_adddup2(&fd_actions, fd1, fd2);
 	    if (code != 0)
 		MAIN_ERROR(code, "netsys_posix_spawn/psfa_actions_adddup2 [3]");
+	    n_fd_actions++;
 	    break;
 	default:
 	    MAIN_ERROR(EINVAL, "netsys_posix_spawn/assert_fd");
 	};
     };
 
+#ifdef __APPLE__
+    /* MacOS limits the number of file actions arbitrarily to the max number
+       of open files. We catch this here, and cause a failure "USE_FORK_EXEC",
+       so the alternate fork/exec-based spawn will be used instead.
+    */
+    if (n_fd_actions > nofile) {
+	use_fork_exec = 1;
+	goto main_exit;
+    };
+#endif
+    
 
     code = posix_spawn(&pid,
 		       String_val(v_cmd),
@@ -782,6 +803,9 @@ main_exit:
     if (uerror_errno != 0)
 	unix_error(uerror_errno, uerror_function, Nothing);
 
+    if (use_fork_exec != 0)
+	failwith("USE_FORK_EXEC");
+
     return return_value;
 
 #else
