@@ -1,4 +1,4 @@
-(* $Id: netsys_posix.ml 1790 2012-07-17 14:14:14Z gerd $ *)
+(* $Id: netsys_posix.ml 1885 2013-08-30 10:13:41Z gerd $ *)
 
 open Unix
 open Printf
@@ -702,6 +702,12 @@ let run_post_fork_handlers() =
 type at_flag = AT_EACCESS | AT_SYMLINK_NOFOLLOW | AT_REMOVEDIR
 
 (* The stubs assume these type definitions: *)
+IFDEF HAVE_O_CLOEXEC THEN
+type open_flag1 = Unix.open_flag =
+    O_RDONLY | O_WRONLY | O_RDWR | O_NONBLOCK | O_APPEND | O_CREAT | O_TRUNC
+  | O_EXCL | O_NOCTTY | O_DSYNC | O_SYNC | O_RSYNC | O_SHARE_DELETE
+  | O_CLOEXEC
+ELSE
 IFDEF HAVE_O_SHARE_DELETE THEN
 type open_flag1 = Unix.open_flag =
     O_RDONLY | O_WRONLY | O_RDWR | O_NONBLOCK | O_APPEND | O_CREAT | O_TRUNC
@@ -711,6 +717,7 @@ type open_flag1 = Unix.open_flag =
     O_RDONLY | O_WRONLY | O_RDWR | O_NONBLOCK | O_APPEND | O_CREAT | O_TRUNC
   | O_EXCL | O_NOCTTY | O_DSYNC | O_SYNC | O_RSYNC
 ENDIF
+ENDIF
 
 type access_permission1 = Unix.access_permission =
     R_OK | W_OK | X_OK | F_OK
