@@ -1,15 +1,18 @@
-(* $Id: rpc_auth_local.ml 915 2006-06-15 00:12:40Z gerd $
+(* $Id: rpc_auth_local.ml 1671 2011-09-22 22:25:06Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
 
 external get_peer_credentials : Unix.file_descr -> (int * int)
-  = "unix_get_peer_credentials"
+  = "netsys_get_peer_credentials"
 ;;
 
+(*
 external peek_peer_credentials : Unix.file_descr -> (int * int)
-  = "unix_peek_peer_credentials"
+  = "netsys_peek_peer_credentials"
 ;;
+*
+ *)
 
 
 class server_auth_method : Rpc_server.auth_method =
@@ -19,11 +22,11 @@ object
   method peek =
     `Peek_descriptor
       (fun d ->
-	 match Unix.getpeername d with
+	 match Netsys.getpeername d with
 	     Unix.ADDR_UNIX _ ->
 	       (* Try now peek_peer_credentials: *)
 	       begin try
-		 let uid, gid = peek_peer_credentials d in
+		 let uid, gid = get_peer_credentials d in
 		 let username =
 		   string_of_int uid ^ "." ^ string_of_int gid ^ "@localhost" in
 		 Some username
@@ -45,7 +48,7 @@ object
 	       None
       )
 
-  method authenticate _ _ _ _ _ _ _ _ _ = ()
+  method authenticate _ _ _ _ = ()
 
 end
 
