@@ -1,4 +1,4 @@
-(* $Id: rename.ml 258 2004-05-25 16:49:11Z gerd $
+(* $Id: rename.ml 1873 2013-07-31 19:11:09Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -25,8 +25,9 @@ let name_mapping
       | T_array_unlimited t' -> check_type t'
       | T_enum l ->
 	  List.iter (fun (id,_) -> rename_enum id) l
-      | T_struct tdl ->
-	  List.iter (fun td -> rename_struct_component td.decl_symbol) tdl;
+      | T_struct (opts,tdl) ->
+          if not (List.mem `Tuple opts) then
+	    List.iter (fun td -> rename_struct_component td.decl_symbol) tdl;
 	  List.iter (fun td -> check_type td.decl_type) tdl;
       | T_union u ->
 	  let decls =
