@@ -1,4 +1,4 @@
-(* $Id: rpc_packer.ml 1062 2006-12-17 20:17:36Z gerd $
+(* $Id: rpc_packer.ml 1555 2011-03-03 01:18:51Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -13,12 +13,13 @@ open Rpc_common
 (* declaration of the RPC message type system: *)
 
 let auth_flavor_of_pos =
-  (* map XV_enum_fast-style positions to names *)
+  (* map XV_enum_fast-style _positions_ to names *)
   function
       0 -> "AUTH_NONE"
     | 1 -> "AUTH_SYS"
     | 2 -> "AUTH_SHORT"
     | 3 -> "AUTH_DH"
+    | 4 (* sic *) -> "RPCSEC_GSS"
     | _ -> assert false
 ;;
 
@@ -27,10 +28,10 @@ let rpc_ts_unvalidated =
   [ "auth_flavor",         X_enum [ "AUTH_NONE",     int4_of_int 0;
 				       (* also known as AUTH_NULL *)
 				    "AUTH_SYS",      int4_of_int 1;
-				       (* also known as AUTH_SYS *)
 				    "AUTH_SHORT",    int4_of_int 2;
 				    "AUTH_DH",       int4_of_int 3;
 				       (* also known as AUTH_DES *)
+				    "RPCSEC_GSS",    int4_of_int 6;
 				  ];
 
     "opaque_auth",         X_struct [ "flavor", X_type "auth_flavor";
@@ -60,6 +61,8 @@ let rpc_ts_unvalidated =
 				    "AUTH_TOOWEAK",       int4_of_int 5;
 				    "AUTH_INVALIDRESP",   int4_of_int 6;
 				    "AUTH_FAILED",        int4_of_int 7;
+				    "RPCSEC_GSS_CREDPROBLEM", int4_of_int 13;
+				    "RPCSEC_GSS_CTXPROBLEM",  int4_of_int 14;
 				  ];
 
     "call_body",           X_struct [ "rpcvers", X_uint;
@@ -78,6 +81,13 @@ let rpc_ts_unvalidated =
 				      "verf",    X_type "opaque_auth";
 				      	(* "param",   X_param "in" *) ];
 
+    "call_body_frame_up_to_cred", 
+                           X_struct [ "rpcvers", X_uint;
+				      "prog",    X_uint;
+				      "vers",    X_uint;
+				      "proc",    X_uint;
+				      "cred",    X_type "opaque_auth" ];
+
     "accepted_reply",      X_struct [ "verf",    X_type "opaque_auth";
 				      "reply_data",
 				      X_union_over_enum
@@ -119,6 +129,16 @@ let rpc_ts_unvalidated =
 					    (* "REPLY", X_type "reply_body"*) ],
 					 None) ];
 
+    "rpc_msg_call_frame_up_to_cred",
+                           X_struct [ "xid",  X_uint;
+				      "body",
+				      X_union_over_enum
+					((X_type "msg_type"),
+					 [ "CALL",  
+					     X_type "call_body_frame_up_to_cred"
+					 ],
+					 None) ];
+
     "rpc_msg_call_body",   X_param "in";
 
   ]
@@ -140,6 +160,11 @@ let rpc_msg_call_frame =
         expanded_xdr_type rpc_ts (X_type "rpc_msg_call_frame")
     )
 
+let rpc_msg_call_frame_up_to_cred =
+    ( let rpc_ts = validate_xdr_type_system rpc_ts_unvalidated in
+        expanded_xdr_type rpc_ts (X_type "rpc_msg_call_frame_up_to_cred")
+    )
+
 let rpc_msg_call_body =
     ( let rpc_ts = validate_xdr_type_system rpc_ts_unvalidated in
         expanded_xdr_type rpc_ts (X_type "rpc_msg_call_body")
@@ -150,12 +175,12 @@ let valid_void = validate_xdr_type X_void
 (****)
 
 type packed_value =
-    PV_rm of string  (* The packed value with record mark at beginning *)
-  | PV of string     (* The pure packed value *)
+  | PV of string                       (* as simple string *)
+  | PV_ms of Xdr_mstring.mstring list  (* as concatenation of mstrings *)
 
 (****)
 
-let pack_call prog xid proc flav_cred data_cred flav_verf data_verf
+let pack_call ?encoder prog xid proc flav_cred data_cred flav_verf data_verf
               proc_parm =
 
   let prog_nr = Rpc_program.program_number prog in
@@ -188,26 +213,72 @@ let pack_call prog xid proc flav_cred data_cred flav_verf data_verf
             )
        |]) in
 
-  PV_rm
-    (pack_xdr_value_as_string
-       ~rm:true
+  let encode =
+    match encoder with
+      | None -> []
+      | Some e -> [ "in", e ] in
+
+  PV_ms
+    (pack_xdr_value_as_mstrings
+       ~encode
        message_v            (* the value to pack *)
        message_t            (* the message type... *)
        [ "in", in_t;        (* ...instantiated with input type...*)
 	 "out", out_t ]     (* ...and output type *)
     )
 
+
+let pack_call_gssapi_header prog xid proc flav_cred data_cred =
+
+  let prog_nr = Rpc_program.program_number prog in
+  let vers_nr = Rpc_program.version_number prog in
+  let proc_nr, in_t, out_t = Rpc_program.signature prog proc in
+
+  let message_t = rpc_msg_call_frame_up_to_cred in
+
+  let message_v =                            (* value of the message *)
+    (XV_struct_fast
+       [| (* xid *)  XV_uint xid;
+	  (* body *) XV_union_over_enum_fast
+	  ( (* CALL *)
+	    0,
+	    XV_struct_fast
+	      [| (* rpcvers *) XV_uint (uint4_of_int 2);
+		 (* prog *)    XV_uint prog_nr;
+		 (* vers *)    XV_uint vers_nr;
+		 (* proc *)    XV_uint proc_nr;
+		 (* cred *)    XV_struct_fast
+			         [| (* flavor *) XV_enum flav_cred;
+				    (* Body *)   XV_opaque data_cred
+				 |];
+	      |]
+            )
+       |]) in
+
+  PV_ms
+    (pack_xdr_value_as_mstrings
+       message_v            (* the value to pack *)
+       message_t            (* the message type... *)
+       [ "in", in_t;        (* ...instantiated with input type...*)
+	 "out", out_t ]     (* ...and output type *)
+    )
+
+
 (****)
 
-let unpack_call_frame_l pv =
+let unpack_call_frame_l  pv =
   let message_t = rpc_msg_call_frame in
 
   let message_v, len =
     match pv with
 	PV octets ->
-	  unpack_xdr_value_l ~fast:true ~prefix:true octets message_t []
-      | PV_rm octets ->
-	  unpack_xdr_value_l ~pos:4 ~fast:true ~prefix:true octets message_t []
+	  unpack_xdr_value_l
+	    ~fast:true ~prefix:true octets message_t []
+      | PV_ms mstrings ->
+	  (* FIXME: There is no faster method than this right now: *)
+	  let octets = Xdr_mstring.concat_mstrings mstrings in
+	  unpack_xdr_value_l
+	    ~fast:true ~prefix:true octets message_t []
   in
 
   match message_v with
@@ -254,21 +325,28 @@ let unpack_call_frame octets =
 
 (****)
 
-let unpack_call_body prog proc pv pos =
+let unpack_call_body ?mstring_factories ?decoder prog proc pv pos =
   let proc_nr, in_t, out_t = Rpc_program.signature prog proc in
 
   let message_t = rpc_msg_call_body in
 
-  let octets, rm_pos =
+  let octets =
     match pv with
-	PV octets -> (octets, pos)
-      | PV_rm octets -> (octets, pos+4)
+	PV octets -> octets
+      | PV_ms mstrings -> Xdr_mstring.concat_mstrings mstrings
   in
 
+  let decode =
+    match decoder with
+      | None -> []
+      | Some d -> [ "in", d ] in
+
   let message_v =                            (* unpack the value *)
     unpack_xdr_value
-      ~pos:rm_pos
+      ~pos
       ~fast:true
+      ?mstring_factories
+      ~decode
       octets                                 (* XDR encoded value *)
       message_t                              (* generic type *)
       [ "in", in_t ]                         (* instance for "in" *)
@@ -278,15 +356,45 @@ let unpack_call_body prog proc pv pos =
 ;;
 
 
+let unpack_call_body_raw pv pos =
+  let octets =
+    match pv with
+	PV octets -> octets
+      | PV_ms mstrings -> Xdr_mstring.concat_mstrings mstrings
+  in
+
+  String.sub octets pos (String.length octets - pos)
+
+
 (****)
 
-let unpack_call prog proc pv =
+let extract_call_gssapi_header pv =
+  let octets = 
+    match pv with
+      | PV s -> s
+      | PV_ms mstrings -> (* FIXME *)
+	  Xdr_mstring.concat_mstrings mstrings in
+  (* The first 7 words have constant length. The 8th word contains the
+     length of the rest (the data part of cred)
+   *)
+  if String.length octets < 32 then
+    failwith "Rpc_packer.extract_call_gssapi_header: too short";
+  let n =
+    Rtypes.int_of_uint4 (Rtypes.read_uint4 octets 28) in
+  let m = if n land 3 = 0 then n else n+4-(n land 3) in
+  32 + m
+
+
+(****)
+
+let unpack_call ?mstring_factories ?decoder prog proc pv =
   (* compatibility *)
   let (xid, prog_nr, vers_nr, proc_nr,
        flav_cred, data_cred,
        flav_verf, data_verf,
        len) = unpack_call_frame_l pv in
-  let proc_parm = unpack_call_body prog proc pv len in
+  let proc_parm = 
+    unpack_call_body ?mstring_factories ?decoder prog proc pv len in
   (xid, prog_nr, vers_nr, proc_nr,
    flav_cred, data_cred,
    flav_verf, data_verf,
@@ -294,7 +402,8 @@ let unpack_call prog proc pv =
 
 (****)
 
-let pack_successful_reply prog proc xid flav_verf data_verf return_value =
+let pack_successful_reply ?encoder
+       prog proc xid flav_verf data_verf return_value =
 
   let proc_nr, in_t, out_t = Rpc_program.signature prog proc in
 
@@ -319,14 +428,57 @@ let pack_successful_reply prog proc xid flav_verf data_verf return_value =
 		  |] ))
        |] ) in
 
-  PV_rm
-    (pack_xdr_value_as_string
-       ~rm:true
+  let encode =
+    match encoder with
+      | None -> []
+      | Some e -> [ "out", e ] in
+
+  PV_ms
+    (pack_xdr_value_as_mstrings
+       ~encode
        message_v            (* the value to pack *)
        message_t            (* the message type... *)
        [ "in", in_t;        (* ...instantiated with input type...*)
 	 "out", out_t ]     (* ...and output type *)
     )
+
+(****)
+
+let t_void = validate_xdr_type X_void
+
+let pack_successful_reply_raw
+       xid flav_verf data_verf return_data =
+
+  let message_t = rpc_msg in      (* type of generic message *)
+
+  let message_v =                            (* value of the message *)
+    (XV_struct_fast
+       [| (* xid *)  XV_uint xid;
+	  (* body *) XV_union_over_enum_fast
+	  ( (* REPLY *)
+	    1,
+	    XV_union_over_enum_fast
+	      ( (* MSG_ACCEPTED *)
+		0,
+		XV_struct_fast
+		  [| (* verf *)
+		       XV_struct_fast [| (* flavor *) XV_enum flav_verf;
+	 		                 (* body *)   XV_opaque data_verf |];
+		     (* reply_data *)
+		       XV_union_over_enum_fast
+			 ( (* SUCCESS *) 0, XV_void)
+		  |] ))
+       |] ) in
+
+  PV_ms
+    (pack_xdr_value_as_mstrings
+       message_v            (* the value to pack *)
+       message_t            (* the message type... *)
+       [ "in", t_void;      (* dummy *)
+	 "out", t_void ]    (* ...and output type *)
+     @ return_data
+    )
+
 (****)
 
 let pack_accepting_reply xid flav_verf data_verf condition =
@@ -363,9 +515,8 @@ let pack_accepting_reply xid flav_verf data_verf condition =
 		   |] ))
        |] ) in
 
-  PV_rm
-    (pack_xdr_value_as_string
-       ~rm:true
+  PV_ms
+    (pack_xdr_value_as_mstrings
        message_v            (* the value to pack *)
        message_t            (* the message type... *)
        [ "in", valid_void;      (* ...instantiated with input type...*)
@@ -395,6 +546,8 @@ let pack_rejecting_reply xid condition =
                             XV_enum_fast 5 (* AUTH_INVALIDRESP *)
     | Auth_failed        -> (* AUTH_ERROR *) 1,
                             XV_enum_fast 6 (* AUTH_FAILED *)
+    | RPCSEC_GSS_credproblem ->  1, XV_enum_fast 13
+    | RPCSEC_GSS_ctxproblem ->   1, XV_enum_fast 14
   in
 
 (*
@@ -414,9 +567,8 @@ let pack_rejecting_reply xid condition =
 		1,
 		XV_union_over_enum_fast (case, explanation)))|]) in
 
-  PV_rm
-    (pack_xdr_value_as_string
-       ~rm:true
+  PV_ms
+    (pack_xdr_value_as_mstrings
        message_v            (* the value to pack *)
        message_t            (* the message type... *)
        [ "in", valid_void;        (* ...instantiated with input type...*)
@@ -425,22 +577,28 @@ let pack_rejecting_reply xid condition =
 
 (****)
 
-let unpack_reply prog proc pv =
+let unpack_reply ?mstring_factories ?decoder prog proc pv =
 
   let proc_nr, in_t, out_t = Rpc_program.signature prog proc in
 
   let message_t = rpc_msg in      (* type of generic message *)
 
-  let (octets, pos) =
+  let octets =
     match pv with
-	PV octets -> (octets, 0)
-      | PV_rm octets -> (octets, 4)
+	PV octets -> octets
+      | PV_ms mstrings -> Xdr_mstring.concat_mstrings mstrings
   in
 
+  let decode =
+    match decoder with
+      | None -> []
+      | Some d -> [ "out", d ] in
+
   let message_v =                            (* unpack the value *)
     unpack_xdr_value
       ~fast:true
-      ~pos
+      ?mstring_factories
+      ~decode
       octets                                 (* XDR encoded value *)
       message_t                              (* generic type *)
       [ "in", in_t;                          (* instance for "in" *)
@@ -516,6 +674,10 @@ let unpack_reply prog proc pv =
 			  raise (Rpc_server Auth_invalid_resp)
 		      | XV_enum_fast 6 ->
 			  raise (Rpc_server Auth_failed)
+		      | XV_enum_fast 13 ->
+			  raise (Rpc_server RPCSEC_GSS_credproblem)
+		      | XV_enum_fast 14 ->
+			  raise (Rpc_server RPCSEC_GSS_ctxproblem)
 		  end
 	  end
     with
@@ -531,17 +693,16 @@ let unpack_reply_verifier prog proc pv =
 
   let message_t = rpc_msg in      (* type of generic message *)
 
-  let (octets, pos) =
+  let octets =
     match pv with
-	PV octets -> (octets, 0)
-      | PV_rm octets -> (octets, 4)
+	PV octets -> octets
+      | PV_ms mstrings -> Xdr_mstring.concat_mstrings mstrings
   in
 
   let message_v =                            (* unpack the value *)
     unpack_xdr_value
       ~fast:true
       ~prefix:true
-      ~pos
       octets                                 (* XDR encoded value *)
       message_t                              (* generic type *)
       [ "in", valid_void;                    (* instance for "in" *)
@@ -584,39 +745,52 @@ let peek_xid pv =
   match pv with
       PV octets ->
 	if String.length octets < 4 then
-	  failwith "peek_xid: message too short";
+	  failwith "peek_xid: message too short [1]";
 
 	Rtypes.mk_uint4 (octets.[0], octets.[1], octets.[2], octets.[3])
 
-    | PV_rm octets ->
-	if String.length octets < 8 then
-	  failwith "peek_xid: message too short";
+    | PV_ms mstrings ->
+	if Xdr_mstring.length_mstrings mstrings < 4 then
+	  failwith "peek_xid: message too short [2]";
 
-	Rtypes.mk_uint4 (octets.[4], octets.[5], octets.[6], octets.[7])
+	let s = 
+	  Xdr_mstring.prefix_mstrings mstrings 4 in
+
+	Rtypes.mk_uint4 (s.[0], s.[1], s.[2], s.[3])
 
 (*****)
 
 let peek_auth_error pv =
-  let octets, offset =
+  let len =
     match pv with
-	PV octets -> (octets, 0)
-      | PV_rm octets -> (octets, 4)
-  in
+	PV octets -> String.length octets
+      | PV_ms mstrings -> Xdr_mstring.length_mstrings mstrings in
 
-  if String.length octets <> 20+offset ||
-     String.sub octets (4+offset) 12 <> "\000\000\000\001\000\000\000\001\000\000\000\001"
-  then
+  if len <> 20 then
     None
-  else
-    match String.sub octets (16+offset) 4 with
-	"\000\000\000\001" -> Some Auth_bad_cred
-      | "\000\000\000\002" -> Some Auth_rejected_cred
-      | "\000\000\000\003" -> Some Auth_bad_verf
-      | "\000\000\000\004" -> Some Auth_rejected_verf
-      | "\000\000\000\005" -> Some Auth_too_weak
-      | "\000\000\000\006" -> Some Auth_invalid_resp
-      | "\000\000\000\007" -> Some Auth_failed
-      | _                  -> None
+  else (
+    let octets =
+      match pv with
+	  PV octets -> octets
+	| PV_ms mstrings -> Xdr_mstring.concat_mstrings mstrings in  
+
+    if String.sub octets 4 12 <> 
+          "\000\000\000\001\000\000\000\001\000\000\000\001"
+    then
+      None
+    else
+      match String.sub octets 16 4 with
+	  "\000\000\000\001" -> Some Auth_bad_cred
+	| "\000\000\000\002" -> Some Auth_rejected_cred
+	| "\000\000\000\003" -> Some Auth_bad_verf
+	| "\000\000\000\004" -> Some Auth_rejected_verf
+	| "\000\000\000\005" -> Some Auth_too_weak
+	| "\000\000\000\006" -> Some Auth_invalid_resp
+	| "\000\000\000\007" -> Some Auth_failed
+	| "\000\000\000\013" -> Some RPCSEC_GSS_credproblem
+	| "\000\000\000\014" -> Some RPCSEC_GSS_ctxproblem
+	| _                  -> None
+  )
 ;;
 
 (*****)
@@ -624,19 +798,31 @@ let peek_auth_error pv =
 let length_of_packed_value pv =
   match pv with
       PV octets -> String.length octets
-    | PV_rm octets -> String.length octets - 4
+    | PV_ms mstrings -> Xdr_mstring.length_mstrings mstrings
 ;;
 
 let string_of_packed_value pv =
   match pv with
       PV octets -> octets
-    | PV_rm octets -> String.sub octets 4 (String.length octets - 4)
+    | PV_ms mstrings -> Xdr_mstring.concat_mstrings mstrings
 ;;
 
 let packed_value_of_string s = PV s;;
 
-let rm_string_of_packed_value pv =
+let packed_value_of_mstrings mstrings = PV_ms mstrings
+
+let mstrings_of_packed_value pv =
   match pv with
-      PV octets -> (String.make 4 '\000') ^ octets
-    | PV_rm octets -> octets
+    | PV octets -> 
+	[ Xdr_mstring.string_based_mstrings # create_from_string
+	    octets 0 (String.length octets) false ]
+    | PV_ms mstrings -> 
+	mstrings
 ;;
+
+let prefix_of_packed_value pv n =
+  match pv with
+    | PV octets ->
+	String.sub octets 0 n
+    | PV_ms ms ->
+	Xdr_mstring.prefix_mstrings ms n
