@@ -1,4 +1,4 @@
-(* $Id: rpc_server.mli 986 2006-08-28 14:12:35Z gerd $
+(* $Id: rpc_server.mli 1651 2011-08-03 16:38:17Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -69,7 +69,11 @@ type connector =
 	(** The service is installed on a Unix domain socket.
 	 * Note: the socket path must not exist when the server is started,
 	 * and the socket must be unlinked when the server terminates.
+         * Note Win32: Unix domain sockets are emulated by writing the
+         * inet4 port number into a one-line file.
 	 *)
+  | W32_pipe of string
+      (** The service is installed for a named pipe. (Only for Win32.) *)
   | Descriptor of Unix.file_descr
 	(** The service listens on the given file descriptor. *)
   | Dynamic_descriptor of (unit -> Unix.file_descr)
@@ -93,6 +97,12 @@ type binding =
   | Async of binding_async   (** bind an asynchonous procedure *)
 
 
+val connector_of_sockaddr : Unix.sockaddr -> connector
+  (** Converts the socket address into a connector *)
+
+val connector_of_socksymbol : Netsockaddr.socksymbol -> connector
+  (** Converts the {!Netsockaddr.socksymbol} into a connector *)
+
 val create :
     ?program_number:uint4 ->  (* Override program number *)
     ?version_number:uint4 ->  (* Override version number *)
@@ -147,6 +157,9 @@ val create :
     *   terminate if they see an EOF on the stream; in this case the stream
     *   is closed by the server
     * - the [create] function may block if the connector is Portmapped
+    *
+    * {b Note for UDP servers:} Due to limitations of the ocaml runtime
+    * there is a limit of 16K per message.
     *)
 
 class type socket_config =
@@ -165,6 +178,7 @@ type mode2 =
     [ `Socket_endpoint of protocol * Unix.file_descr
     | `Multiplexer_endpoint of Rpc_transport.rpc_multiplex_controller
     | `Socket of protocol * connector * socket_config
+    | `Dummy of protocol
     ]
   (** Determines the type of the server for [create2]:
     *
@@ -179,6 +193,9 @@ type mode2 =
     *   according to [conn]. [proto] determines the 
     *   encapsulation; should be [Tcp] for stream sockets and [Udp] for
     *   datagram sockets. [config] specifies configuration details.
+    *
+    * Despite their names, [`Socket_endpoint] and [`Socket] also support
+    * Win32 named pipes.
    *)
 
 val create2 : 
@@ -228,13 +245,15 @@ val get_xid : session -> Rtypes.uint4
 val get_socket_name : session -> Unix.sockaddr
 val get_peer_name : session -> Unix.sockaddr
   (** Return the address of the socket serving the session, and the client
-   * socket, resp.
+   * socket, resp. These functions fail if the server is not running on
+   * a socket.
    *)
 
 val get_conn_socket_name : connection_id -> Unix.sockaddr
 val get_conn_peer_name : connection_id -> Unix.sockaddr
   (** Return the address of the socket serving the connection, and the client
-   * socket, resp.
+   * socket, resp. These functions fail if the server is not running on
+   * a socket.
    *)
 
 val get_server : session -> t
@@ -252,23 +271,34 @@ val get_protocol : t -> protocol
 val get_srv_event_system : t -> Unixqueue.unix_event_system
   (** Returns the event system *)
 
+val get_last_proc_info : t -> string
+  (** Get a debug string describing the last invoked procedure *)
+
+val is_dummy : t -> bool
+  (** Whether this is a server in [`Dummy] mode. These servers cannot be
+      used for communication
+   *)
 
 type rule =
     [ `Deny
     | `Drop
     | `Reject
+    | `Reject_with of Rpc.server_error
     | `Accept
     | `Accept_limit_length of (int * rule)
     ]
+  (* similar to Rpc_transport.in_rule *)
 
-val set_session_filter : t -> (Unix.sockaddr -> rule) -> unit
+val set_session_filter : t -> (Rpc_transport.sockaddr -> rule) -> unit
   (** If set, the filter function is invoked every time the beginning of a new
    * RPC call is received, and the result of the filter function determines
    * what to do with the call:
    *
    * `Deny: TCP connections are immediately closed; UDP packets are dropped
    * `Drop: The call is dropped (it does not allocate memory)
-   * `Reject: A response is sent back that the call is rejected (unauthorized)
+   * `Reject_with: A response is sent back that the call is rejected. The
+   *   parameter specified the error code
+   * `Reject: The same as [`Reject_with Rpc.Auth_too_weak]
    * `Accept: The call is accepted without limitation (the default if no
    *   filter is installed)
    * `Accept_limit_length(n,r): If the call is longer than n bytes, the rule
@@ -283,7 +313,7 @@ val set_session_filter : t -> (Unix.sockaddr -> rule) -> unit
    * which accepts messages up to n bytes without limit, and denies longer
    * messages. n is the length of the longest sensible message.
    *
-   * For UDP servers, there is an implicit limit of 8000 bytes, so it is
+   * For UDP servers, there is an implicit limit of 16K, so it is
    * not necessary to care about this.
    *
    * Another application is to restrict which systems can contact this
@@ -293,11 +323,16 @@ val set_session_filter : t -> (Unix.sockaddr -> rule) -> unit
    * attacks.
    *)
 
-val set_session_filter_2 : t -> (Unix.sockaddr -> connection_id -> rule) -> unit
+val set_session_filter_2 : t -> (Rpc_transport.sockaddr -> connection_id -> rule) -> unit
   (** Same as [set_session_filter], but the filter gets as second argument the
     * connection ID.
    *)
 
+val set_mstring_factories : t -> Xdr_mstring.named_mstring_factories -> unit
+  (** Sets the mstring factories to use for decoding requests containing
+      managed strings
+   *)
+
 val reply : session -> xdr_value -> unit
   (** Asynchronous procedures can reply their results with this function.
    *
@@ -323,7 +358,7 @@ val set_exception_handler : t -> (exn -> unit) -> unit
    * exceptions and any exceptions resulting from I/O problems.
    *
    * NOTES ABOUT EXCEPTIONS:
-   * - The default exception handler just prints the exception on stderr.
+   * - The default exception handler logs a [`Crit] message using {!Netlog}.
    * - I/O problems usually lead to an 'Abort' of the whole server.
    *)
 
@@ -368,9 +403,32 @@ val stop_connection : t -> connection_id -> unit
 (************************* authentication **************************)
 
 type auth_result =
-    Auth_positive of (string * string * string)
-      (* (username, returned_verifier_flavour, returned_verifier_data) *)
+    Auth_positive of (string * string * string * 
+			Xdr.encoder option * Xdr.decoder option)
+      (** Successful authentication:
+          [(username, returned_verifier_flavour, returned_verifier_data, 
+	    enc_opt, dec_opt
+	  )]
+
+	  Encoders and decoders are allowed to raise the exceptions
+	  {!Rpc_server.Late_drop} and {!Rpc.Rpc_server}.
+       *)
   | Auth_negative of Rpc.server_error
+      (** Failed authentication *)
+  | Auth_reply of (Xdr_mstring.mstring list * string * string)
+      (** The authentication method generates the positive response
+	  of this RPC call:
+	  [(data, verf_flavor, verf_data)]
+	  (new in Ocamlnet-3.3)
+       *)
+  | Auth_drop
+      (** Authentication demands to drop the message *)
+
+
+exception Late_drop
+  (** This can be raised in encryption/decryption functions to prevent
+      that a response is sent.
+   *)
 
 type auth_peeker =
     [ `None
@@ -378,6 +436,21 @@ type auth_peeker =
     | `Peek_multiplexer of Rpc_transport.rpc_multiplex_controller -> string option
     ]
 
+class type auth_details =
+object
+  method server_addr : Unix.sockaddr option
+  method client_addr : Unix.sockaddr option
+  method program : uint4
+  method version : uint4
+  method procedure : uint4
+  method xid : uint4
+  method credential : string * string
+  method verifier : string * string
+  method frame_len : int
+  method message : Rpc_packer.packed_value
+end
+
+
 class type auth_method =
 object
   method name : string
@@ -398,20 +471,20 @@ object
   method authenticate :
            t ->
 	   connection_id ->
-	   Unix.sockaddr option ->          (* address of serving socket *)
-	   Unix.sockaddr option ->          (* address of calling socket *)
-	   string ->                        (* flavor of credentials *)
-	   string ->                        (* data of credentials *)
-	   string ->                        (* flavor of verifier *)
-	   string ->                        (* data of verifier *)
+           auth_details ->
 	   (auth_result -> unit) ->
 	     unit
-    (** This method is called when a remote call has arrived. Its task is
+    (** [authenticate srv conn_id details f]:
+     *	This method is called when a remote call has arrived. Its task is
      * to determine the client user and pass the user name (and the verifier)
      * back. If the user cannot be authenticated, the call must be rejected.
      * When the method has done the authentication, it calls the passed
-     * function with the [auth_result]. This function can be called
+     * function [f] with the [auth_result]. This function can be called
      * immediately or asynchronously.
+     *
+     * Changed in Ocamlnet-3.3: the arguments [program], [version],
+     * [procedure], and [xid] are new. Added new [auth_result] of
+     * [Auth_reply].
      *)
 end
 
@@ -443,13 +516,41 @@ val get_auth_method : session -> auth_method
   (** Returns the method that was used to authenticate the user. *)
 
 val verbose : bool -> unit
-  (** Set whether you want debug messages to stderr or not *)
-
-val debug_internals_log : (string -> unit) option ref
-val debug_service_log : (string -> unit) option ref
-  (** These are the two debug logging facilities. "Internals" is for debugging
-    * Rpc_server, "service" is for debugging how the user code interacts
-    * with Rpc_server.
-    *
-    * The [verbose] function simply sets these functions.
+  (** {b Deprecated.}
+      Set whether you want debug messages to stderr or not
    *)
+
+val detach : t -> unit
+  (** {b Internal function.} Cancels all pending I/O operations, and
+      deallocates buffers. This function has only one purpose: The
+      RPC servers inherited by a Netplex child process return memory.
+      The RPC server is unusable after this.
+   *)
+
+module Debug : sig
+  val enable : bool ref
+    (** Whether debug messages of general kind are enabled. 
+        See {!Netlog.Debug} for more information.
+     *)
+
+  val enable_ctrace : bool ref
+    (** Whether debug messages are enabled that trace connection events.
+        See {!Netlog.Debug} for more information.
+
+        The "module name" for these messages is "Rpc_server.Ctrace".
+     *)
+
+  val enable_ptrace : bool ref 
+    (** Whether the procedure trace is enabled.
+        The procedure trace outputs for every RPC call and response
+        a debug message. [ptrace_verbosity] says how verbose.
+     *)
+
+  val ptrace_verbosity : Rpc_util.verbosity ref
+    (** How verbose the ptrace is. Defaults to [`Name_abbrev_args] *)
+
+  val disable_for_server : t -> unit
+    (** Disables logging for this server *)
+
+
+end
