@@ -1,4 +1,4 @@
-(* $Id: netsmtp.ml 1018 2006-10-02 14:00:18Z gerd $
+(* $Id: netsmtp.ml 1614 2011-06-09 15:08:56Z gerd $
  * ----------------------------------------------------------------------
  *
  * This is an implementation of the Simple Mail Transfer Protocol (SMTP) 
@@ -66,7 +66,7 @@ object (self)
     oc # output_string "EHLO ";
     self # smtp_cmd (
       match host with
-        | None -> (gethostbyname (gethostname ())).h_name
+        | None -> (Uq_resolver.get_host_by_name (gethostname ())).h_name
         | Some s -> s
     );
     snd (handle_answer ic)
